@@ -4,15 +4,15 @@
  *
  * Authors:
  * Jan-Oliver Wagner <jan-oliver.wagner@greenbone.net>
- * Matthew Mundell <matt@mundell.ukfsn.org>
+ * Matthew Mundell <matthew.mundell@greenbone.net>
  *
  * Copyright:
- * Copyright (C) 2009 Greenbone Networks GmbH
+ * Copyright (C) 2009,2011 Greenbone Networks GmbH
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2,
- * or, at your option, any later version as published by the Free
- * Software Foundation
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,11 +36,17 @@
  */
 
 #include <string.h>
+#include <stdlib.h>     /* for strtod   */
+#include <math.h>       /* for HUGE_VAL */
 #include <stdio.h>
 #include <errno.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <utime.h>
 
 #include "nvti.h"
+#include "cvss.h"  /* for get_cvss_score_from_base_metrics */
+#include "../misc/openvas_logging.h"
 
 /**
  * @brief Create a new nvtpref structure filled with the given values.
@@ -81,6 +87,9 @@ nvtpref_new (gchar * name, gchar * type, gchar * dflt)
 void
 nvtpref_free (nvtpref_t * np)
 {
+  if (!np)
+    return;
+
   if (np->name)
     g_free (np->name);
   if (np->type)
@@ -101,7 +110,7 @@ nvtpref_free (nvtpref_t * np)
 gchar *
 nvtpref_name (const nvtpref_t * np)
 {
-  return (np->name);
+  return (np ? np->name : NULL);
 }
 
 /**
@@ -115,7 +124,7 @@ nvtpref_name (const nvtpref_t * np)
 gchar *
 nvtpref_type (const nvtpref_t * np)
 {
-  return (np->type);
+  return (np ? np->type : NULL);
 }
 
 /**
@@ -129,7 +138,7 @@ nvtpref_type (const nvtpref_t * np)
 gchar *
 nvtpref_default (const nvtpref_t * np)
 {
-  return (np->dflt);
+  return (np ? np->dflt : NULL);
 }
 
 /**
@@ -154,6 +163,9 @@ nvti_new (void)
 void
 nvti_free (nvti_t * n)
 {
+  if (!n)
+    return;
+
   if (n->oid)
     g_free (n->oid);
   if (n->version)
@@ -162,8 +174,6 @@ nvti_free (nvti_t * n)
     g_free (n->name);
   if (n->summary)
     g_free (n->summary);
-  if (n->description)
-    g_free (n->description);
   if (n->copyright)
     g_free (n->copyright);
   if (n->cve)
@@ -176,8 +186,6 @@ nvti_free (nvti_t * n)
     g_free (n->tag);
   if (n->cvss_base)
     g_free (n->cvss_base);
-  if (n->risk_factor)
-    g_free (n->risk_factor);
   if (n->dependencies)
     g_free (n->dependencies);
   if (n->required_keys)
@@ -190,12 +198,8 @@ nvti_free (nvti_t * n)
     g_free (n->required_ports);
   if (n->required_udp_ports)
     g_free (n->required_udp_ports);
-  if (n->sign_key_ids)
-    g_free (n->sign_key_ids);
   if (n->family)
     g_free (n->family);
-  if (n->src)
-    g_free (n->src);
   if (n->prefs)
     {
       guint len = g_slist_length (n->prefs);
@@ -218,7 +222,7 @@ nvti_free (nvti_t * n)
 gchar *
 nvti_oid (const nvti_t * n)
 {
-  return (n->oid);
+  return (n ? n->oid : NULL);
 }
 
 /**
@@ -232,7 +236,7 @@ nvti_oid (const nvti_t * n)
 gchar *
 nvti_version (const nvti_t * n)
 {
-  return (n->version);
+  return (n ? n->version : NULL);
 }
 
 /**
@@ -246,7 +250,7 @@ nvti_version (const nvti_t * n)
 gchar *
 nvti_name (const nvti_t * n)
 {
-  return (n->name);
+  return (n ? n->name : NULL);
 }
 
 /**
@@ -260,21 +264,7 @@ nvti_name (const nvti_t * n)
 gchar *
 nvti_summary (const nvti_t * n)
 {
-  return (n->summary);
-}
-
-/**
- * @brief Get the description.
- *
- * @param n The NVT Info structure of which the name should
- *          be returned.
- *
- * @return The description string. Don't free this.
- */
-gchar *
-nvti_description (const nvti_t * n)
-{
-  return (n->description);
+  return (n ? n->summary : NULL);
 }
 
 /**
@@ -288,7 +278,7 @@ nvti_description (const nvti_t * n)
 gchar *
 nvti_copyright (const nvti_t * n)
 {
-  return (n->copyright);
+  return (n ? n->copyright : NULL);
 }
 
 /**
@@ -302,7 +292,7 @@ nvti_copyright (const nvti_t * n)
 gchar *
 nvti_cve (const nvti_t * n)
 {
-  return (n->cve);
+  return (n ? n->cve : NULL);
 }
 
 /**
@@ -316,7 +306,7 @@ nvti_cve (const nvti_t * n)
 gchar *
 nvti_bid (const nvti_t * n)
 {
-  return (n->bid);
+  return (n ? n->bid : NULL);
 }
 
 /**
@@ -330,7 +320,7 @@ nvti_bid (const nvti_t * n)
 gchar *
 nvti_xref (const nvti_t * n)
 {
-  return (n->xref);
+  return (n ? n->xref : NULL);
 }
 
 /**
@@ -344,7 +334,7 @@ nvti_xref (const nvti_t * n)
 gchar *
 nvti_tag (const nvti_t * n)
 {
-  return (n->tag);
+  return (n ? n->tag : NULL);
 }
 
 /**
@@ -353,26 +343,12 @@ nvti_tag (const nvti_t * n)
  * @param n The NVT Info structure of which the CVSS base should
  *          be returned.
  *
- * @return The cvss_bas string. Don't free this.
+ * @return The cvss_base string. Don't free this.
  */
 gchar *
 nvti_cvss_base (const nvti_t * n)
 {
-  return (n->cvss_base);
-}
-
-/**
- * @brief Get the risk factor.
- *
- * @param n The NVT Info structure of which the risk factor should
- *          be returned.
- *
- * @return The risk_factors string. Don't free this.
- */
-gchar *
-nvti_risk_factor (const nvti_t * n)
-{
-  return (n->risk_factor);
+  return (n ? n->cvss_base : NULL);
 }
 
 /**
@@ -386,7 +362,7 @@ nvti_risk_factor (const nvti_t * n)
 gchar *
 nvti_dependencies (const nvti_t * n)
 {
-  return (n->dependencies);
+  return (n ? n->dependencies : NULL);
 }
 
 /**
@@ -400,7 +376,7 @@ nvti_dependencies (const nvti_t * n)
 gchar *
 nvti_required_keys (const nvti_t * n)
 {
-  return (n->required_keys);
+  return (n ? n->required_keys : NULL);
 }
 
 /**
@@ -414,7 +390,7 @@ nvti_required_keys (const nvti_t * n)
 gchar *
 nvti_mandatory_keys (const nvti_t * n)
 {
-  return (n->mandatory_keys);
+  return (n ? n->mandatory_keys : NULL);
 }
 
 /**
@@ -428,7 +404,7 @@ nvti_mandatory_keys (const nvti_t * n)
 gchar *
 nvti_excluded_keys (const nvti_t * n)
 {
-  return (n->excluded_keys);
+  return (n ? n->excluded_keys : NULL);
 }
 
 /**
@@ -442,7 +418,7 @@ nvti_excluded_keys (const nvti_t * n)
 gchar *
 nvti_required_ports (const nvti_t * n)
 {
-  return (n->required_ports);
+  return (n ? n->required_ports : NULL);
 }
 
 /**
@@ -456,21 +432,7 @@ nvti_required_ports (const nvti_t * n)
 gchar *
 nvti_required_udp_ports (const nvti_t * n)
 {
-  return (n->required_udp_ports);
-}
-
-/**
- * @brief Get the sign key ids (fingerptints) list.
- *
- * @param n The NVT Info structure of which the name should
- *          be returned.
- *
- * @return The sign key ids string. Don't free this.
- */
-gchar *
-nvti_sign_key_ids (const nvti_t * n)
-{
-  return (n->sign_key_ids);
+  return (n ? n->required_udp_ports : NULL);
 }
 
 /**
@@ -484,7 +446,7 @@ nvti_sign_key_ids (const nvti_t * n)
 gchar *
 nvti_family (const nvti_t * n)
 {
-  return (n->family);
+  return (n ? n->family : NULL);
 }
 
 /**
@@ -497,7 +459,7 @@ nvti_family (const nvti_t * n)
 guint
 nvti_pref_len (const nvti_t * n)
 {
-  return (g_slist_length (n->prefs));
+  return (n ? g_slist_length (n->prefs) : 0);
 }
 
 /**
@@ -509,26 +471,10 @@ nvti_pref_len (const nvti_t * n)
  *
  * @return The number of preferences. NULL if
  */
-nvtpref_t *
+const nvtpref_t *
 nvti_pref (const nvti_t * n, guint p)
 {
-  return (g_slist_nth_data (n->prefs, p));
-}
-
-/**
- * @brief Get the source URI of a NVT Info.
- *
- * @param n The NVT Info structure of which the source URI should
- *          be returned.
- *
- * @return The source URI string. This can be NULL if the NVT
- *         Info was created in memory. It can also be a file path.
- *         Don't free this.
- */
-gchar *
-nvti_src (const nvti_t * n)
-{
-  return (n->src);
+  return (n ? g_slist_nth_data (n->prefs, p) : NULL);
 }
 
 /**
@@ -542,7 +488,7 @@ nvti_src (const nvti_t * n)
 gint
 nvti_timeout (const nvti_t * n)
 {
-  return (n->timeout);
+  return (n ? n->timeout : -1);
 }
 
 /**
@@ -555,7 +501,7 @@ nvti_timeout (const nvti_t * n)
 gint
 nvti_category (const nvti_t * n)
 {
-  return (n->category);
+  return (n ? n->category : -1);
 }
 
 /**
@@ -570,6 +516,9 @@ nvti_category (const nvti_t * n)
 int
 nvti_set_oid (nvti_t * n, const gchar * oid)
 {
+  if (! n)
+    return (-1);
+
   if (n->oid)
     g_free (n->oid);
   n->oid = g_strdup (oid);
@@ -588,6 +537,9 @@ nvti_set_oid (nvti_t * n, const gchar * oid)
 int
 nvti_set_version (nvti_t * n, const gchar * version)
 {
+  if (! n)
+    return (-1);
+
   if (n->version)
     g_free (n->version);
   n->version = g_strdup (version);
@@ -606,6 +558,9 @@ nvti_set_version (nvti_t * n, const gchar * version)
 int
 nvti_set_name (nvti_t * n, const gchar * name)
 {
+  if (! n)
+    return (-1);
+
   if (n->name)
     g_free (n->name);
   n->name = g_strdup (name);
@@ -624,30 +579,15 @@ nvti_set_name (nvti_t * n, const gchar * name)
 int
 nvti_set_summary (nvti_t * n, const gchar * summary)
 {
+  if (! n)
+    return (-1);
+
   if (n->summary)
     g_free (n->summary);
   n->summary = g_strdup (summary);
   return (0);
 }
 
-/**
- * @brief Set the description of a NVT.
- *
- * @param n The NVT Info structure.
- *
- * @param description The description to set. A copy will be created from this.
- *
- * @return 0 for success. Anything else indicates an error.
- */
-int
-nvti_set_description (nvti_t * n, const gchar * description)
-{
-  if (n->description)
-    g_free (n->description);
-  n->description = g_strdup (description);
-  return (0);
-}
-
 /**
  * @brief Set the copyright of a NVT.
  *
@@ -660,6 +600,9 @@ nvti_set_description (nvti_t * n, const gchar * description)
 int
 nvti_set_copyright (nvti_t * n, const gchar * copyright)
 {
+  if (! n)
+    return (-1);
+
   if (n->copyright)
     g_free (n->copyright);
   n->copyright = g_strdup (copyright);
@@ -678,6 +621,9 @@ nvti_set_copyright (nvti_t * n, const gchar * copyright)
 int
 nvti_set_cve (nvti_t * n, const gchar * cve)
 {
+  if (! n)
+    return (-1);
+
   if (n->cve)
     g_free (n->cve);
   n->cve = g_strdup (cve);
@@ -696,6 +642,9 @@ nvti_set_cve (nvti_t * n, const gchar * cve)
 int
 nvti_set_bid (nvti_t * n, const gchar * bid)
 {
+  if (! n)
+    return (-1);
+
   if (n->bid)
     g_free (n->bid);
   n->bid = g_strdup (bid);
@@ -714,6 +663,9 @@ nvti_set_bid (nvti_t * n, const gchar * bid)
 int
 nvti_set_xref (nvti_t * n, const gchar * xref)
 {
+  if (! n)
+    return (-1);
+
   if (n->xref)
     g_free (n->xref);
   if (xref && xref[0])
@@ -735,6 +687,9 @@ nvti_set_xref (nvti_t * n, const gchar * xref)
 int
 nvti_set_tag (nvti_t * n, const gchar * tag)
 {
+  if (! n)
+    return (-1);
+
   if (n->tag)
     g_free (n->tag);
   if (tag && tag[0])
@@ -756,6 +711,9 @@ nvti_set_tag (nvti_t * n, const gchar * tag)
 int
 nvti_set_cvss_base (nvti_t * n, const gchar * cvss_base)
 {
+  if (! n)
+    return (-1);
+
   if (n->cvss_base)
     g_free (n->cvss_base);
   if (cvss_base && cvss_base[0])
@@ -765,27 +723,6 @@ nvti_set_cvss_base (nvti_t * n, const gchar * cvss_base)
   return (0);
 }
 
-/**
- * @brief Set the risk factor of an NVT.
- *
- * @param n The NVT Info structure.
- *
- * @param tag The risk factor to set. A copy will be created from this.
- *
- * @return 0 for success. Anything else indicates an error.
- */
-int
-nvti_set_risk_factor (nvti_t * n, const gchar * risk_factor)
-{
-  if (n->risk_factor)
-    g_free (n->risk_factor);
-  if (risk_factor && risk_factor[0])
-    n->risk_factor = g_strdup (risk_factor);
-  else
-    n->risk_factor = NULL;
-  return (0);
-}
-
 /**
  * @brief Set the dependencies of a NVT.
  *
@@ -798,6 +735,9 @@ nvti_set_risk_factor (nvti_t * n, const gchar * risk_factor)
 int
 nvti_set_dependencies (nvti_t * n, const gchar * dependencies)
 {
+  if (! n)
+    return (-1);
+
   if (n->dependencies)
     g_free (n->dependencies);
   if (dependencies && dependencies[0])
@@ -819,6 +759,9 @@ nvti_set_dependencies (nvti_t * n, const gchar * dependencies)
 int
 nvti_set_required_keys (nvti_t * n, const gchar * required_keys)
 {
+  if (! n)
+    return (-1);
+
   if (n->required_keys)
     g_free (n->required_keys);
   if (required_keys && required_keys[0])
@@ -840,6 +783,9 @@ nvti_set_required_keys (nvti_t * n, const gchar * required_keys)
 int
 nvti_set_mandatory_keys (nvti_t * n, const gchar * mandatory_keys)
 {
+  if (! n)
+    return (-1);
+
   if (n->mandatory_keys)
     g_free (n->mandatory_keys);
   if (mandatory_keys && mandatory_keys[0])
@@ -861,6 +807,9 @@ nvti_set_mandatory_keys (nvti_t * n, const gchar * mandatory_keys)
 int
 nvti_set_excluded_keys (nvti_t * n, const gchar * excluded_keys)
 {
+  if (! n)
+    return (-1);
+
   if (n->excluded_keys)
     g_free (n->excluded_keys);
   if (excluded_keys && excluded_keys[0])
@@ -882,6 +831,9 @@ nvti_set_excluded_keys (nvti_t * n, const gchar * excluded_keys)
 int
 nvti_set_required_ports (nvti_t * n, const gchar * required_ports)
 {
+  if (! n)
+    return (-1);
+
   if (n->required_ports)
     g_free (n->required_ports);
   if (required_ports && required_ports[0])
@@ -903,6 +855,9 @@ nvti_set_required_ports (nvti_t * n, const gchar * required_ports)
 int
 nvti_set_required_udp_ports (nvti_t * n, const gchar * required_udp_ports)
 {
+  if (! n)
+    return (-1);
+
   if (n->required_udp_ports)
     g_free (n->required_udp_ports);
   if (required_udp_ports && required_udp_ports[0])
@@ -912,27 +867,6 @@ nvti_set_required_udp_ports (nvti_t * n, const gchar * required_udp_ports)
   return (0);
 }
 
-/**
- * @brief Set the sign key ids of a NVT.
- *
- * @param n The NVT Info structure.
- *
- * @param sign_key_ids The sign key ids to set. A copy will be created from this.
- *
- * @return 0 for success. Anything else indicates an error.
- */
-int
-nvti_set_sign_key_ids (nvti_t * n, const gchar * sign_key_ids)
-{
-  if (n->sign_key_ids)
-    g_free (n->sign_key_ids);
-  if (sign_key_ids && sign_key_ids[0])
-    n->sign_key_ids = g_strdup (sign_key_ids);
-  else
-    n->sign_key_ids = NULL;
-  return (0);
-}
-
 /**
  * @brief Set the family of a NVT.
  *
@@ -945,30 +879,15 @@ nvti_set_sign_key_ids (nvti_t * n, const gchar * sign_key_ids)
 int
 nvti_set_family (nvti_t * n, const gchar * family)
 {
+  if (! n)
+    return (-1);
+
   if (n->family)
     g_free (n->family);
   n->family = g_strdup (family);
   return (0);
 }
 
-/**
- * @brief Set the source identifier for the acutal NVT.
- *
- * @param n The NVT Info structure.
- *
- * @param src The URI to set. A copy will be created from this.
- *
- * @return 0 for success. Anything else indicates an error.
- */
-int
-nvti_set_src (nvti_t * n, const gchar * src)
-{
-  if (n->src)
-    g_free (n->src);
-  n->src = g_strdup (src);
-  return (0);
-}
-
 /**
  * @brief Set the timout of a NVT Info.
  *
@@ -981,6 +900,9 @@ nvti_set_src (nvti_t * n, const gchar * src)
 int
 nvti_set_timeout (nvti_t * n, const gint timeout)
 {
+  if (! n)
+    return (-1);
+
   n->timeout = timeout;
   return (0);
 }
@@ -997,10 +919,223 @@ nvti_set_timeout (nvti_t * n, const gint timeout)
 int
 nvti_set_category (nvti_t * n, const gint category)
 {
+  if (! n)
+    return (-1);
+
   n->category = category;
   return (0);
 }
 
+/**
+ * @brief Add a single CVE ID of a NVT.
+ *
+ * @param n The NVT Info structure.
+ *
+ * @param cve_id The CVE ID to add. A copy will be created from this.
+ *
+ * @return 0 for success. 1 if n was NULL, 2 if cve_id was NULL.
+ */
+int
+nvti_add_cve (nvti_t * n, const gchar * cve_id)
+{
+  gchar * old;
+
+  if (! n) return (1);
+  if (! cve_id) return (2);
+
+  old = n->cve;
+
+  if (old)
+  {
+    n->cve = g_strdup_printf ("%s, %s", old, cve_id);
+    g_free (old);
+  }
+  else
+    n->cve = g_strdup (cve_id);
+
+  return (0);
+}
+
+/**
+ * @brief Add a single BID ID of a NVT.
+ *
+ * @param n The NVT Info structure.
+ *
+ * @param cve_id The BID ID to add. A copy will be created from this.
+ *
+ * @return 0 for success. 1 if n was NULL. 2 if bid_id was NULL.
+ */
+int
+nvti_add_bid (nvti_t * n, const gchar * bid_id)
+{
+  gchar * old;
+
+  if (! n) return (1);
+  if (! bid_id) return (2);
+
+  old = n->bid;
+
+  if (old)
+  {
+    n->bid = g_strdup_printf ("%s, %s", old, bid_id);
+    g_free (old);
+  }
+  else
+    n->bid = g_strdup (bid_id);
+
+  return (0);
+}
+
+/**
+ * @brief Add a required key of a NVT.
+ *
+ * @param n The NVT Info structure.
+ *
+ * @param key The required key to add. A copy will be created from this.
+ *
+ * @return 0 for success. 1 if n was NULL. 2 if key was NULL.
+ */
+int
+nvti_add_required_keys (nvti_t * n, const gchar * key)
+{
+  gchar * old;
+
+  if (! n) return (1);
+  if (! key) return (2);
+
+  old = n->required_keys;
+
+  if (old)
+  {
+    n->required_keys = g_strdup_printf ("%s, %s", old, key);
+    g_free (old);
+  }
+  else
+    n->required_keys = g_strdup (key);
+
+  return (0);
+}
+
+/**
+ * @brief Add a mandatory key of a NVT.
+ *
+ * @param n The NVT Info structure.
+ *
+ * @param key The mandatory key to add. A copy will be created from this.
+ *
+ * @return 0 for success. 1 if n was NULL. 2 if key was NULL.
+ */
+int
+nvti_add_mandatory_keys (nvti_t * n, const gchar * key)
+{
+  gchar * old;
+
+  if (! n) return (1);
+  if (! key) return (2);
+
+  old = n->mandatory_keys;
+
+  if (old)
+  {
+    n->mandatory_keys = g_strdup_printf ("%s, %s", old, key);
+    g_free (old);
+  }
+  else
+    n->mandatory_keys = g_strdup (key);
+
+  return (0);
+}
+
+/**
+ * @brief Add a excluded key of a NVT.
+ *
+ * @param n The NVT Info structure.
+ *
+ * @param key The excluded key to add. A copy will be created from this.
+ *
+ * @return 0 for success. 1 if n was NULL. 2 if key was NULL.
+ */
+int
+nvti_add_excluded_keys (nvti_t * n, const gchar * key)
+{
+  gchar * old;
+
+  if (! n) return (1);
+  if (! key) return (2);
+
+  old = n->excluded_keys;
+
+  if (old)
+  {
+    n->excluded_keys = g_strdup_printf ("%s, %s", old, key);
+    g_free (old);
+  }
+  else
+    n->excluded_keys = g_strdup (key);
+
+  return (0);
+}
+
+/**
+ * @brief Add a required port of a NVT.
+ *
+ * @param n The NVT Info structure.
+ *
+ * @param port The required port to add. A copy will be created from this.
+ *
+ * @return 0 for success. 1 if n was NULL. 2 if port was NULL.
+ */
+int
+nvti_add_required_ports (nvti_t * n, const gchar * port)
+{
+  gchar * old;
+
+  if (! n) return (1);
+  if (! port) return (2);
+
+  old = n->required_ports;
+
+  if (old)
+  {
+    n->required_ports = g_strdup_printf ("%s, %s", old, port);
+    g_free (old);
+  }
+  else
+    n->required_ports = g_strdup (port);
+
+  return (0);
+}
+
+/**
+ * @brief Add a required udp port of a NVT.
+ *
+ * @param n The NVT Info structure.
+ *
+ * @param port The required udp port to add. A copy will be created from this.
+ *
+ * @return 0 for success. 1 if n was NULL. 2 if port was NULL.
+ */
+int
+nvti_add_required_udp_ports (nvti_t * n, const gchar * port)
+{
+  gchar * old;
+
+  if (! n) return (1);
+  if (! port) return (2);
+
+  old = n->required_udp_ports;
+
+  if (old)
+  {
+    n->required_udp_ports = g_strdup_printf ("%s, %s", old, port);
+    g_free (old);
+  }
+  else
+    n->required_udp_ports = g_strdup (port);
+
+  return (0);
+}
+
 /**
  * @brief Add a preference to the NVT Info.
  *
@@ -1013,79 +1148,37 @@ nvti_set_category (nvti_t * n, const gint category)
 int
 nvti_add_pref (nvti_t * n, nvtpref_t * np)
 {
+  if (! n)
+    return (-1);
+
   n->prefs = g_slist_append (n->prefs, np);
   return (0);
 }
 
 /**
- * @brief Create a human readable text representation of a NVT Info.
- *        This is mainly for debug purposes.
+ * @brief Read NVT Info from a keyfile.
  *
- * @param n The NVT Info structure.
- *
- * @return A newly allocated string with multi-line text.
- *         The string needs to be freed with g_free().
+ * @param keyfile Keyfile.
+ * @param name    Key name.
+ * @param nvti    NVTI.
+ * @param set     Set function.
  */
-gchar *
-nvti_as_text (const nvti_t * n)
+static void
+set_from_key (GKeyFile *keyfile, const gchar *name,
+              nvti_t *nvti, int set (nvti_t * n, const gchar * oid))
 {
-  return (g_strconcat
-          ("NVT Info for OID ", (n->oid ? n->oid : "(unset)"), ":\n\n",
-           "\nVersion: ",
-           (n->version ? n->version : "(unset, probably in-memory)"),
-           "\nName: ", (n->name ? n->name : "(unset, probably in-memory)"),
-           "\nSummary: ",
-           (n->summary ? n->summary : "(unset, probably in-memory)"),
-           "\nDescription: ",
-           (n->description ? n->description : "(unset, probably in-memory)"),
-           "\nCopyright: ",
-           (n->copyright ? n->copyright : "(unset, probably in-memory)"),
-           "\nCVE: ", (n->cve ? n->cve : "(unset, probably in-memory)"),
-           "\nBID: ", (n->bid ? n->bid : "(unset, probably in-memory)"),
-           "\nXref: ", (n->xref ? n->xref : "(unset, probably in-memory)"),
-           "\nTag: ", (n->tag ? n->tag : "(unset, probably in-memory)"),
-           "\nDependencies: ",
-           (n->dependencies ? n->dependencies : "(unset, probably in-memory)"),
-           "\nRequired Keys: ",
-           (n->required_keys ? n->
-            required_keys : "(unset, probably in-memory)"),
-           "\nMandatory Keys: ",
-           (n->mandatory_keys ? n->
-            mandatory_keys : "(unset, probably in-memory)"),
-           "\nExcluded Keys: ",
-           (n->excluded_keys ? n->
-            excluded_keys : "(unset, probably in-memory)"),
-           "\nRequired Ports: ",
-           (n->required_ports ? n->
-            required_ports : "(unset, probably in-memory)"),
-           "\nRequired UDP ports: ",
-           (n->required_udp_ports ? n->
-            required_udp_ports : "(unset, probably in-memory)"),
-           "\nSignKey IDs: ",
-           (n->sign_key_ids ? n->sign_key_ids : "(unset, probably in-memory)"),
-           "\nFamily: ",
-           (n->family ? n->family : "(unset, probably in-memory)"),
-           "\nSource: ", (n->src ? n->src : "(unset, probably in-memory)"),
-//         "\nTimeout: ", (n->timeout <= 0 ? n->timeout : "(unset, probably in-memory)"),
-//         "\nCategory: ", (n->category <= 0 ? n->category : "(unset, probably in-memory)"),
-           "\n", NULL));
-}
+  gchar *utf8;
+  utf8 = g_key_file_get_string (keyfile, "NVT Info", name, NULL);
+  if (utf8)
+    {
+      gsize size;
+      gchar *iso;
 
-/**
- * @brief Create a single line representation of a NVT Info as
- *        used in "openvas_nvt_cache" files of OpenVAS-Client.
- *
- * @param n The NVT Info structure.
- *
- * @return NULL (not implemented yet).
- *
- * @todo return A newly allocated string
- *       The string needs to be freed with g_free().
- */
-gchar *
-nvti_as_openvas_nvt_cache_entry (const nvti_t * n)
-{
-  return (NULL);                // not implemented yet
+      iso = g_convert (utf8, -1, "ISO_8859-1", "UTF-8", NULL, &size, NULL);
+      set (nvti, iso);
+      g_free (iso);
+      g_free (utf8);
+    }
 }
 
 /**
@@ -1104,83 +1197,30 @@ nvti_from_keyfile (const gchar * fn)
   GError *error = NULL;
   gchar **keys;
   int i;
-  gsize size_dummy;
-  gchar *utf8str = NULL;
-  gchar *gStr;
 
   if (!g_key_file_load_from_file (keyfile, fn, G_KEY_FILE_NONE, &error))
     {
-      g_error ("%s", error->message);
+      g_warning ("%s: %s", fn, error->message);
       return NULL;
     }
 
   n = nvti_new ();
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "OID", NULL);
-  nvti_set_oid (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "Version", NULL);
-  nvti_set_version (n, gStr); g_free(gStr);
-  utf8str = g_key_file_get_string (keyfile, "NVT Info", "Name", NULL);
-  if (utf8str)
-    {
-    nvti_set_name (n,
-                   g_convert (utf8str, -1, "ISO_8859-1", "UTF-8", NULL,
-                              &size_dummy, NULL));
-    g_free(utf8str);
-    }
-  utf8str = g_key_file_get_string (keyfile, "NVT Info", "Summary", NULL);
-  if (utf8str)
-    {
-    gStr = g_convert (utf8str, -1, "ISO_8859-1", "UTF-8", NULL, &size_dummy,
-    		NULL);
-    nvti_set_summary (n, gStr);
-    g_free(gStr);
-    g_free(utf8str);
-    }
-  utf8str = g_key_file_get_string (keyfile, "NVT Info", "Description", NULL);
-  if (utf8str)
-    {
-    gStr = g_convert (utf8str, -1, "ISO_8859-1", "UTF-8", NULL,
-                                     &size_dummy, NULL);
-    nvti_set_description (n, gStr);
-    g_free(gStr);
-    g_free(utf8str);
-    }
-  utf8str = g_key_file_get_string (keyfile, "NVT Info", "Copyright", NULL);
-  if (utf8str)
-    {
-    gStr = g_convert (utf8str, -1, "ISO_8859-1", "UTF-8", NULL, &size_dummy,
-    		NULL);
-    nvti_set_copyright (n, gStr);
-    g_free(gStr);
-    g_free(utf8str);
-    }
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "CVEs", NULL);
-  nvti_set_cve (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "BIDs", NULL);
-  nvti_set_bid (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "XREFs", NULL);
-  nvti_set_xref (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "Tags", NULL);
-  nvti_set_tag (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "Dependencies", NULL);
-  nvti_set_dependencies (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "RequiredKeys", NULL);
-  nvti_set_required_keys (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "MandatoryKeys", NULL);
-  nvti_set_mandatory_keys (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "ExcludedKeys", NULL);
-  nvti_set_excluded_keys (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "RequiredPorts", NULL);
-  nvti_set_required_ports (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "RequiredUDPPorts",
-                                                      NULL);
-  nvti_set_required_udp_ports (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "SignKeyIDs", NULL);
-  nvti_set_sign_key_ids (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "Family", NULL);
-  nvti_set_family (n, gStr); g_free(gStr);
-  gStr = g_key_file_get_string (keyfile, "NVT Info", "src", NULL);
-  nvti_set_src (n, gStr); g_free(gStr);
+  set_from_key (keyfile, "OID", n, nvti_set_oid);
+  set_from_key (keyfile, "Version", n, nvti_set_version);
+  set_from_key (keyfile, "Name", n, nvti_set_name);
+  set_from_key (keyfile, "Summary", n, nvti_set_summary);
+  set_from_key (keyfile, "Copyright", n, nvti_set_copyright);
+  set_from_key (keyfile, "CVEs", n, nvti_set_cve);
+  set_from_key (keyfile, "BIDs", n, nvti_set_bid);
+  set_from_key (keyfile, "XREFs", n, nvti_set_xref);
+  set_from_key (keyfile, "Tags", n, nvti_set_tag);
+  set_from_key (keyfile, "Dependencies", n, nvti_set_dependencies);
+  set_from_key (keyfile, "RequiredKeys", n, nvti_set_required_keys);
+  set_from_key (keyfile, "MandatoryKeys", n, nvti_set_mandatory_keys);
+  set_from_key (keyfile, "ExcludedKeys", n, nvti_set_excluded_keys);
+  set_from_key (keyfile, "RequiredPorts", n, nvti_set_required_ports);
+  set_from_key (keyfile, "RequiredUDPPorts", n, nvti_set_required_udp_ports);
+  set_from_key (keyfile, "Family", n, nvti_set_family);
   nvti_set_timeout (n,
                     g_key_file_get_integer (keyfile, "NVT Info", "Timeout",
                                             NULL));
@@ -1194,14 +1234,26 @@ nvti_from_keyfile (const gchar * fn)
       for (i = 0; keys[i]; i++)
         {
           gsize len;
+          gchar *name, *type, *dflt;
           gchar **items =
             g_key_file_get_string_list (keyfile, "NVT Prefs", keys[i], &len,
                                         NULL);
           if (len != 3)
             continue;           // format error for this pref.
-          nvtpref_t *np = nvtpref_new (items[0], items[1], items[2]);
+
+          name = g_convert (items[0], -1, "ISO_8859-1", "UTF-8", NULL, &len,
+                            NULL);
+          type = g_convert (items[1], -1, "ISO_8859-1", "UTF-8", NULL, &len,
+                            NULL);
+          dflt = g_convert (items[2], -1, "ISO_8859-1", "UTF-8", NULL, &len,
+                            NULL);
+
+          nvtpref_t *np = nvtpref_new (name, type, dflt);
           nvti_add_pref (n, np);
           g_strfreev (items);
+          g_free (name);
+          g_free (type);
+          g_free (dflt);
         }
       g_strfreev (keys);
     }
@@ -1211,6 +1263,29 @@ nvti_from_keyfile (const gchar * fn)
   return (n);
 }
 
+/**
+ * @brief Read NVT Info from a keyfile.
+ *
+ * @param keyfile Keyfile.
+ * @param name    Key name.
+ * @param nvti    NVTI.
+ * @param value   Value.
+ */
+static void
+set_from_nvti (GKeyFile *keyfile, const gchar *name, const nvti_t *nvti,
+               const gchar *value)
+{
+  if (value)
+    {
+      gsize size;
+      gchar *utf8;
+
+      utf8 = g_convert (value, -1, "UTF-8", "ISO_8859-1", NULL, &size, NULL);
+      g_key_file_set_string (keyfile, "NVT Info", name, utf8);
+      g_free (utf8);
+    }
+}
+
 /**
  * @brief Store NVT Info into a keyfile.
  *
@@ -1221,80 +1296,28 @@ nvti_from_keyfile (const gchar * fn)
  * @return 0 on success. @todo Anything else indicates an error.
  */
 int
-nvti_to_keyfile (const nvti_t * n, const gchar * fn)
+nvti_to_keyfile (const nvti_t * n, const char *src, const gchar *fn)
 {
   GKeyFile *keyfile = g_key_file_new ();
   gchar *text;
   GError *error = NULL;
 
-  if (n->oid)
-    g_key_file_set_string (keyfile, "NVT Info", "OID", n->oid);
-  if (n->version)
-    g_key_file_set_string (keyfile, "NVT Info", "Version", n->version);
-  if (n->name)
-    {
-      gsize size_dummy;
-      gchar *utf8str = g_convert (n->name, -1, "UTF-8", "ISO_8859-1",
-                                  NULL, &size_dummy, NULL);
-      g_key_file_set_string (keyfile, "NVT Info", "Name", utf8str);
-      g_free (utf8str);
-    }
-  if (n->summary)
-    {
-      gsize size_dummy;
-      gchar *utf8str = g_convert (n->summary, -1, "UTF-8", "ISO_8859-1",
-                                  NULL, &size_dummy, NULL);
-      g_key_file_set_string (keyfile, "NVT Info", "Summary", utf8str);
-      g_free (utf8str);
-    }
-  if (n->description)
-    {
-      gsize size_dummy;
-      gchar *utf8str = g_convert (n->description, -1, "UTF-8", "ISO_8859-1",
-                                  NULL, &size_dummy, NULL);
-      g_key_file_set_string (keyfile, "NVT Info", "Description", utf8str);
-      g_free (utf8str);
-    }
-  if (n->copyright)
-    {
-      gsize size_dummy;
-      gchar *utf8str = g_convert (n->copyright, -1, "UTF-8", "ISO_8859-1",
-                                  NULL, &size_dummy, NULL);
-      g_key_file_set_string (keyfile, "NVT Info", "Copyright", utf8str);
-      g_free (utf8str);
-    }
-  if (n->cve)
-    g_key_file_set_string (keyfile, "NVT Info", "CVEs", n->cve);
-  if (n->bid)
-    g_key_file_set_string (keyfile, "NVT Info", "BIDs", n->bid);
-  if (n->xref)
-    g_key_file_set_string (keyfile, "NVT Info", "XREFs", n->xref);
-  if (n->tag)
-    g_key_file_set_string (keyfile, "NVT Info", "Tags", n->tag);
-  if (n->dependencies)
-    g_key_file_set_string (keyfile, "NVT Info", "Dependencies",
-                           n->dependencies);
-  if (n->required_keys)
-    g_key_file_set_string (keyfile, "NVT Info", "RequiredKeys",
-                           n->required_keys);
-  if (n->mandatory_keys)
-    g_key_file_set_string (keyfile, "NVT Info", "MandatoryKeys",
-                           n->mandatory_keys);
-  if (n->excluded_keys)
-    g_key_file_set_string (keyfile, "NVT Info", "ExcludedKeys",
-                           n->excluded_keys);
-  if (n->required_ports)
-    g_key_file_set_string (keyfile, "NVT Info", "RequiredPorts",
-                           n->required_ports);
-  if (n->required_udp_ports)
-    g_key_file_set_string (keyfile, "NVT Info", "RequiredUDPPorts",
-                           n->required_udp_ports);
-  if (n->sign_key_ids)
-    g_key_file_set_string (keyfile, "NVT Info", "SignKeyIDs", n->sign_key_ids);
-  if (n->family)
-    g_key_file_set_string (keyfile, "NVT Info", "Family", n->family);
-  if (n->src)
-    g_key_file_set_string (keyfile, "NVT Info", "src", n->src);
+  set_from_nvti (keyfile, "OID", n, n->oid);
+  set_from_nvti (keyfile, "Version", n, n->version);
+  set_from_nvti (keyfile, "Name", n, n->name);
+  set_from_nvti (keyfile, "Summary", n, n->summary);
+  set_from_nvti (keyfile, "Copyright", n, n->copyright);
+  set_from_nvti (keyfile, "CVEs", n, n->cve);
+  set_from_nvti (keyfile, "BIDs", n, n->bid);
+  set_from_nvti (keyfile, "XREFs", n, n->xref);
+  set_from_nvti (keyfile, "Tags", n, n->tag);
+  set_from_nvti (keyfile, "Dependencies", n, n->dependencies);
+  set_from_nvti (keyfile, "RequiredKeys", n, n->required_keys);
+  set_from_nvti (keyfile, "MandatoryKeys", n, n->mandatory_keys);
+  set_from_nvti (keyfile, "ExcludedKeys", n, n->excluded_keys);
+  set_from_nvti (keyfile, "RequiredPorts", n, n->required_ports);
+  set_from_nvti (keyfile, "RequiredUDPPorts", n, n->required_udp_ports);
+  set_from_nvti (keyfile, "Family", n, n->family);
   if (n->timeout > 0)
     g_key_file_set_integer (keyfile, "NVT Info", "Timeout", n->timeout);
   if (n->category > 0)
@@ -1303,24 +1326,33 @@ nvti_to_keyfile (const nvti_t * n, const gchar * fn)
   int i;
   for (i = 0; i < nvti_pref_len (n); i++)
     {
-      nvtpref_t *np = nvti_pref (n, i);
+      const nvtpref_t *np = nvti_pref (n, i);
       gchar *lst[3];
       gchar buf[10];
-      lst[0] = ((nvtpref_t *) np)->name;
-      lst[1] = ((nvtpref_t *) np)->type;
-      lst[2] = ((nvtpref_t *) np)->dflt;
+      gsize size;
+
+      lst[0] = g_convert (((nvtpref_t *) np)->name, -1, "UTF-8", "ISO_8859-1",
+                          NULL, &size, NULL);
+      lst[1] = g_convert (((nvtpref_t *) np)->type, -1, "UTF-8", "ISO_8859-1",
+                          NULL, &size, NULL);
+      lst[2] = g_convert (((nvtpref_t *) np)->dflt, -1, "UTF-8", "ISO_8859-1",
+                          NULL, &size, NULL);
 
       g_snprintf (buf, 10, "P%d", i);
       g_key_file_set_string_list ((GKeyFile *) keyfile, "NVT Prefs", buf,
                                   (const gchar **) lst, 3);
 //    g_key_file_set_string_list((GKeyFile *)keyfile, "NVT Prefs", (gchar *)lst[0], (const gchar **)lst, 3);
+
+      g_free (lst[0]);
+      g_free (lst[1]);
+      g_free (lst[2]);
     }
 
   text = g_key_file_to_data (keyfile, NULL, &error);
   if (error != NULL)
     {
-      fprintf (stderr, "Error occured while preparing %s: %s\n", fn,
-               error->message);
+      log_legacy_write ("Error occured while preparing %s: %s\n", fn,
+                        error->message);
       g_error_free (error);
     }
   else
@@ -1331,17 +1363,20 @@ nvti_to_keyfile (const nvti_t * n, const gchar * fn)
           gchar *cache_dir = g_path_get_dirname (fn);
           if ((g_mkdir_with_parents (cache_dir, 0755) < 0) && (errno != EEXIST))
             {
-              fprintf (stderr, "mkdir(%s) : %s\n", cache_dir, strerror (errno));
+              log_legacy_write ("mkdir(%s) : %s\n", cache_dir,
+                                strerror (errno));
+              g_free (cache_dir);
               g_free (text);
               g_key_file_free (keyfile);
               return (1);
             }
+          g_free (cache_dir);
           fp = fopen (fn, "w");
         }
 
       if (!fp)
         {                       // again failed
-          fprintf (stderr, "fopen(%s) : %s\n", fn, strerror (errno));
+          log_legacy_write ("fopen(%s) : %s\n", fn, strerror (errno));
           g_free (text);
           g_key_file_free (keyfile);
           return (2);
@@ -1349,6 +1384,24 @@ nvti_to_keyfile (const nvti_t * n, const gchar * fn)
 
       fputs (text, fp);
       fclose (fp);
+
+      /* Set timestamp of cache file to the timestamp of the original NVT, if
+       * possible */
+      if (src)
+        {
+          struct stat src_stat;
+          if (stat (src, &src_stat) == 0)
+            {
+              struct utimbuf src_timestamp;
+              src_timestamp.actime = src_stat.st_atime;
+              src_timestamp.modtime = src_stat.st_mtime;
+              if (utime (fn, &src_timestamp) != 0)
+                log_legacy_write ("utime(%s) : %s\n", fn, strerror (errno));
+            }
+          else
+            log_legacy_write ("stat(%s) : %s\n", src, strerror (errno));
+        }
+
       g_free (text);
     }
 
@@ -1357,7 +1410,6 @@ nvti_to_keyfile (const nvti_t * n, const gchar * fn)
   return (0);
 }
 
-
 /* Collections of nvtis. */
 
 /**
@@ -1375,7 +1427,7 @@ free_nvti_for_hash_table (gpointer nvti)
  * @brief Make a collection of NVT Infos.
  */
 nvtis_t *
-nvtis_new ()
+nvtis_new (void)
 {
   return g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
                                 free_nvti_for_hash_table);
@@ -1393,17 +1445,6 @@ nvtis_free (nvtis_t * nvtis)
     g_hash_table_destroy (nvtis);
 }
 
-/**
- * @brief Get the size of a collection of NVT Infos.
- *
- * @return The number of entries in the collection.
- */
-guint
-nvtis_size (nvtis_t * nvtis)
-{
-  return g_hash_table_size (nvtis);
-}
-
 /**
  * @brief Add an NVT Info to a collection of NVT Infos.
  *
