@@ -39,12 +39,12 @@
 
 #include "arglists.h"
 #include "bpf_share.h"
-#include "resolve.h"
 #include "ids_send.h"
 #include "network.h"
 #include "pcap_openvas.h"
 #include "plugutils.h"
-#include "system.h"
+#include "openvas_logging.h"
+#include "support.h"
 
 /** @todo: It still needs to be taken care
  * BSD_BYTE_ORDERING gets here if defined (e.g. by config.h) */
@@ -147,8 +147,8 @@ union sockaddr_u {
  * This function returns the TTL we should use when forging our own
  * IP packets. If <method & OPENVAS_CNX_IDS_EVASION_SHORT_TTL>, then
  * it returns an estimation of the number of hops between us and the
- * remote host (see comment), minus 1. 
- * 
+ * remote host (see comment), minus 1.
+ *
  * By default, it returns the TTL our OS usually returns (to be improved)
  *
  */
@@ -165,7 +165,7 @@ which_ttl (method, old_ttl)
        * To find out the number of hops to the remote host,
        * we assume that the TTL set remotely is one of {32,64,128,255}.
        * (by default, I know of no OS which uses a different value)
-       * 
+       *
        * We could fine grain that by reading Host/OS or even by
        * computing the traceroute by ourselves (especially since we're
        * not sure that our packets will go through the same path as those
@@ -265,7 +265,7 @@ tcp_cksum (packet, len)
 {
   struct pseudohdr pseudoheader;
   char *tcpsumdata =
-    (char *) emalloc (sizeof (struct pseudohdr) + (len % 2 ? len + 1 : 0));
+    (char *) g_malloc0 (sizeof (struct pseudohdr) + (len % 2 ? len + 1 : 0));
   struct in_addr source, dest;
   struct ip_packet *ip = (struct ip_packet *) packet;
   struct tcp_packet *tcp = (struct tcp_packet *) (packet + ip->ip_hl * 4);
@@ -288,18 +288,18 @@ tcp_cksum (packet, len)
   tcp->th_sum =
     in_cksum ((unsigned short *) tcpsumdata,
               12 + sizeof (struct tcp_packet) + len);
-  efree (&tcpsumdata);
+  g_free (tcpsumdata);
   return 0;
 }
 
 
 /*
  * This function injects a tcp packet in a stream. If
- * method & OPENVAS_CNX_IDS_EVASION_SHORT_TTL != 0, then 
+ * method & OPENVAS_CNX_IDS_EVASION_SHORT_TTL != 0, then
  * the injected tcp packet will be completely valid but will
  * have a short TTL (so that it does not reach the remote host).
  *
- * If method & OPENVAS_CNX_IDS_EVASION_INJECT != 0, then 
+ * If method & OPENVAS_CNX_IDS_EVASION_INJECT != 0, then
  * the injected tcp packet will have a normal TTL but will
  * have a bad checksum.
  *
@@ -319,7 +319,7 @@ tcp_cksum (packet, len)
  *
  * <data> is the data we should append to our tcp packet. This can
  * be NULL
- * <data_len> is the length of the data. 
+ * <data_len> is the length of the data.
  *
  *
  * Note that this function opens a raw socket each time it's called, which
@@ -361,12 +361,8 @@ inject (orig_packet, packet_len, method, flags, data, data_len)
   if (soc < 0)
     return -1;
 
-#ifdef IP_HDRINCL
   setsockopt (soc, IPPROTO_IP, IP_HDRINCL, &one, sizeof (one));
-#endif
-
-
-  packet = emalloc (tot_len);
+  packet = g_malloc0 (tot_len);
 
   /*
    * Copy data, if any
@@ -377,7 +373,7 @@ inject (orig_packet, packet_len, method, flags, data, data_len)
   ip = (struct ip_packet *) packet;
   tcp = (struct tcp_packet *) (packet + sizeof (struct ip_packet));
 
-  /* 
+  /*
    * for the sake of code shortness, we copy the header of the
    * received packet into the packet we forge, and we'll change
    * some stuff in it.
@@ -426,7 +422,7 @@ inject (orig_packet, packet_len, method, flags, data, data_len)
       perror
         ("openvas-libraries : libopenvas : ids_send.c : inject() : sendto() ");
     }
-  efree (&packet);
+  g_free (packet);
   close (soc);
   return 0;
 }
@@ -460,7 +456,7 @@ injectv6 (orig_packet, packet_len, method, flags, data, data_len)
   if (soc < 0)
     return -1;
 
-  packet = emalloc (tot_len);
+  packet = g_malloc0 (tot_len);
 
   /*
    * Copy data, if any
@@ -520,7 +516,7 @@ injectv6 (orig_packet, packet_len, method, flags, data, data_len)
       perror
         ("openvas-libraries : libopenvas : ids_send.c : inject() : sendto() ");
     }
-  efree (&packet);
+  g_free (packet);
   close (soc);
   return 0;
 }
@@ -553,10 +549,10 @@ ids_send (fd, buf0, n, method)
   char hostname[INET6_ADDRSTRLEN];
   int family;
 
-  bzero (&(su->sockaddr), sizeof (su->sockaddr));
   if (getpeername (fd, &(su->sockaddr), &sz) < 0)
     {
       perror ("getpeername() ");
+      return -1;
     }
   sa = &(su->sockaddr);
   if (sa->sa_family == AF_INET)
@@ -568,14 +564,14 @@ ids_send (fd, buf0, n, method)
       src.s_addr = 0;
       iface = routethrough (&dst, &src);
 
-      src_host = estrdup (inet_ntoa (src));
-      dst_host = estrdup (inet_ntoa (dst));
+      src_host = g_strdup (inet_ntoa (src));
+      dst_host = g_strdup (inet_ntoa (dst));
 
       snprintf (filter, sizeof (filter),
                 "tcp and (src host %s and dst host %s and src port %d)",
                 dst_host, src_host, port);
-      efree (&src_host);
-      efree (&dst_host);
+      g_free (src_host);
+      g_free (dst_host);
     }
   else
     {
@@ -587,14 +583,14 @@ ids_send (fd, buf0, n, method)
       iface = v6_routethrough (&dst6, &src6);
 
       src_host =
-        estrdup (inet_ntop (AF_INET6, &src6, hostname, sizeof (hostname)));
+        g_strdup (inet_ntop (AF_INET6, &src6, hostname, sizeof (hostname)));
       dst_host =
-        estrdup (inet_ntop (AF_INET6, &dst6, hostname, sizeof (hostname)));
+        g_strdup (inet_ntop (AF_INET6, &dst6, hostname, sizeof (hostname)));
       snprintf (filter, sizeof (filter),
                 "tcp and (src host %s and dst host %s and src port %d)",
                 dst_host, src_host, port);
-      efree (&src_host);
-      efree (&dst_host);
+      g_free (src_host);
+      g_free (dst_host);
     }
 
   bpf = bpf_open_live (iface, filter);
@@ -711,7 +707,7 @@ ids_open_sock_tcp (args, port, method, timeout)
   dst6 = plug_get_host_ip (args);
   if (!dst6)
     {
-      fprintf (stderr, "Error - no address associated with name\n");
+      log_legacy_write ("Error - no address associated with name\n");
       return -1;
     }
   if (IN6_IS_ADDR_V4MAPPED (dst6))
@@ -720,25 +716,25 @@ ids_open_sock_tcp (args, port, method, timeout)
       dst.s_addr = dst6->s6_addr32[3];
       src.s_addr = 0;
       iface = routethrough (&dst, &src);
-      src_host = estrdup (inet_ntoa (src));
-      dst_host = estrdup (inet_ntoa (dst));
+      src_host = g_strdup (inet_ntoa (src));
+      dst_host = g_strdup (inet_ntoa (dst));
     }
   else
     {
       family = AF_INET6;
       iface = v6_routethrough (dst6, &src6);
       src_host =
-        estrdup (inet_ntop (AF_INET6, &src6, hostname, sizeof (hostname)));
+        g_strdup (inet_ntop (AF_INET6, &src6, hostname, sizeof (hostname)));
       dst_host =
-        estrdup (inet_ntop (AF_INET6, dst6, hostname, sizeof (hostname)));
+        g_strdup (inet_ntop (AF_INET6, dst6, hostname, sizeof (hostname)));
     }
 
   snprintf (filter, sizeof (filter),
             "tcp and (src host %s and dst host %s and src port %d)", dst_host,
             src_host, port);
 
-  efree (&src_host);
-  efree (&dst_host);
+  g_free (src_host);
+  g_free (dst_host);
 
 
   bpf = bpf_open_live (iface, filter);
