@@ -48,6 +48,7 @@
 #include "kb.h"                 /* for kb_item_get_str() */
 
 #include "ids_send.h"
+#include "prefs.h"
 #include "plugutils.h"
 #include "internal_com.h" /* for INTERNAL_COMM_MSG_TYPE_CTRL */
 #include "support.h"
@@ -201,7 +202,7 @@ get_connection_fd (void)
 
 
 static int
-release_connection_fd (int fd)
+release_connection_fd (int fd, int already_closed)
 {
   openvas_connection *p;
 
@@ -239,7 +240,7 @@ release_connection_fd (int fd)
           pid_perror ("release_connection_fd: shutdown()");
 #endif
         }
-      if (socket_close (p->fd) < 0)
+      if (!already_closed && socket_close (p->fd) < 0)
         pid_perror ("release_connection_fd: close()");
     }
 
@@ -742,6 +743,10 @@ set_ids_evasion_mode (struct arglist *args, openvas_connection * fp)
                          sizeof (n));
       fp->options |= option;
     }
+  g_free (ids_evasion_split);
+  g_free (ids_evasion_inject);
+  g_free (ids_evasion_short_ttl);
+  g_free (ids_evasion_fake_rst);
 }
 
 /*
@@ -775,7 +780,7 @@ socket_negotiate_ssl (int fd, openvas_encaps_t transport, struct arglist *args)
   if (open_SSL_connection (fp, cert, key, passwd, cafile) <= 0)
     {
       log_legacy_write ("socket_negotiate_ssl: SSL connection failed.\n");
-      release_connection_fd (fd);
+      release_connection_fd (fd, 0);
       return -1;
     }
   return fd;
@@ -1053,6 +1058,7 @@ open_stream_connection_ext (struct arglist *args, unsigned int port,
 
   switch (transport)
     {
+    int ret;
     case OPENVAS_ENCAPS_IP:
       break;
     case OPENVAS_ENCAPS_SSLv23:
@@ -1072,7 +1078,12 @@ open_stream_connection_ext (struct arglist *args, unsigned int port,
 
     case OPENVAS_ENCAPS_SSLv2:
       /* We do not need a client certificate in this case */
-      if (open_SSL_connection (fp, cert, key, passwd, cafile) <= 0)
+      ret = open_SSL_connection (fp, cert, key, passwd, cafile);
+      g_free (cert);
+      g_free (key);
+      g_free (passwd);
+      g_free (cafile);
+      if (ret <= 0)
         goto failed;
       break;
     }
@@ -1080,7 +1091,7 @@ open_stream_connection_ext (struct arglist *args, unsigned int port,
   return fd;
 
 failed:
-  release_connection_fd (fd);
+  release_connection_fd (fd, 0);
   return -1;
 }
 
@@ -1139,8 +1150,11 @@ struct ovas_scanner_context_s
 
   /** GnuTLS priority string */
   char *priority;
+
+  gnutls_session_t tls_session;
 };
 
+
 /**
  * @brief Creates a new ovas_scanner_context_t.
  *
@@ -1157,7 +1171,7 @@ ovas_scanner_context_new (openvas_encaps_t encaps, const char *certfile,
 {
   ovas_scanner_context_t ctx = NULL;
 
-  ctx = g_malloc0 (sizeof (ovas_scanner_context_t));
+  ctx = g_malloc0 (sizeof (*ctx));
   ctx->encaps = encaps;
   ctx->priority = g_strdup (priority);
 
@@ -1254,6 +1268,7 @@ ovas_scanner_context_attach (ovas_scanner_context_t ctx, int soc)
           tlserror ("gnutls_init", ret);
           goto fail;
         }
+      ctx->tls_session = fp->tls_session;
       my_gnutls_transport_set_lowat_default (fp->tls_session);
 
       ret = set_gnutls_protocol (fp->tls_session, fp->transport, ctx->priority);
@@ -1305,7 +1320,7 @@ ovas_scanner_context_attach (ovas_scanner_context_t ctx, int soc)
   return fd;
 
 fail:
-  release_connection_fd (fd);
+  release_connection_fd (fd, 0);
   return -1;
 }
 
@@ -1463,12 +1478,10 @@ read_stream_connection_unbuffered (int fd, void *buf0, int min_len, int max_len)
               if (ret > 0)
                 {
                   total += ret;
+                  if (total >= max_len)
+                    return total;
                 }
-
-              if (total >= max_len)
-                return total;
-
-              if (ret != GNUTLS_E_INTERRUPTED && ret != GNUTLS_E_AGAIN)
+              else if (ret != GNUTLS_E_INTERRUPTED && ret != GNUTLS_E_AGAIN)
                 {
                   /* This branch also handles the case where ret == 0,
                    * i.e. that the connection has been closed.  This is
@@ -1492,15 +1505,7 @@ read_stream_connection_unbuffered (int fd, void *buf0, int min_len, int max_len)
                 }
             }
 
-          if (min_len <= 0)
-            {
-              /* Be smart */
-              if (total > 0 && flag)
-                return total;
-              else
-                flag++;
-            }
-          else if (total >= min_len)
+          if (min_len > 0 && total >= min_len)
             return total;
         }
       if (t >= timeout)
@@ -1874,11 +1879,7 @@ close_stream_connection (int fd)
   log_legacy_write ("close_stream_connection TCP:%d (fd=%d)\n", fp->port, fd);
 #endif
 
-  if (0)
-    ;
-  else if (!run_csc_hooks (fd))
-    return 0; /* A hook already closed the fd.  */
-  else if (!OPENVAS_STREAM (fd))     /* Will never happen if debug is on! */
+  if (!OPENVAS_STREAM (fd))     /* Will never happen if debug is on! */
     {
       if (fd < 0 || fd > 1024)
         {
@@ -1888,8 +1889,10 @@ close_stream_connection (int fd)
       shutdown (fd, 2);
       return socket_close (fd);
     }
+  if (!run_csc_hooks (fd))
+    return release_connection_fd (fd, 1);
   else
-    return release_connection_fd (fd);
+    return release_connection_fd (fd, 0);
 }
 
 const char *
@@ -2084,15 +2087,38 @@ open_sock_opt_hn (const char *hostname, unsigned int port, int type,
 int
 open_sock_tcp (struct arglist *args, unsigned int port, int timeout)
 {
-  int ret;
+  int ret, retry = 0;
+  const char *timeout_retry;
 
-  errno = 0;
-  ret = open_sock_option (args, port, SOCK_STREAM, IPPROTO_TCP, timeout);
+  timeout_retry = prefs_get ("timeout_retry");
+  if (timeout_retry)
+    retry = atoi (timeout_retry);
+  if (retry < 0)
+    retry = 0;
+
+  while (retry >= 0)
+    {
+      errno = 0;
+      ret = open_sock_option (args, port, SOCK_STREAM, IPPROTO_TCP, timeout);
+      if (ret >= 0 || errno != ETIMEDOUT)
+        break;
+      retry--;
+    }
   if (ret < 0 && errno == ETIMEDOUT)
     {
-      char *ip_str = plug_get_host_ip_str (args);
+      int log_count;
+      char *ip_str = plug_get_host_ip_str (args), buffer[1024];
+      kb_t kb = plug_get_kb (args);
 
-      log_legacy_write ("open_sock_tcp: %s:%d time-out.\n", ip_str, port);
+      g_snprintf (buffer, sizeof (buffer), "ConnectTimeout/%s/%d", ip_str,
+                  port);
+      log_count = kb_item_get_int (kb, buffer);
+      if (log_count == -1)
+        log_count = 0;
+      if (log_count < 3)
+        log_legacy_write ("open_sock_tcp: %s:%d time-out.", ip_str, port);
+      log_count++;
+      kb_item_set_int (kb, buffer, log_count);
       g_free (ip_str);
     }
 
@@ -2217,6 +2243,8 @@ recv_line (int soc, char *buf, size_t bufsiz)
           tv.tv_sec = 5;
           tv.tv_usec = 0;
           e = select (soc + 1, &rd, NULL, NULL, &tv);
+          if (e == 0 && !FD_ISSET (soc, &rd))
+            return -1;
           if (e < 0 && errno == EINTR)
             goto again;
           if (e > 0)
