@@ -31,9 +31,33 @@
 
 #include <glib.h>
 
-void openvas_auth_init ();
+#include "../base/array.h"
 
-void openvas_auth_tear_down ();
+/**
+ * @brief Numerical representation of the supported authentication methods.
+ * @brief Beware to have it in sync with \ref authentication_methods.
+ */
+enum authentication_method
+{
+  AUTHENTICATION_METHOD_FILE = 0,
+  AUTHENTICATION_METHOD_LDAP_CONNECT,
+  AUTHENTICATION_METHOD_RADIUS_CONNECT,
+  AUTHENTICATION_METHOD_LAST
+};
+
+/** @brief Type for the numerical representation of the supported
+ *  @brief authentication methods. */
+typedef enum authentication_method auth_method_t;
+
+const gchar *auth_method_name (auth_method_t);
+
+int openvas_auth_init_funcs (gchar * (*) (const gchar *),
+                             int (*) (const gchar *, const gchar *,
+                                      const gchar *),
+                             int (*) (const gchar *, auth_method_t),
+                             gchar * (*) (const gchar *, auth_method_t));
+
+void openvas_auth_tear_down (void);
 
 int openvas_auth_write_config (GKeyFile * keyfile);
 
@@ -41,25 +65,14 @@ gchar *get_password_hashes (int, const gchar *);
 
 gchar *digest_hex (int, const guchar *);
 
-int openvas_authenticate (const gchar *, const gchar *);
+int openvas_authenticate_method (const gchar *, const gchar *, auth_method_t *);
 
 int openvas_authenticate_uuid (const gchar *, const gchar *, gchar ** uuid);
 
+int openvas_user_exists (const char *);
+
 gchar *openvas_user_uuid (const char *name);
 
-int openvas_is_user_admin (const gchar *);
-
-int openvas_set_user_role (const gchar *, const gchar *,
-                           const gchar * user_dir_name);
-
-int openvas_auth_mkrulesdir (const gchar * user_dir_name);
-
-int openvas_auth_user_rules (const gchar * username, gchar ** rules);
-
-int openvas_auth_user_uuid_rules (const gchar * username,
-                                  const gchar * user_uuid, gchar ** rules);
-
-int openvas_auth_store_user_rules (const gchar * user_dir, const gchar * hosts,
-                                   int hosts_allow);
-
+int
+openvas_auth_radius_enabled ();
 #endif /* not _OPENVAS_AUTH_H */
