@@ -45,8 +45,10 @@
 #include <syslog.h>             /* for syslog */
 #undef SYSLOG_NAMES
 #include <unistd.h>             /* for getpid */
+#include <stdarg.h>
 #include <libgen.h>
 #include <errno.h>
+#include <gnutls/gnutls.h>
 
 #include "openvas_logging.h"
 
@@ -69,6 +71,16 @@ typedef struct
 } openvas_logging_t;
 
 
+/**
+ * @brief Handler for log output from legacy functions
+ *
+ * If set by @ref setup_legacy_log_handler, @ref log_legacy_write uses
+ * this function to output log messages.  The legacy log facility
+ * should eventually be removed.
+ */
+static void (*legacy_log_handler)(const char *format, va_list args);
+
+
 /**
  * @brief Returns time as specified in time_fmt strftime format.
  *
@@ -305,6 +317,7 @@ free_log_configuration (GSList * log_domain_list)
       g_free (log_domain_entry->prepend_time_format);
       g_free (log_domain_entry->log_file);
       g_free (log_domain_entry->default_level);
+      g_free (log_domain_entry->syslog_ident);
 
       /* Drop the reference to the GIOChannel. */
       if (log_domain_entry->log_channel)
@@ -336,6 +349,35 @@ openvas_log_silent (const char *log_domain, GLogLevelFlags log_level,
   return;
 }
 
+static GMutex *logger_mutex = NULL;
+
+static void
+openvas_log_lock_init (void)
+{
+#if GLIB_CHECK_VERSION (2, 31, 0)
+  if (logger_mutex == NULL)
+    {
+      logger_mutex = g_malloc (sizeof (*logger_mutex));
+      g_mutex_init (logger_mutex);
+    }
+#else
+  if (logger_mutex == NULL)
+    logger_mutex = g_mutex_new ();
+#endif
+}
+
+static void
+openvas_log_lock (void)
+{
+  g_mutex_lock (logger_mutex);
+}
+
+static void
+openvas_log_unlock (void)
+{
+  g_mutex_unlock (logger_mutex);
+}
+
 /**
  * @brief Creates the formatted string and outputs it to the log destination.
  *
@@ -353,6 +395,8 @@ openvas_log_func (const char *log_domain, GLogLevelFlags log_level,
   gchar *prepend_tmp;
   gchar *prepend_tmp1;
   gchar *tmp;
+  gchar *tmpstr;
+  int messagelen;
 
   /* For link list operations. */
   GSList *log_domain_list_tmp;
@@ -376,6 +420,9 @@ openvas_log_func (const char *log_domain, GLogLevelFlags log_level,
   gchar *syslog_facility = "local0";
   gchar *syslog_ident = NULL;
 
+  /* Initialize logger lock if not done. */
+  openvas_log_lock_init ();
+
   /* Let's load the default configuration file directives from the
    * linked list. Scanning the link list twice is inefficient but
    * leaves the source cleaner.
@@ -558,15 +605,20 @@ openvas_log_func (const char *log_domain, GLogLevelFlags log_level,
       break;
     }
 
-  /* If the current log entry is more severe than the specified log level,
-   * print out the message.
+  /* If the current log entry is more severe than the specified log
+   * level, print out the message.  In case MESSAGE already ends in a
+   * LF and there is not only the LF, remove the LF to avoid empty
+   * lines in the log.
    */
-  GString *log_str = g_string_new ("");
-  g_string_append_printf (log_str, "%s%s%s%s %s\n",
-                          log_domain ? log_domain : "", log_separator, prepend,
-                          log_separator, message);
+  messagelen = message? strlen (message) : 0;
+  if (messagelen > 1 && message[messagelen-1] == '\n')
+    messagelen--;
+  tmpstr = g_strdup_printf ("%s%s%s%s %.*s\n",
+                            log_domain ? log_domain : "", log_separator,
+                            prepend, log_separator, messagelen, message);
+  g_free (prepend);
 
-  gchar *tmpstr = g_string_free (log_str, FALSE);
+  openvas_log_lock ();
   /* Output everything to stderr if logfile is "-". */
   if (g_ascii_strcasecmp (log_file, "-") == 0)
     {
@@ -660,10 +712,26 @@ openvas_log_func (const char *log_domain, GLogLevelFlags log_level,
       g_io_channel_flush (channel, NULL);
 
     }
+  openvas_log_unlock ();
   g_free (tmpstr);
   g_free (prepend_buf);
 }
 
+
+/**
+ * @brief This function logs debug messages from gnutls.
+ *
+ * To enable GNUTLS debug messages, the environment variable @ref @c
+ * OPENVAS_GNUTLS_DEBUG is to be set to the desired log level as
+ * described in the GNUTLS manual.
+ */
+static void
+log_func_for_gnutls (int level, const char *message)
+{
+  g_log ("x  gnutls", G_LOG_LEVEL_INFO, "tls(%d): %s", level, message);
+}
+
+
 /**
  * @brief Sets up routing of logdomains to log handlers.
  *
@@ -722,4 +790,67 @@ setup_log_handlers (GSList * openvas_log_config_list)
                                        G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL |
                                        G_LOG_FLAG_RECURSION),
                      (GLogFunc) openvas_log_func, openvas_log_config_list);
+
+  /* Check whether GNUTLS debugging has been enabled.  */
+  {
+    const char *s;
+    if ((s=getenv ("OPENVAS_GNUTLS_DEBUG")))
+      {
+        gnutls_global_set_log_function (log_func_for_gnutls);
+        gnutls_global_set_log_level (atoi (s));
+      }
+  }
+}
+
+
+/**
+ * @brief Sets up a simple logging function
+ *
+ * The openvas-scanner has not yet been changed to use the new logging
+ * facility.  However, it uses libary functions and those should use a
+ * proper log function instead of writing to stderr.  This function
+ * can be used to register an existing log handler which will then be
+ * used by @ref log_legacy_write.
+ *
+ * @param handler  A printf style log handler or NULL to use stderr.
+ */
+void
+setup_legacy_log_handler (void (*handler)(const char *, va_list))
+{
+  legacy_log_handler = handler;
+}
+
+
+/**
+ * @brief Legacy function to write a log message
+ *
+ * This function shall be used instead of fprintf in legacy code.  It
+ * will eventually be replaced by glib log functions.
+ *
+ * @param format   printf style format string
+ */
+void
+log_legacy_write (const char *format, ...)
+{
+  va_list arg_ptr;
+
+  va_start (arg_ptr, format);
+  if (legacy_log_handler)
+    legacy_log_handler (format, arg_ptr);
+  else
+    vfprintf (stderr, format, arg_ptr);
+  va_end (arg_ptr);
+}
+
+/**
+ * @brief Legacy function to flush a log message
+ *
+ * This function shall be used instead of fflush for log messages via
+ * fprintf.  It will eventually be removed.
+ */
+void
+log_legacy_fflush (void)
+{
+  if (!legacy_log_handler)
+    fflush (stderr);
 }
