@@ -336,7 +336,10 @@ openvas_server_open_with_cert (gnutls_session_t *session, const char *host,
     {
       /* Make server socket. */
 
-      server_socket = socket (PF_INET, SOCK_STREAM, 0);
+      if (address->ai_family == AF_INET6)
+        server_socket = socket (PF_INET6, SOCK_STREAM, 0);
+      else
+        server_socket = socket (PF_INET, SOCK_STREAM, 0);
       if (server_socket == -1)
         {
           g_warning ("Failed to create server socket");
@@ -801,12 +804,12 @@ server_new_gnutls_set (unsigned int end_type, const char *priority,
      possible. At least from 3.0 this is an option:
      "NONE:+VERS-TLS1.0:+CIPHER-ALL:+COMP-ALL:+RSA:+DHE-RSA:+DHE-DSS:+MAC-ALL"
      But in fact this function is only for OpenVAS internal
-     purposes, not for scanning abilities. So, the conservative "SECURE"
+     purposes, not for scanning abilities. So, the conservative "NORMAL"
      is choosen.
   */
 
   if ((err_gnutls = gnutls_priority_set_direct (*server_session,
-                                                priority? priority : "SECURE",
+                                                priority? priority : "NORMAL",
                                                 NULL)))
     {
       g_warning ("%s: failed to set tls priorities: %s\n", __FUNCTION__,
