@@ -42,37 +42,58 @@ extern "C"
 #endif
 #endif
 
+#include <stdarg.h>
 #include <gnutls/gnutls.h>
+#include <gnutls/x509.h>
 #ifdef _WIN32
 #include <winsock2.h>
 #else
 #include <netinet/ip.h>
 #endif
 
+int openvas_server_verify (gnutls_session_t);
+
 int openvas_server_open (gnutls_session_t *, const char *, int);
 
+int
+openvas_server_open_with_cert (gnutls_session_t *, const char *, int,
+                               const char *, const char *, const char *);
+
 int openvas_server_close (int, gnutls_session_t);
 
-int openvas_server_connect (int, struct sockaddr_in *, gnutls_session_t *,
-                            gboolean);
+int openvas_server_connect (int, struct sockaddr_in *, gnutls_session_t *);
 
 int openvas_server_attach (int, gnutls_session_t *);
 
-int openvas_server_send (gnutls_session_t *, const char *);
-
 int openvas_server_sendf (gnutls_session_t *, const char *, ...);
 
-int openvas_server_sendf_xml (gnutls_session_t *, const char *, ...);
+int openvas_server_vsendf (gnutls_session_t *, const char *, va_list);
 
-int openvas_server_new (gnutls_connection_end_t, gchar *, gchar *, gchar *,
+int openvas_server_sendf_xml (gnutls_session_t *, const char *, ...);
+int openvas_server_sendf_xml_quiet (gnutls_session_t *, const char *, ...);
+
+int openvas_server_new (unsigned int, gchar *, gchar *, gchar *,
                         gnutls_session_t *, gnutls_certificate_credentials_t *);
 
+int openvas_server_new_mem (unsigned int, const char *, const char *,
+                            const char *, gnutls_session_t *,
+                            gnutls_certificate_credentials_t *);
+
 int openvas_server_free (int, gnutls_session_t,
                          gnutls_certificate_credentials_t);
 
 int openvas_server_session_free (gnutls_session_t,
                                  gnutls_certificate_credentials_t);
 
+int
+load_gnutls_file (const char *, gnutls_datum_t *);
+
+void
+unload_gnutls_file(gnutls_datum_t *);
+
+int
+set_gnutls_dhparams (gnutls_certificate_credentials_t, const char *);
+
 #if 0
 {
 #endif
