@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <netinet/in.h>
@@ -32,8 +32,9 @@
 
 #include "bpf_share.h"
 #include "pcap_openvas.h"
-#include "system_internal.h"
+#include "openvas_logging.h"
 #include "network.h"
+#include "support.h"
 
 #define MAXROUTES 1024
 
@@ -157,26 +158,6 @@ v6_is_local_ip (struct in6_addr *addr)
   return 0;
 }
 
-int
-is_local_ip (struct in_addr addr)
-{
-  int ifaces;
-  struct interface_info *ifs;
-  int i;
-
-  if ((ifs = getinterfaces (&ifaces)) == NULL)
-    return -1;
-  for (i = 0; i < ifaces; i++)
-    {
-      bpf_u_int32 net, mask;
-      char errbuf[PCAP_ERRBUF_SIZE];
-      pcap_lookupnet (ifs[i].name, &net, &mask, errbuf);
-      if ((net & mask) == (addr.s_addr & mask))
-        return 1;
-    }
-  return 0;
-}
-
 /**
  * @brief We send an empty UDP packet to the remote host, and read back its mac
  * @brief address.
@@ -210,13 +191,13 @@ v6_get_mac_addr (struct in6_addr *addr, char **mac)
         return -1;
 
       inaddr.s_addr = me.s6_addr32[3];
-      src_host = estrdup (inet_ntoa (inaddr));
+      src_host = g_strdup (inet_ntoa (inaddr));
       inaddr.s_addr = addr->s6_addr32[3];
-      dst_host = estrdup (inet_ntoa (inaddr));
-      snprintf (filter, sizeof (filter), "ip and (src host %s and dst host %s)",
+      dst_host = g_strdup (inet_ntoa (inaddr));
+      snprintf (filter, sizeof (filter)-1, "ip and (src host %s and dst host %s)",
                 src_host, dst_host);
-      efree (&src_host);
-      efree (&dst_host);
+      g_free (src_host);
+      g_free (dst_host);
 
       bpf = bpf_open_live (iface, filter);
       if (bpf < 0)
@@ -254,13 +235,13 @@ v6_get_mac_addr (struct in6_addr *addr, char **mac)
       if (soc < 0)
         return -1;
       src_host =
-        estrdup (inet_ntop (AF_INET6, &me, hostname, sizeof (hostname)));
+        g_strdup (inet_ntop (AF_INET6, &me, hostname, sizeof (hostname)));
       dst_host =
-        estrdup (inet_ntop (AF_INET6, addr, hostname, sizeof (hostname)));
-      snprintf (filter, sizeof (filter),
+        g_strdup (inet_ntop (AF_INET6, addr, hostname, sizeof (hostname)));
+      snprintf (filter, sizeof (filter)-1,
                 "ip6 and (src host %s and dst host %s)", src_host, dst_host);
-      efree (&src_host);
-      efree (&dst_host);
+      g_free (src_host);
+      g_free (dst_host);
 
 
       bpf = bpf_open_live (iface, filter);
@@ -308,7 +289,7 @@ v6_get_mac_addr (struct in6_addr *addr, char **mac)
               return 1;
             }
 
-          *mac = emalloc (22);
+          *mac = g_malloc0 (23);
           snprintf (*mac, 22, "%.2x.%.2x.%.2x.%.2x.%.2x.%.2x",
                     (unsigned char) packet[0], (unsigned char) packet[1],
                     (unsigned char) packet[2], (unsigned char) packet[3],
@@ -328,103 +309,6 @@ v6_get_mac_addr (struct in6_addr *addr, char **mac)
 }
 
 
-/**
- * @brief We send an empty UDP packet to the remote host, and read back its mac
- * @brief address.
- *
- * (we should first interrogate the kernel's arp cache - we may
- * rely on libdnet in the future to do that)
- *
- * As a bonus, this function works well as a local ping.
- */
-int
-get_mac_addr (struct in_addr addr, char **mac)
-{
-  int soc = socket (AF_INET, SOCK_DGRAM, 0);
-  struct sockaddr_in soca;
-  int bpf;
-  struct in_addr me;
-  char *iface = routethrough (&addr, &me);
-  char filter[255];
-  char *src_host, *dst_host;
-  unsigned char *packet;
-  int len;
-
-  *mac = NULL;
-  if (soc < 0)
-    return -1;
-
-  src_host = estrdup (inet_ntoa (me));
-  dst_host = estrdup (inet_ntoa (addr));
-  snprintf (filter, sizeof (filter), "ip and (src host %s and dst host %s)",
-            src_host, dst_host);
-  efree (&src_host);
-  efree (&dst_host);
-
-
-  bpf = bpf_open_live (iface, filter);
-  if (bpf < 0)
-    {
-      close (soc);
-      return -1;
-    }
-
-  /*
-   * We only deal with ethernet
-   */
-  if (bpf_datalink (bpf) != DLT_EN10MB)
-    {
-      bpf_close (bpf);
-      close (soc);
-      return -1;
-    }
-
-
-  soca.sin_addr.s_addr = addr.s_addr;
-  soca.sin_port = htons (9);    /* or whatever */
-  soca.sin_family = AF_INET;
-  if (sendto (soc, NULL, 0, 0, (struct sockaddr *) &soca, sizeof (soca)) == 0)
-    {
-      packet = (unsigned char *) bpf_next (bpf, &len);
-      if (packet)
-        {
-          if (len >= get_datalink_size (bpf_datalink (bpf)))
-            {
-              int i;
-              for (i = 0; i < 6; i++)
-                if (packet[i] != 0xFF)
-                  break;
-
-              if (i == 6)
-                {
-                  bpf_close (bpf);
-                  close (soc);
-                  return 1;
-                }
-
-              *mac = emalloc (22);
-              snprintf (*mac, 22, "%.2x.%.2x.%.2x.%.2x.%.2x.%.2x",
-                        (unsigned char) packet[0], (unsigned char) packet[1],
-                        (unsigned char) packet[2], (unsigned char) packet[3],
-                        (unsigned char) packet[4], (unsigned char) packet[5]);
-              bpf_close (bpf);
-              close (soc);
-              return 0;
-            }
-        }
-      else
-        {
-          bpf_close (bpf);
-          close (soc);
-          return 1;
-        }
-    }
-  bpf_close (bpf);
-  close (soc);
-  return -1;
-}
-
-
 /*
  * Taken straight out of Fyodor's Nmap
  */
@@ -535,7 +419,7 @@ islocalhost (struct in_addr *addr)
   if (addr == NULL)
     return -1;
 
-  /* If it is 0.0.0.0 or starts with 127.0.0.1 then it is 
+  /* If it is 0.0.0.0 or starts with 127.0.0.1 then it is
      probably localhost */
   if ((addr->s_addr & htonl (0xFF000000)) == htonl (0x7F000000))
     return 1;
@@ -731,6 +615,8 @@ v6_getinterfaces (int *howmany)
             }
         }
       *howmany = numinterfaces;
+
+      freeifaddrs (ifaddr);
     }
   return mydevs;
 }
@@ -757,17 +643,17 @@ getinterfaces (int *howmany)
   sd = socket (AF_INET, SOCK_DGRAM, 0);
   bzero (buf, sizeof (buf));
   if (sd < 0)
-    printf ("socket in getinterfaces");
+    log_legacy_write ("socket in getinterfaces");
 
   ifc.ifc_len = sizeof (buf);
   ifc.ifc_buf = buf;
   if (ioctl (sd, SIOCGIFCONF, &ifc) < 0)
-    printf ("Failed to determine your configured interfaces!\n");
+    log_legacy_write ("Failed to determine your configured interfaces!");
 
   close (sd);
   if (ifc.ifc_len == 0)
-    printf
-      ("getinterfaces: SIOCGIFCONF claims you have no network interfaces!\n");
+    log_legacy_write
+     ("getinterfaces: SIOCGIFCONF claims you have no network interfaces!");
 
   len = sizeof (struct ifmap);
 
@@ -786,8 +672,9 @@ getinterfaces (int *howmany)
       numinterfaces++;
       if (numinterfaces == 1023)
         {
-          printf
-            ("You seem to have more than 1023 network interfaces. Things may not work right.\n");
+          log_legacy_write
+           ("You seem to have more than 1023 network interfaces."
+            " Things may not work right.");
           break;
         }
       mydevs[numinterfaces].name[0] = '\0';
@@ -813,15 +700,6 @@ v6_getsourceip (struct in6_addr *src, struct in6_addr *dst)
   char name[INET6_ADDRSTRLEN];
 #endif
 
-  if (IN6_IS_ADDR_V4MAPPED (dst))
-    *src = socket_get_next_source_v4_addr ();
-  else
-    *src = socket_get_next_source_v6_addr ();
-  if (!IN6_ARE_ADDR_EQUAL (src, &in6addr_any))
-    {
-      return 1;
-    }
-
   get_random_bytes (&p1, 2);
   if (p1 < 5000)
     p1 += 5000;
@@ -839,7 +717,6 @@ v6_getsourceip (struct in6_addr *src, struct in6_addr *dst)
       if (connect (sd, (struct sockaddr *) &sock, sizeof (struct sockaddr_in))
           == -1)
         {
-          perror ("UDP connect()");
           close (sd);
           return 0;
         }
@@ -879,7 +756,6 @@ v6_getsourceip (struct in6_addr *src, struct in6_addr *dst)
       if (connect (sd, (struct sockaddr *) &sock6, sizeof (struct sockaddr_in6))
           == -1)
         {
-          perror ("UDP connect()");
           close (sd);
           return 0;
         }
@@ -914,13 +790,6 @@ getsourceip (struct in_addr *src, struct in_addr *dst)
   unsigned int socklen = sizeof (struct sockaddr_in);
   unsigned short p1;
 
-
-  *src = socket_get_next_source_addr ();
-  if (src->s_addr != INADDR_ANY)
-    {
-      return 1;
-    }
-
   get_random_bytes (&p1, 2);
   if (p1 < 5000)
     p1 += 5000;
@@ -936,7 +805,6 @@ getsourceip (struct in_addr *src, struct in_addr *dst)
   if (connect (sd, (struct sockaddr *) &sock, sizeof (struct sockaddr_in)) ==
       -1)
     {
-      perror ("UDP connect()");
       close (sd);
       return 0;
     }
@@ -981,7 +849,7 @@ getipv4routes (struct myroute *myroutes, int *numroutes)
       if (fgets (buf, sizeof (buf), routez) == NULL)  /* Kill the first line */
         {
           // /proc/net/route was empty or an error occurred.
-          printf ("Could not read from /proc/net/route");
+          log_legacy_write ("Could not read from /proc/net/route");
           fclose (routez);
           return -1;
         }
@@ -990,7 +858,8 @@ getipv4routes (struct myroute *myroutes, int *numroutes)
           p = strtok (buf, " \t\n");
           if (!p)
             {
-              printf ("Could not find interface in /proc/net/route line");
+              log_legacy_write ("Could not find interface in"
+                                " /proc/net/route line");
               continue;
             }
           strncpy (iface, p, sizeof (iface));
@@ -1006,7 +875,8 @@ getipv4routes (struct myroute *myroutes, int *numroutes)
 #endif
           if (!endptr || *endptr)
             {
-              printf ("Failed to determine Destination from /proc/net/route");
+              log_legacy_write ("Failed to determine Destination from"
+                                " /proc/net/route");
               continue;
             }
           inaddr.s_addr = dest;
@@ -1022,23 +892,24 @@ getipv4routes (struct myroute *myroutes, int *numroutes)
             }
           if (!p)
             {
-              printf ("Failed to find field %d in /proc/net/route", i + 2);
+              log_legacy_write ("Failed to find field %d in"
+                                " /proc/net/route", i + 2);
               continue;
             }
           endptr = NULL;
           mask = strtoul (p, &endptr, 16);
-          i = 31;
           ones = 0;
           i = 0;
           while (mask & (1 << i++) && i < 32)
             ones++;
           myroutes[*numroutes].mask = ones + 96;
 #ifdef TCPIP_DEBUGGING
-          printf ("mask is %d\n", myroutes[*numroutes].mask);
+          printf ("mask is %lu\n", myroutes[*numroutes].mask);
 #endif
           if (!endptr || *endptr)
             {
-              printf ("Failed to determine mask from /proc/net/route");
+              log_legacy_write ("Failed to determine mask from"
+                                " /proc/net/route");
               continue;
             }
 
@@ -1055,13 +926,13 @@ getipv4routes (struct myroute *myroutes, int *numroutes)
                 break;
               }
           if (i == numinterfaces)
-            printf
-              ("Failed to find interface %s mentioned in /proc/net/route\n",
+            log_legacy_write
+             ("Failed to find interface %s mentioned in /proc/net/route",
                iface);
           (*numroutes)++;
           if (*numroutes >= MAXROUTES)
             {
-              printf ("My god!  You seem to have WAY to many routes!\n");
+              log_legacy_write ("You seem to have WAY to many routes!");
               break;
             }
         }
@@ -1120,7 +991,7 @@ getipv6routes (struct myroute *myroutes, int *numroutes)
 #endif
               if (inet_pton (AF_INET6, v6addr, &in6addr) <= 0)
                 {
-                  printf ("invalid ipv6 addressd\n");
+                  log_legacy_write ("invalid ipv6 addressd");
                   continue;
                 }
               memcpy (&myroutes[*numroutes].dest6, &in6addr,
@@ -1137,7 +1008,7 @@ getipv6routes (struct myroute *myroutes, int *numroutes)
             {
               token = strtok (NULL, " \t\n");
               if (!token)
-                printf ("error\n");
+                log_legacy_write ("getipv6routes error");
             }
 
           token = strtok (NULL, " \t\n");
@@ -1156,13 +1027,13 @@ getipv6routes (struct myroute *myroutes, int *numroutes)
                 break;
               }
           if (i == numinterfaces)
-            printf
-              ("Failed to find interface %s mentioned in /proc/net/route\n",
-               iface);
+            log_legacy_write
+             ("Failed to find interface %s mentioned in /proc/net/route\n",
+              iface);
           (*numroutes)++;
           if (*numroutes >= MAXROUTES)
             {
-              printf ("My god!  You seem to have WAY to many routes!\n");
+              log_legacy_write ("You seem to have WAY to many routes!");
               break;
             }
         }
@@ -1171,7 +1042,7 @@ getipv6routes (struct myroute *myroutes, int *numroutes)
     }
   else
     {
-      printf ("Didn't find IPv6 routes\n");
+      log_legacy_write ("Didn't find IPv6 routes");
       return -1;
     }
 }
@@ -1205,15 +1076,13 @@ v6_routethrough (struct in6_addr *dest, struct in6_addr *source)
 #endif
   struct in6_addr src;
 
-  *source = in6addr_any;
-
   if (!dest)
-    printf ("ipaddr2devname passed a NULL dest address");
+    log_legacy_write ("ipaddr2devname passed a NULL dest address");
 
   if (IN6_IS_ADDR_V4MAPPED (dest))
-    src = socket_get_next_source_v4_addr ();
+    openvas_source_addr_as_addr6 (&src);
   else
-    src = socket_get_next_source_v6_addr ();
+    openvas_source_addr6 (&src);
 
   if (!initialized)
     {
@@ -1323,8 +1192,7 @@ v6_routethrough (struct in6_addr *dest, struct in6_addr *source)
 
           myhostent = gethostbyname (myname);
           if (gethostname (myname, MAXHOSTNAMELEN) || !myhostent)
-            printf
-              ("Cannot get hostname!  Try using -S <my_IP_address> or -e <interface to scan through>\n");
+            log_legacy_write ("Cannot get hostname!");
           if (myhostent->h_addrtype == AF_INET)
             {
               addy.s6_addr32[0] = 0;
@@ -1357,8 +1225,8 @@ v6_routethrough (struct in6_addr *dest, struct in6_addr *source)
       return NULL;
     }
   else
-    printf
-      ("I know sendmail technique ... I know rdist technique ... but I don't know what the hell kindof technique you are attempting!!!");
+    log_legacy_write ("%s: Provided technique is neither proc route nor"
+                      " connect socket", __FUNCTION__);
   return NULL;
 }
 
@@ -1393,11 +1261,14 @@ routethrough (struct in_addr *dest, struct in_addr *source)
   char iface[64];
   static int numroutes = 0;
   FILE *routez;
+  long match = -1;
+  unsigned long bestmatch = 0;
 
-  struct in_addr src = socket_get_next_source_addr ();
+  struct in_addr src;
 
+  openvas_source_addr (&src);
   if (!dest)
-    printf ("ipaddr2devname passed a NULL dest address");
+    log_legacy_write ("ipaddr2devname passed a NULL dest address");
 
   if (!initialized)
     {
@@ -1413,13 +1284,14 @@ routethrough (struct in_addr *dest, struct in_addr *source)
           /* OK, linux style /proc/net/route ... we can handle this ... */
           /* Now that we've got the interfaces, we g0 after the r0ut3Z */
           if (fgets (buf, sizeof (buf), routez) == NULL)  /* Kill the first line */
-            printf ("Could not read from /proc/net/route");
+            log_legacy_write ("Could not read from /proc/net/route");
           while (fgets (buf, sizeof (buf), routez))
             {
               p = strtok (buf, " \t\n");
               if (!p)
                 {
-                  printf ("Could not find interface in /proc/net/route line");
+                  log_legacy_write ("Could not find interface in"
+                                    " /proc/net/route line");
                   continue;
                 }
               strncpy (iface, p, sizeof (iface));
@@ -1432,8 +1304,8 @@ routethrough (struct in_addr *dest, struct in_addr *source)
               myroutes[numroutes].dest = strtoul (p, &endptr, 16);
               if (!endptr || *endptr)
                 {
-                  printf
-                    ("Failed to determine Destination from /proc/net/route");
+                  log_legacy_write
+                   ("Failed to determine Destination from /proc/net/route");
                   continue;
                 }
               for (i = 0; i < 6; i++)
@@ -1444,14 +1316,16 @@ routethrough (struct in_addr *dest, struct in_addr *source)
                 }
               if (!p)
                 {
-                  printf ("Failed to find field %d in /proc/net/route", i + 2);
+                  log_legacy_write ("Failed to find field %d in"
+                                    " /proc/net/route", i + 2);
                   continue;
                 }
               endptr = NULL;
               myroutes[numroutes].mask = strtoul (p, &endptr, 16);
               if (!endptr || *endptr)
                 {
-                  printf ("Failed to determine mask from /proc/net/route");
+                  log_legacy_write ("Failed to determine mask"
+                                    " from /proc/net/route");
                   continue;
                 }
 
@@ -1468,13 +1342,13 @@ routethrough (struct in_addr *dest, struct in_addr *source)
                     break;
                   }
               if (i == numinterfaces)
-                printf
-                  ("Failed to find interface %s mentioned in /proc/net/route\n",
-                   iface);
+                log_legacy_write
+                 ("Failed to find interface %s mentioned in /proc/net/route",
+                  iface);
               numroutes++;
               if (numroutes >= MAXROUTES)
                 {
-                  printf ("My god!  You seem to have WAY to many routes!\n");
+                  log_legacy_write ("You seem to have WAY to many routes!");
                   break;
                 }
             }
@@ -1511,7 +1385,7 @@ routethrough (struct in_addr *dest, struct in_addr *source)
     {
       for (i = 0; i < numroutes; i++)
         {
-          if ((dest->s_addr & myroutes[i].mask) == myroutes[i].dest)
+          if ((dest->s_addr & myroutes[i].mask) == myroutes[i].dest && myroutes[i].mask >= bestmatch)
             {
               if (source)
                 {
@@ -1520,9 +1394,12 @@ routethrough (struct in_addr *dest, struct in_addr *source)
                   else
                     source->s_addr = myroutes[i].dev->addr.s_addr;
                 }
-              return myroutes[i].dev->name;
+              match = i;
+              bestmatch = myroutes[i].mask;
             }
         }
+      if (match != -1)
+          return myroutes[match].dev->name;
     }
   else if (technique == connectsockettechnique)
     {
@@ -1534,8 +1411,7 @@ routethrough (struct in_addr *dest, struct in_addr *source)
           char myname[MAXHOSTNAMELEN + 1];
           myhostent = gethostbyname (myname);
           if (gethostname (myname, MAXHOSTNAMELEN) || !myhostent)
-            printf
-              ("Cannot get hostname!  Try using -S <my_IP_address> or -e <interface to scan through>\n");
+            log_legacy_write ("Cannot get hostname!");
           memcpy (&(addy.s_addr), myhostent->h_addr_list[0],
                   sizeof (struct in_addr));
         }
@@ -1553,7 +1429,7 @@ routethrough (struct in_addr *dest, struct in_addr *source)
       return NULL;
     }
   else
-    printf
-      ("I know sendmail technique ... I know rdist technique ... but I don't know what the hell kindof technique you are attempting!!!");
+    log_legacy_write ("%s: Provided technique is neither proc route"
+                      " nor connect socket", __FUNCTION__);
   return NULL;
 }
