@@ -12,7 +12,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
@@ -24,7 +24,7 @@
 
 #include "kb.h"
 #include "plugutils.h"
-#include "system.h"
+#include "support.h"
 
 /*
  * This function implements "whisker like" IDS evasion tactics plus a couple
@@ -36,7 +36,7 @@
  * 2002-02-26: added Pavel kankovsky's "absolute URI"
  * 2002-03-06: added "CGI.pm parameters" by Noam Rathaus from securiteam.com
  * Partial support: we should changed & to ; in the "posted" data.
- * 2002-05-29: added "protocol string". See Alla Bezroutchko's message on 
+ * 2002-05-29: added "protocol string". See Alla Bezroutchko's message on
  * VULN-DEV (Date: 15 Feb 2002; From: <alla@scanit.be>;
  * Subject: Possible IDS-evasion technique
  * Message-ID: <3C6D434B.4377034A@scanit.be>)
@@ -99,7 +99,7 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
 #define URL_CODE_UTF8BAD	4
   int url_encoding;
   char gizmo[32];
-  struct kb_item **kb = plug_get_kb (data);
+  kb_t kb = plug_get_kb (data);
 
   /* Basically, we need to store the path, a slash, and the name.
    * Encoding will expand this
@@ -110,14 +110,14 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
   l += strlen (name) + (path != NULL);
 
   /** @todo Evaluate if GLib functions for building paths are applicable here */
-  ret = emalloc (l + 1);
+  ret = g_malloc0 (l + 1);
   if (path == NULL)
     strcpy (ret, name);
   else
     sprintf (ret, "%s/%s", path, name);
 
 #ifdef URL_DEBUG
-  fprintf (stderr, "Request => %s\n", ret);
+  log_legacy_write ("Request => %s\n", ret);
 #endif
 
   for (s = ret; *s != '\0'; s++)
@@ -148,7 +148,7 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
             }
 #ifdef URL_DEBUG
       if (i > 0)
-        fprintf (stderr, "Request =  %s\n", ret);
+        log_legacy_write ("Request =  %s\n", ret);
 #endif
     }
 
@@ -157,7 +157,7 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
   if (self_ref_dir)
     {
       l += 2 * n_slash;
-      ret2 = emalloc (l + 1);
+      ret2 = g_malloc0 (l + 1);
       for (s = ret, s2 = ret2; *s != '\0' && *s != '?'; s++)
         if (*s != '/')
           *s2++ = *s;
@@ -169,11 +169,11 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
       while (*s != '\0')
         *s2++ = *s++;
       *s2 = '\0';
-      efree (&ret);
+      g_free (ret);
       ret = ret2;
       n_slash *= 2;
 #ifdef URL_DEBUG
-      fprintf (stderr, "Request =  %s\n", ret);
+      log_legacy_write ("Request =  %s\n", ret);
 #endif
     }
 
@@ -183,7 +183,7 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
   if (reverse_traversal > 0)
     {
       l += (reverse_traversal + 4) * n_slash;
-      ret2 = emalloc (l + 1);
+      ret2 = g_malloc0 (l + 1);
 
       for (s = ret, s2 = ret2; *s != '\0' && *s != '?'; s++)
         if (*s != '/')
@@ -199,11 +199,11 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
       while (*s != '\0')
         *s2++ = *s++;
       *s2 = '\0';
-      efree (&ret);
+      g_free (ret);
       ret = ret2;
       n_slash *= 3;
 #ifdef URL_DEBUG
-      fprintf (stderr, "Request =  %s\n", ret);
+      log_legacy_write ("Request =  %s\n", ret);
 #endif
     }
 
@@ -212,7 +212,7 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
   if (prem_req_end)
     {
       l += 36;
-      ret2 = emalloc (l + 1);
+      ret2 = g_malloc0 (l + 1);
       n_slash += 4;
 
       s = gizmo;
@@ -221,10 +221,10 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
         *s++ = lrand48 () % 26 + 'a';   /* RATS: ignore */
       *s++ = '\0';
       snprintf (ret2, l, "/%%20HTTP/1.0%%0d%%0a%s:%%20/../..%s", gizmo, ret);   /* RATS: ignore */
-      efree (&ret);
+      g_free (ret);
       ret = ret2;
 #ifdef URL_DEBUG
-      fprintf (stderr, "Request =  %s\n", ret);
+      log_legacy_write ("Request =  %s\n", ret);
 #endif
     }
 
@@ -233,7 +233,7 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
   if (param_hiding)
     {
       l += 25;
-      ret2 = emalloc (l + 1);
+      ret2 = g_malloc0 (l + 1);
       n_slash += 2;
 
       s = gizmo;
@@ -241,10 +241,10 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
         *s++ = lrand48 () % 26 + 'a';   /* RATS: ignore */
       *s++ = '\0';
       snprintf (ret2, l, "/index.htm%%3f%s=/..%s", gizmo, ret); /* RATS: ignore */
-      efree (&ret);
+      g_free (ret);
       ret = ret2;
 #ifdef URL_DEBUG
-      fprintf (stderr, "Request =  %s\n", ret);
+      log_legacy_write ("Request =  %s\n", ret);
 #endif
     }
 
@@ -254,7 +254,7 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
     {
       l += n_slash;
 
-      ret2 = emalloc (l + 1);
+      ret2 = g_malloc0 (l + 1);
       for (s = ret, s2 = ret2; *s != '\0' && *s != '?'; s++)
         if (*s != '/')
           *s2++ = *s;
@@ -266,11 +266,11 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
       while (*s != '\0')
         *s2++ = *s++;
       *s2 = '\0';
-      efree (&ret);
+      g_free (ret);
       ret = ret2;
       n_slash *= 2;
 #ifdef URL_DEBUG
-      fprintf (stderr, "Request =  %s\n", ret);
+      log_legacy_write ("Request =  %s\n", ret);
 #endif
     }
 
@@ -285,7 +285,7 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
             n_backslash++;
           }
 #ifdef URL_DEBUG
-      fprintf (stderr, "Request =  %s\n", ret);
+      log_legacy_write ("Request =  %s\n", ret);
 #endif
     }
 
@@ -313,7 +313,7 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
       l = (l - n_slash - n_backslash) * 6 + n_slash + n_backslash;
       break;
     case URL_CODE_HEX:
-      /* We do not encode slashes, as this does not work against Apache, 
+      /* We do not encode slashes, as this does not work against Apache,
        * at least apache-1.3.22-2 from redhat */
       l = (l - n_slash) * 3 + n_slash;
       break;
@@ -321,7 +321,7 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
 
   if (url_encoding != URL_CODE_NONE)
     {
-      ret2 = emalloc (l + 1);
+      ret2 = g_malloc0 (l + 1);
 
       for (s = ret, s2 = ret2; *s != '\0'; s++)
         if (*s == '/'
@@ -375,10 +375,10 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
               n_backslash--;
           }
       *s2 = '\0';
-      efree (&ret);
+      g_free (ret);
       ret = ret2;
 #ifdef URL_DEBUG
-      fprintf (stderr, "Request =  %s\n", ret);
+      log_legacy_write ("Request =  %s\n", ret);
 #endif
     }
 
@@ -404,25 +404,12 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
           else if (strcmp (abs_URI_host, "host IP") == 0)
             {
               struct in6_addr *ptr;
-              char *asc;
-              char hostname[255];
 
               if ((ptr = plug_get_host_ip (data)) != NULL)
                 {
-                  if (IN6_IS_ADDR_V4MAPPED (ptr))
-                    {
-                      struct in_addr *v4_addr = NULL;
-                      v4_addr->s_addr = ptr->s6_addr32[3];
-                      asc =
-                        estrdup (inet_ntop (AF_INET, v4_addr, hostname,
-                                            sizeof (hostname)));
-                    }
-                  else
-                    asc =
-                      estrdup (inet_ntop
-                               (AF_INET6, &ptr, hostname, sizeof (hostname)));
-
+                  char *asc = addr6_as_str (ptr);
                   strncpy (h, asc, sizeof (h));
+                  g_free (asc);
                 }
               h[sizeof (h) - 1] = '\0';
             }
@@ -435,21 +422,16 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
           else if (strcmp (abs_URI_host, "random IP") == 0)
             sprintf (h, "%d.%d.%d.%d", rand () % 256, rand () % 256,
                      rand () % 256, rand () % 256);
-#if 0
-          else
-            fprintf (stderr, "Unhandled value %s\n", abs_URI_host);
-#endif
         }
 
       l += strlen (h) + strlen (abs_URI_type) + 3;
-      n_slash += 2;
-      ret2 = emalloc (l + 1);
+      ret2 = g_malloc0 (l + 1);
 
       snprintf (ret2, l, "%s://%s%s", abs_URI_type, h, ret);    /* RATS: ignore */
-      efree (&ret);
+      g_free (ret);
       ret = ret2;
 #ifdef URL_DEBUG
-      fprintf (stderr, "Request =  %s\n", ret);
+      log_legacy_write ("Request =  %s\n", ret);
 #endif
     }
 
@@ -459,10 +441,10 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
   if (null_method)
     {
       l += 3;
-      ret2 = emalloc (l + 1);
+      ret2 = g_malloc0 (l + 1);
       strncpy (ret2, "%00", l);
       strncpy (ret2 + 3, ret, (l - 3));
-      efree (&ret);
+      g_free (ret);
       ret = ret2;
     }
 
@@ -474,12 +456,7 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
     {
       s = kb_item_get_str (kb, "NIDS/HTTP/protocol_string");
       if (s != NULL && *s != '\0')
-        {
-          httpver = s;
-#if 0
-          fprintf (stderr, "NIDS/HTTP/protocol_string = %s\n", s);
-#endif
-        }
+        httpver = s;
       l += strlen (httpver) + 2;
     }
 
@@ -488,16 +465,16 @@ build_encode_URL (struct arglist *data, char *method, char *path, char *name,
   tab_sep = (s != NULL && strcmp (s, "yes") == 0);
   sep_c = (tab_sep ? '\t' : ' ');
 
-  ret2 = emalloc (l + 1);
+  ret2 = g_malloc0 (l + 1);
   if (http09)
     snprintf (ret2, l, "%s%c%s", method, sep_c, ret);   /* RATS: ignore */
   else
     snprintf (ret2, l, "%s%c%s%c%s", method, sep_c, ret, sep_c, httpver);       /* RATS: ignore */
-  efree (&ret);
+  g_free (ret);
   ret = ret2;
 
 #ifdef URL_DEBUG
-  fprintf (stderr, "Request <= %s\n", ret);
+  log_legacy_write ("Request <= %s\n", ret);
 #endif
   return ret;
 }
