@@ -18,7 +18,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    MODIFICATIONS: only those functions that are required for OpenVAS are retained, others are removed
    Modified By Preeti Subramanian <spreeti@secpod.com>
@@ -36,6 +36,7 @@
 #include "iconv.h"
 #include "smb.h"
 #include "proto.h"
+#include "openvas_logging.h"
 
 #ifndef SMB_STRDUP
 #define SMB_STRDUP(s) strdup(s)
@@ -263,7 +264,8 @@ void init_iconv_ntlmssp(void)
         }
         conv_handles_ntlmssp[c1][c2] = smb_iconv_open_ntlmssp(n2,n1);
         if (!conv_handles_ntlmssp[c1][c2]) {
-          printf("init_iconv_ntlmssp: conv_handle initialization failed");
+          log_legacy_write ("init_iconv_ntlmssp: conv_handle"
+                            " initialization failed");
         }
       }
     }
@@ -330,27 +332,26 @@ static size_t convert_string_internal_ntlmssp(charset_t from, charset_t to,
 
   retval = smb_iconv_ntlmssp(descriptor, &inbuf, &i_len, &outbuf, &o_len);
   if(retval==(size_t)-1) {
-    const char *reason="unknown error";
     switch(errno) {
       case EINVAL:
-        reason="Incomplete multibyte sequence";
+        /* Incomplete multibyte sequence */
         if (!conv_silent_ntlmssp)
           if (allow_bad_conv)
             goto use_as_is;
         return (size_t)-1;
       case E2BIG:
-        reason="No more room";
+        /* No more room */
       break;
       case EILSEQ:
-        reason="Illegal multibyte sequence";
+        /* Illegal multibyte sequence */
         if (allow_bad_conv)
           goto use_as_is;
 
         return (size_t)-1;
       default:
+        /* unknown error */
         return (size_t)-1;
     }
-    /* printf(reason); */
   }
   return destlen-o_len;
 
@@ -574,7 +575,7 @@ size_t convert_string_ntlmssp(charset_t from, charset_t to,
      if (((slen != (size_t)-1) && slen) ||
         ((slen == (size_t)-1) && lastp)) {
            errno = E2BIG;
-     }   
+     }
     }
     return retval;
   }
@@ -607,13 +608,13 @@ size_t push_ascii_ntlmssp(void *dest, const char *src, size_t dest_len, int flag
 
   /* No longer allow a length of -1. */
   if (dest_len == (size_t)-1) {
-    printf("push_ascii - dest_len == -1");
+    log_legacy_write ("push_ascii - dest_len == -1");
   }
 
   if (flags & STR_UPPER) {
     tmpbuf = SMB_STRDUP(src);
     if (!tmpbuf) {
-      printf("malloc fail");
+      log_legacy_write ("malloc fail");
     }
     strupper_m_ntlmssp(tmpbuf);
     src = tmpbuf;
