@@ -30,6 +30,7 @@
 #include "regex.h"
 #include "../misc/openvas_logging.h"
 #include "../misc/prefs.h"           /* for prefs_get */
+#include "../misc/plugutils.h"
 
 #include "nasl.h"
 #include "nasl_tree.h"
@@ -166,7 +167,10 @@ cell2str (lex_ctxt * lexic, tree_cell * c)
       if (c->x.str_val == NULL)
         p = g_strdup ("");
       else
-        p = g_memdup (c->x.str_val, c->size + 1);
+        {
+          p = g_malloc0 (c->size + 1);
+          memcpy (p, c->x.str_val, c->size);
+        }
       return p;
 
     case REF_ARRAY:
@@ -1679,7 +1683,7 @@ exec_nasl_script (struct arglist *script_infos, const char *name,
 {
   naslctxt ctx;
   nasl_func *pf;
-  int err = 0;
+  int err = 0, to, process_id;
   tree_cell *ret;
   lex_ctxt *lexic;
   gchar *old_dir;
@@ -1687,7 +1691,6 @@ exec_nasl_script (struct arglist *script_infos, const char *name,
   char *old;
   tree_cell tc;
   const char *str;
-  int to;
 
   srand48 (getpid () + getppid () + (long) time (NULL));
 
@@ -1717,6 +1720,7 @@ exec_nasl_script (struct arglist *script_infos, const char *name,
   bzero (&ctx, sizeof (ctx));
   if (mode & NASL_ALWAYS_SIGNED)
     ctx.always_authenticated = 1;
+  ctx.kb = plug_get_kb (script_infos);
 
   if (init_nasl_ctx (&ctx, name) == 0)
     {
@@ -1761,6 +1765,7 @@ exec_nasl_script (struct arglist *script_infos, const char *name,
   *        of which copies are made when needed. */
   init_nasl_library (lexic);
 
+  process_id = getpid ();
   if (mode & NASL_LINT)
     {
       if (nasl_lint (lexic, ctx.tree) == NULL)
@@ -1830,6 +1835,8 @@ exec_nasl_script (struct arglist *script_infos, const char *name,
 
   nasl_clean_ctx (&ctx);
   free_lex_ctxt (lexic);
+  if (process_id != getpid ())
+    exit (0);
 
   return err;
 }
