@@ -1,4 +1,4 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2003 Michel Arboi and Renaud Deraison
  *
@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef EXEC_H_INCLUDED
@@ -22,5 +22,4 @@
 tree_cell *nasl_exec (lex_ctxt *, tree_cell *);
 int cell_cmp (lex_ctxt *, tree_cell *, tree_cell *);
 tree_cell *cell2atom (lex_ctxt *, tree_cell *);
-int check_authenticated (lex_ctxt *);
 #endif
