@@ -12,7 +12,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
@@ -33,9 +33,8 @@
 #include "../misc/arglists.h" /* for struct arglist */
 #include "nvt_categories.h" /* for ACT_SCANNER */
 #include "../misc/plugutils.h" /* for find_in_path */
-#include "../misc/scanners_utils.h" /* for comm_send_status */
-#include "../misc/system.h" /* for efree */
-#include "../misc/services.h" /* for get_tcp_svcs */
+#include "../misc/prefs.h"          /* for prefs_get */
+#include "../misc/openvas_logging.h"
 
 #include "nasl_lex_ctxt.h"
 
@@ -44,7 +43,7 @@
 #include <netinet/ip.h>
 #endif
 #include <limits.h>
-#include <math.h>
+#include <math.h>         /* for sqrt(), floor() */
 
 #if ! defined FD_SETSIZE || FD_SETSIZE > 1024
 #define GRAB_MAX_SOCK		1024
@@ -126,16 +125,11 @@ my_socket_close(int s)
 
 static int std_port(int port)
 {
-  const char	*name;
-
-  if (port < 1 || port > 65535) return 0;
-  name = openvas_get_svc_name(port, NULL);
-  if  (name == NULL || strcmp(name, "unknown") == 0)
-    return 0;
-#if DEBUG > 2
-  fprintf(stderr, "openvas_tcp_scanner: std_port(%d)=%s\n", port, name != NULL ? name : "(null)");
-#endif
-  return 1;
+  return 0; /** @todo: We are not able anymore to judge wether a port is a standard
+             * port. Previously a port was believed to be a standard port
+             * when it occured in the currently configured list of ports.
+             * This needs to be resolved.
+             */
 }
 
 static int
@@ -151,11 +145,13 @@ double_check_std_ports(unsigned char* ports_states)
       }
     else if (ports_states[port] == GRAB_PORT_UNKNOWN)
       {
-	fprintf(stderr, "openvas_tcp_scanner: bug in double_check_std_ports! Unknown port %d status\n", port);
+	log_legacy_write ("openvas_tcp_scanner: bug in double_check_std_ports!"
+                          " Unknown port %d status", port);
 	tbd_nb ++;
       }
 #if DEBUG > 0
-  fprintf(stderr, "opanvas_tcp_scanner: double_check_std_ports found %d filtered standard ports\n", tbd_nb);
+  log_legacy_write ("opanvas_tcp_scanner: double_check_std_ports found %d"
+                    " filtered standard ports\n", tbd_nb);
 #endif
   return tbd_nb;
 }
@@ -165,9 +161,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	    const int read_timeout,
 	    int		min_cnx,
 	    int		max_cnx,
-	    struct arglist *globals,
-	    struct arglist *desc,
-	    struct arglist *hostinfos)
+	    struct arglist *desc)
 {
   char			buf[2048], kb[64];
   int			s, tcpproto, pass;
@@ -196,7 +190,6 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 #if DEBUG > 1
   int			done_ports_nb;
 #endif
-  int			scanned_port_nb;
   int			cnx_max[3], rtt_max[3], rtt_min[3], ping_rtt = 0;
 #if defined COMPUTE_RTT
   double		rtt_sum[3], rtt_sum2[3];
@@ -235,18 +228,20 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
   {
     char	*k;
     int		type = 0;
-    k = plug_get_key(desc, "/tmp/ping/RTT", &type);
+    k = plug_get_key (desc, "/tmp/ping/RTT", &type);
     if (type == ARG_STRING && k != NULL)
       ping_rtt = atoi(k);
     else if (type == ARG_INT)
       ping_rtt = GPOINTER_TO_SIZE(k);
     else if (type >= 0)
-      fprintf(stderr, "openvas_tcp_scanner: unknown key type %d\n", type);
+      log_legacy_write ("openvas_tcp_scanner: unknown key type %d", type);
+    g_free (k);
     if (ping_rtt < 0 || ping_rtt > MAX_SANE_RTT)
       ping_rtt = 0;
 #if DEBUG > 0
     else
-	fprintf(stderr, "openvas_tcp_scanner(%s): ping_rtt=%g s\n", inet_ntoa(*pia), ping_rtt / 1e6);
+	log_legacy_write ("openvas_tcp_scanner(%s): ping_rtt=%g s",
+                          inet_ntoa(*pia), ping_rtt / 1e6);
 #endif
   }
 
@@ -256,29 +251,21 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
     p = (char*)portrange;
     untested_ports_nb = 0;
 
-    if (p == NULL || *p == '\0' || strcmp(p, "default") == 0)
-      {
-	int	last_num = 0;
-	unsigned short * nums = GSIZE_TO_POINTER(get_tcp_svcs(&last_num));
-
-	if (nums == NULL)
-	  {
-	    fprintf(stderr, "openvas_tcp_scanner: Cannot get list of default services\n");
-	    return -1;
-	  }
-	for (i = 0; i < last_num; i ++)
-	    {
-	      ports_states[nums[i]] = GRAB_PORT_UNKNOWN;
-	      untested_ports_nb ++;
-	    }
-	efree(&nums);
-      }
-    else
+    if (p)
       while (*p != '\0')
 	{
 	  while (*p == ',')
 	    p ++;
 
+	  /* Scanner accepts only T:1-3,6,U:103-333,770 due to getpts. */
+
+	  if (*p == 'T' && p[1] && p[1] == ':')
+	    /* Skip over the leading "T:". */
+	    p += 2;
+	  else if (*p == 'U' && p[1] && p[1] == ':')
+	    /* "U:" for UDP.  Skip the rest. */
+	    break;
+
 	  if (*p == '-')
 	    {
 	      po1 = 1;
@@ -286,7 +273,8 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	      po2 = strtol(q, &p, 10);
 	      if (q == p)
 		{
-		  fprintf(stderr, "openvas_tcp_scanner: Cannot parse '%s'\n", p);
+		  log_legacy_write ("openvas_tcp_scanner: Cannot parse '%s'",
+                                    p);
 		  return -1;
 		}
 	    }
@@ -295,7 +283,8 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	      po1 = strtol(p, &q, 10);
 	      if (q == p)
 		{
-		  fprintf(stderr, "openvas_tcp_scanner: Cannot parse '%s'\n", p);
+		  log_legacy_write ("openvas_tcp_scanner: Cannot parse '%s'",
+                                    p);
 		  return -1;
 		}
 	      if (*q == ',')
@@ -320,7 +309,8 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 		      po2 = strtol(q+1, &p, 10);
 		      if (q+1 == p)
 			{
-			  fprintf(stderr, "openvas_tcp_scanner: Cannot parse '%s'\n", p);
+			  log_legacy_write
+                           ("openvas_tcp_scanner: Cannot parse '%s'", p);
 			  return -1;
 			}
 		    }
@@ -332,6 +322,11 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	      untested_ports_nb ++;
 	    }
 	}
+    else
+      {
+         log_legacy_write ("openvas_tcp_scanner: port list empty");
+         return -1;
+      }
   }
 
   for (i = 0; i < max_cnx; i ++)
@@ -349,12 +344,11 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
     {
       int	open_ports_nb1 = 0, closed_ports_nb1 = 0;
       int	wait_sock_nb = 0;
-      int	prev_scanned_port_nb = 0;
 
       minport = 1;
       start_time_1pass = time(NULL);
 #if DEBUG > 0
-      fprintf(stderr, "openvas_tcp_scanner(%s): pass #%d: open_sock_max=%d\topen_sock_max2=%d\n", inet_ntoa(*pia), pass, open_sock_max, open_sock_max2);
+      log_legacy_write ("openvas_tcp_scanner(%s): pass #%d: open_sock_max=%d\topen_sock_max2=%d\n", inet_ntoa(*pia), pass, open_sock_max, open_sock_max2);
 #endif
 
       FD_ZERO(&rfs); FD_ZERO(&wfs); imax = -1;
@@ -362,17 +356,8 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
       while (scanned_ports < 65535)
 	{
 	  total_ports_nb = unfiltered_ports_nb + filtered_ports_nb + untested_ports_nb;
-	  scanned_port_nb = unfiltered_ports_nb + filtered_ports_nb;
-	  if (scanned_port_nb > prev_scanned_port_nb + 99)
-	    {
-	      if ( globals != NULL ) comm_send_status(globals, arg_get_value(hostinfos, "NAME"),"portscan",
-			   scanned_port_nb,
-			   total_ports_nb);
-	      prev_scanned_port_nb = scanned_port_nb;
-	    }
-
 #if DEBUG > 0
-	  fprintf(stderr, "openvas_tcp_scanner(%s): %d / %d = %02d%% - %d ports remaining\n",
+	  log_legacy_write ("openvas_tcp_scanner(%s): %d / %d = %02d%% - %d ports remaining\n",
 		  inet_ntoa(*pia),
 		  unfiltered_ports_nb + filtered_ports_nb,
 		  total_ports_nb,
@@ -390,7 +375,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 
 	      ports_states[port] = GRAB_PORT_TESTING;
 #if DEBUG > 2
-	      fprintf(stderr, "openvas_tcp_scanner: Trying %s:%d\n", inet_ntoa(*pia), port);
+	      log_legacy_write ("openvas_tcp_scanner: Trying %s:%d\n", inet_ntoa(*pia), port);
 #endif
         if(IN6_IS_ADDR_V4MAPPED(pia))
         {
@@ -409,7 +394,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 #if DEBUG > 0
 		      /* DEBUG: otherwise, we print a less frigthtening message */
 		      perror("socket");
-		      fprintf(stderr, "openvas_tcp_scanner(%s): Reducing the number of maximum open connections to %d [ENFILE]\n", inet_ntoa(*pia), open_sock_max);
+		      log_legacy_write ("openvas_tcp_scanner(%s): Reducing the number of maximum open connections to %d [ENFILE]\n", inet_ntoa(*pia), open_sock_max);
 #endif
 		      continue;
 		    }
@@ -422,7 +407,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 #if DEBUG > 0
 		      /* DEBUG: otherwise, we print a less frigthtening message */
 		      perror("socket");
-		      fprintf(stderr, "openvas_tcp_scanner(%s): Reducing the number of maximum open connections to %d [EMFILE]\n", inet_ntoa(*pia), open_sock_max);
+		      log_legacy_write ("openvas_tcp_scanner(%s): Reducing the number of maximum open connections to %d [EMFILE]\n", inet_ntoa(*pia), open_sock_max);
 #endif
 		      continue;
 		    }
@@ -438,7 +423,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 		  open_sock_max --;
 		  open_sock_max2 --;
 #if DEBUG > 0
-		  fprintf(stderr, "openvas_tcp_scanner(%s): socket=%d > FD_SETSIZE=%d - reducing the number of maximum open connections to %d\n", inet_ntoa(*pia), s, FD_SETSIZE, open_sock_max);
+		  log_legacy_write ("openvas_tcp_scanner(%s): socket=%d > FD_SETSIZE=%d - reducing the number of maximum open connections to %d\n", inet_ntoa(*pia), s, FD_SETSIZE, open_sock_max);
 #endif
 		  if (close(s) < 0)
 		    perror("close");
@@ -523,7 +508,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 			open_sock_nb - (x > 0 ? x : 1);
 		      /* If open_sock_max2 < 0, the scanner aborts */
 #if DEBUG > 0
-		      fprintf(stderr, "openvas_tcp_scanner(%s): Reducing the number of maximum open connections to %d [EAGAIN]\n", inet_ntoa(*pia), open_sock_max);
+		      log_legacy_write ("openvas_tcp_scanner(%s): Reducing the number of maximum open connections to %d [EAGAIN]\n", inet_ntoa(*pia), open_sock_max);
 #endif
 		      continue;
 
@@ -579,7 +564,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 		  if (select(imax + 1, NULL, &wfs, NULL, &timeout) > 0)
 		    {
 #if DEBUG > 1
-		      fprintf(stderr, "openvas_tcp_scanner(%s): select! Breaking loop (open_sock_nb=%d / %d)\n", inet_ntoa(*pia), open_sock_nb, open_sock_max);
+		      log_legacy_write ("openvas_tcp_scanner(%s): select! Breaking loop (open_sock_nb=%d / %d)\n", inet_ntoa(*pia), open_sock_nb, open_sock_max);
 #endif
 		      break;
 		    }
@@ -592,7 +577,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	  if (open_sock_nb == 0)
 	    {
 #if DEBUG > 0
-	      fprintf(stderr, "openvas_tcp_scanner(%s): No more open socket\n", inet_ntoa(*pia));
+	      log_legacy_write ("openvas_tcp_scanner(%s): No more open socket\n", inet_ntoa(*pia));
 #endif
 		  goto end;
 	    }
@@ -612,9 +597,6 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 		      FD_SET(sockets[i].fd, &wfs);
 		      break;
 		    default:
-#if 0
-		      fprintf(stderr, "openvas_tcp_scanner(%s): Bad status %d - s=%d\n", inet_ntoa(*pia), sockets[i].state, sockets[i].fd);
-#endif
 		      break;
 		    }
 		  if (sockets[i].fd > imax)
@@ -627,14 +609,14 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	      if (untested_ports_nb > 0)
 		{
 #if DEBUG > 0
-		  fprintf(stderr, "openvas_tcp_scanner(%s): No socket! %d ports remaining\n", inet_ntoa(*pia), untested_ports_nb);
+		  log_legacy_write ("openvas_tcp_scanner(%s): No socket! %d ports remaining\n", inet_ntoa(*pia), untested_ports_nb);
 #endif
 		  return -1;
 		}
 	      else
 		{
 #if DEBUG > 0
-		  fprintf(stderr, "openvas_tcp_scanner(%s): No socket! No port remaining\n", inet_ntoa(*pia));
+		  log_legacy_write ("openvas_tcp_scanner(%s): No socket! No port remaining\n", inet_ntoa(*pia));
 #endif
 		  goto end;
 		}
@@ -657,26 +639,26 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 		  if (em <= moy)
 		    {
 #if DEBUG > 0
-		      fprintf(stderr, "openvas_tcp_scanner: arithmetic overflow: %g -> %d\n", emax, em);
+		      log_legacy_write ("openvas_tcp_scanner: arithmetic overflow: %g -> %d\n", emax, em);
 #endif
 		      em = moy;
 		    }
 		  if (rtt_max[0] > em)
 		    {
 #if DEBUG > 0
-		      fprintf(stderr, "openvas_tcp_scanner(%s): rtt_nb=%d rtt_max = %g > %g (M=%g, SD=%g)\n", inet_ntoa(*pia), rtt_nb[0], (double)rtt_max[0] / 1e6, emax / 1e6, mean / 1e6, sd / 1e6);
+		      log_legacy_write ("openvas_tcp_scanner(%s): rtt_nb=%d rtt_max = %g > %g (M=%g, SD=%g)\n", inet_ntoa(*pia), rtt_nb[0], (double)rtt_max[0] / 1e6, emax / 1e6, mean / 1e6, sd / 1e6);
 #endif
 		      rtt_max[0] = em;
 		    }
 #if DEBUG > 1
 		  else
-		    fprintf(stderr, "openvas_tcp_scanner(%s): rtt_nb=%d rtt_max = %g < %g\n", inet_ntoa(*pia), rtt_nb[0], (double)rtt_max[0] / 1e6, emax / 1e6);
+		    log_legacy_write ("openvas_tcp_scanner(%s): rtt_nb=%d rtt_max = %g < %g\n", inet_ntoa(*pia), rtt_nb[0], (double)rtt_max[0] / 1e6, emax / 1e6);
 #endif
 		}
 	      if (rtt_max[0] < rtt_min[0])
 		{
 #if DEBUG > 0
-		  fprintf(stderr, "openvas_tcp_scanner(%s): absurdly low rtt_max=%g < rtt_min = %g\n", inet_ntoa(*pia), (double)rtt_max[0] / 1e6, (double)rtt_min[0] / 1e6);
+		  log_legacy_write ("openvas_tcp_scanner(%s): absurdly low rtt_max=%g < rtt_min = %g\n", inet_ntoa(*pia), (double)rtt_max[0] / 1e6, (double)rtt_min[0] / 1e6);
 #endif
 		  rtt_max[0] = rtt_min[0];
 		}
@@ -690,18 +672,23 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	  if (wait_sock_nb == 0)
 	    if (rtt_max[0] > 0 || ping_rtt > 0)
 	      {
+#if DEBUG > 1
 		int	y;
+#endif
 		if (rtt_max[0] > 0)
-		  y = x = rtt_max[0];
+		  x = rtt_max[0];
 		else
-		  y = x = ping_rtt;
+		  x = ping_rtt;
+#if DEBUG > 1
+                y = x;
+#endif
 
 		if (doublecheck_flag)
 		  {
 		    x = 3 * x + 20000;
 		    if (x > MAX_SANE_RTT) x = MAX_SANE_RTT;
 #if DEBUG > 1
-		    fprintf(stderr, "openvas_tcp_scanner(%s): basic timeout increased from %g to %g because of \"double check\"\n", inet_ntoa(*pia), y/1e6, x/1e6);
+		    log_legacy_write ("openvas_tcp_scanner(%s): basic timeout increased from %g to %g because of \"double check\"\n", inet_ntoa(*pia), y/1e6, x/1e6);
 #endif
 		  }
 		if (x > 1000000)	/* more that 1 s */
@@ -713,7 +700,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 		timeout.tv_sec = x / 1000000;
 		timeout.tv_usec = x % 1000000;
 #if DEBUG > 2
-		fprintf(stderr, "openvas_tcp_scanner(%s): timeout=%g -> %g\n",
+		log_legacy_write ("openvas_tcp_scanner(%s): timeout=%g -> %g\n",
 			inet_ntoa(*pia), y/1e6, x/1e6);
 #endif
 	      }
@@ -730,9 +717,9 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	    }
 #if DEBUG > 1
 	  if (rtt_max[0] > 0)
-	    fprintf(stderr, "openvas_tcp_scanner(%s): wait_sock_nb=%d - timeout=%u.%06u - RTT=%f/%f/%f/%f\n", inet_ntoa(*pia), wait_sock_nb, timeout.tv_sec, timeout.tv_usec, (double)rtt_min[0] / 1e6, rtt_sum[0] / 1e6 / (rtt_nb[0] > 0 ? rtt_nb[0] : 1), (double)rtt_max[0] / 1e6, (double)cnx_max[0] / 1e6);
+	    log_legacy_write ("openvas_tcp_scanner(%s): wait_sock_nb=%d - timeout=%u.%06u - RTT=%f/%f/%f/%f\n", inet_ntoa(*pia), wait_sock_nb, timeout.tv_sec, timeout.tv_usec, (double)rtt_min[0] / 1e6, rtt_sum[0] / 1e6 / (rtt_nb[0] > 0 ? rtt_nb[0] : 1), (double)rtt_max[0] / 1e6, (double)cnx_max[0] / 1e6);
 	  else
-	    fprintf(stderr, "openvas_tcp_scanner(%s): wait_sock_nb=%d - timeout=%d.%06d\n", inet_ntoa(*pia), wait_sock_nb, timeout.tv_sec, timeout.tv_usec);
+	    log_legacy_write ("openvas_tcp_scanner(%s): wait_sock_nb=%d - timeout=%d.%06d\n", inet_ntoa(*pia), wait_sock_nb, timeout.tv_sec, timeout.tv_usec);
 #endif
 #if DEBUG > 0
 	  gettimeofday(&ti1, NULL);
@@ -743,7 +730,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	      x = select(imax + 1, &rfs, &wfs, NULL, &timeout);
 #if DEBUG > 0
 	      if (errno == EINTR)
-		fprintf(stderr, "openvas_tcp_scanner(%s): select interrupted (i=%d)\n", inet_ntoa(*pia), i);
+		log_legacy_write ("openvas_tcp_scanner(%s): select interrupted (i=%d)\n", inet_ntoa(*pia), i);
 #endif
 	    }
 	  while (i ++ < 10 && x < 0 && errno == EINTR);
@@ -756,7 +743,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	  else if (x == 0)		/* timeout */
 	    {
 #if DEBUG > 1
-	      fprintf(stderr, "openvas_tcp_scanner(%s): select: timeout on all (%d) sockets!\n", inet_ntoa(*pia), imax - 1);
+	      log_legacy_write ("openvas_tcp_scanner(%s): select: timeout on all (%d) sockets!\n", inet_ntoa(*pia), imax - 1);
 #endif
 	      for (i = 0; i < open_sock_nb; i ++)
 		{
@@ -790,7 +777,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	    {
 	      (void) gettimeofday(&ti, NULL);
 #if DEBUG > 1
-	      fprintf(stderr, "openvas_tcp_scanner(%s): select replied in %f s [time=%d.%06d]\n", inet_ntoa(*pia), DIFFTVu(ti, ti1) / 1e6, ti.tv_sec, ti.tv_usec);
+	      log_legacy_write ("openvas_tcp_scanner(%s): select replied in %f s [time=%d.%06d]\n", inet_ntoa(*pia), DIFFTVu(ti, ti1) / 1e6, ti.tv_sec, ti.tv_usec);
 #endif
 	      for (i = 0; i < open_sock_nb; i ++)
 		{
@@ -806,7 +793,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 
 			x = DIFFTVu(ti, sockets[i].tictac);
 #if DEBUG > 2
-			fprintf(stderr, "openvas_tcp_scanner: RTT to %s:%d: %g s\n",
+			log_legacy_write ("openvas_tcp_scanner: RTT to %s:%d: %g s\n",
 				inet_ntoa(*pia), sockets[i].port, x / 1e6);
 #endif
 			if (opt != 0)
@@ -948,7 +935,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	    if (sockets[i].fd >= 0 && DIFFTV(ti, sockets[i].tictac) >= read_timeout)
 	      {
 #if DEBUG > 0
-		fprintf(stderr, "openvas_tcp_scanner(%s): pass #%d: timeout on port %d: %d\n", inet_ntoa(*pia), pass, sockets[i].port, DIFFTV(ti, sockets[i].tictac));
+		log_legacy_write ("openvas_tcp_scanner(%s): pass #%d: timeout on port %d: %d\n", inet_ntoa(*pia), pass, sockets[i].port, DIFFTV(ti, sockets[i].tictac));
 #endif
 		switch(sockets[i].state)
 		  {
@@ -971,21 +958,21 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 		    untested_ports_nb --;
 		    break;
 		  default:
-		    fprintf(stderr, "nesssus_tcp_scanner: Unhandled case %d at %s:%d\n", sockets[i].state, __FILE__, __LINE__);
+		    log_legacy_write ("openvas_tcp_scanner: Unhandled case %d at %s:%d\n", sockets[i].state, __FILE__, __LINE__);
 		    break;
 		  }
 		my_socket_close(sockets[i].fd); sockets[i].fd = -1;
 		sockets[i].state = GRAB_SOCKET_UNUSED;
 	      }
 
-	  x = open_sock_max;
 #if DEBUG > 1
-	  fprintf(stderr, "openvas_tcp_scanner(%s): open_sock_max=%d timeout_nb=%d dropped_nb=%d\n", inet_ntoa(*pia), open_sock_max, timeout_nb, dropped_nb);
+	  x = open_sock_max;
+	  log_legacy_write ("openvas_tcp_scanner(%s): open_sock_max=%d timeout_nb=%d dropped_nb=%d\n", inet_ntoa(*pia), open_sock_max, timeout_nb, dropped_nb);
 	  done_ports_nb = unfiltered_ports_nb + filtered_ports_nb;
 	  if (done_ports_nb > 0 && total_ports_nb > 0)
 	    {
 	      int	dt = time(NULL) - start_time_1pass;
-	      fprintf(stderr, "openvas_tcp_scanner(%s): pass #%d: time spent so far = %d s - estimated total time = %d s - estimated time remaining = %d s\n",
+	      log_legacy_write ("openvas_tcp_scanner(%s): pass #%d: time spent so far = %d s - estimated total time = %d s - estimated time remaining = %d s\n",
 		      inet_ntoa(*pia),  pass,
 		      dt,
 		      dt * total_ports_nb / done_ports_nb,
@@ -999,7 +986,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	    {
 	      /* firewalled machine? */
 #if DEBUG > 1
-	      fprintf(stderr, "openvas_tcp_scanner(%s): %d connections dropped. Firewall?\n", inet_ntoa(*pia), dropped_nb);
+	      log_legacy_write ("openvas_tcp_scanner(%s): %d connections dropped. Firewall?\n", inet_ntoa(*pia), dropped_nb);
 #endif
 	      open_sock_max += dropped_nb;
 	      if (open_sock_max2 < max_cnx) open_sock_max2 ++;
@@ -1022,7 +1009,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 #endif
 #if DEBUG > 0
 	      if (min_cnx < open_sock_max)
-		fprintf(stderr, "openvas_tcp_scanner(%s): %d connections dropped. Slowing down - min_cnx=%d - open_sock_nb=%d - open_sock_max=%d - open_sock_max2=%d\n", inet_ntoa(*pia), dropped_nb, min_cnx, open_sock_nb, open_sock_max, open_sock_max2);
+		log_legacy_write ("openvas_tcp_scanner(%s): %d connections dropped. Slowing down - min_cnx=%d - open_sock_nb=%d - open_sock_max=%d - open_sock_max2=%d\n", inet_ntoa(*pia), dropped_nb, min_cnx, open_sock_nb, open_sock_max, open_sock_max2);
 #endif
 	    }
 	  else if (dropped_nb == 0 && dropped_flag)
@@ -1037,7 +1024,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	  if (open_sock_max > open_sock_max2)
 	    {
 #if  DEBUG > 2
-	      fprintf(stderr, "openvas_tcp_scanner(%s): open_sock_max=%d > %d\n",
+	      log_legacy_write ("openvas_tcp_scanner(%s): open_sock_max=%d > %d\n",
 		      inet_ntoa(*pia), open_sock_max, open_sock_max2);
 #endif
 	      open_sock_max = open_sock_max2;
@@ -1045,14 +1032,14 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	  if (open_sock_max < min_cnx)
 	    {
 #if  DEBUG > 2
-	      fprintf(stderr, "openvas_tcp_scanner(%s): open_sock_max=%d < %d\n",
+	      log_legacy_write ("openvas_tcp_scanner(%s): open_sock_max=%d < %d\n",
 		      inet_ntoa(*pia), open_sock_max, min_cnx);
 #endif
 	      open_sock_max = min_cnx;
 	    }
 #if DEBUG > 1
 	  if (x != open_sock_max)
-	    fprintf(stderr, "openvas_tcp_scanner(%s): open_sock_max=%d (old value %d)\n", inet_ntoa(*pia), open_sock_max, x);
+	    log_legacy_write ("openvas_tcp_scanner(%s): open_sock_max=%d (old value %d)\n", inet_ntoa(*pia), open_sock_max, x);
 #endif
 	  for (i = 0; i < open_sock_nb; )
 	    if (sockets[i].state == GRAB_SOCKET_UNUSED || sockets[i].fd < 0)
@@ -1074,7 +1061,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
       diff_time1 = end_time - start_time_1pass;
       diff_time = end_time - start_time;
 #if DEBUG > 0
-      fprintf(stderr, "openvas_tcp_scanner(%s): pass #%d ran in %d s - filtered_ports_nb=%d closed_ports_nb=%d open_ports_nb=%d\n", inet_ntoa(*pia), pass, diff_time1, filtered_ports_nb, closed_ports_nb, open_ports_nb);
+      log_legacy_write ("openvas_tcp_scanner(%s): pass #%d ran in %d s - filtered_ports_nb=%d closed_ports_nb=%d open_ports_nb=%d\n", inet_ntoa(*pia), pass, diff_time1, filtered_ports_nb, closed_ports_nb, open_ports_nb);
 #endif
       if (dropped_flag ||
 	  (pass == 1 && filtered_ports_nb > 10 && closed_ports_nb > 10) ||
@@ -1083,7 +1070,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	  if (doublecheck_flag && rst_rate_limit_flag && open_ports_nb == old_opened)
 	    {
 #if DEBUG > 0
-	      fprintf(stderr, "openvas_tcp_scanner(%s): Same number of open ports! Stopping now\n", inet_ntoa(*pia));
+	      log_legacy_write ("openvas_tcp_scanner(%s): Same number of open ports! Stopping now\n", inet_ntoa(*pia));
 #endif
 	      break;
 	    }
@@ -1091,12 +1078,12 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 
 	  doublecheck_flag = 0;
 #if DEBUG > 0
-	  fprintf(stderr, "openvas_tcp_scanner(%s): pass #%d: Suspicious number of filtered ports (%d) or closed ports (%d) - running another time\n", inet_ntoa(*pia), pass, filtered_ports_nb, closed_ports_nb);
+	  log_legacy_write ("openvas_tcp_scanner(%s): pass #%d: Suspicious number of filtered ports (%d) or closed ports (%d) - running another time\n", inet_ntoa(*pia), pass, filtered_ports_nb, closed_ports_nb);
 #endif
 	  if (filtered_ports_nb == old_filtered)
 	    {
 #if DEBUG > 0
-	      fprintf(stderr, "openvas_tcp_scanner(%s): Same number of filtered ports! Stopping now\n", inet_ntoa(*pia));
+	      log_legacy_write ("openvas_tcp_scanner(%s): Same number of filtered ports! Stopping now\n", inet_ntoa(*pia));
 #endif
 	      break;
 	    }
@@ -1123,13 +1110,13 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 		  break_flag = 0;
 		}
 #if DEBUG > 0
-	      fprintf(stderr, "openvas_tcp_scanner(%s): system seems to be limiting RST rate - %s - min_cnx=%d - closed_ports_nb1=%d - diff_time1=%d - closed_ports_nb=%d - diff_time=%d\n", inet_ntoa(*pia), break_flag ? "Stopping immediately" : doublecheck_flag ? "Double checking standard ports" : "Running one last pass", min_cnx, closed_ports_nb1, diff_time1, closed_ports_nb, diff_time);
+	      log_legacy_write ("openvas_tcp_scanner(%s): system seems to be limiting RST rate - %s - min_cnx=%d - closed_ports_nb1=%d - diff_time1=%d - closed_ports_nb=%d - diff_time=%d\n", inet_ntoa(*pia), break_flag ? "Stopping immediately" : doublecheck_flag ? "Double checking standard ports" : "Running one last pass", min_cnx, closed_ports_nb1, diff_time1, closed_ports_nb, diff_time);
 #endif
 	      rst_rate_limit_flag ++ ;
 	      if (break_flag) break;
 	    }
 #if DEBUG > 1
-	  fprintf(stderr, "openvas_tcp_scanner(%s): min_cnx=%d - open_ports_nb1=%d - closed_ports_nb1=%d - diff_time1=%d - closed_ports_nb=%d - diff_time=%d\n", inet_ntoa(*pia), min_cnx, open_ports_nb1, closed_ports_nb1, diff_time1, closed_ports_nb, diff_time);
+	  log_legacy_write ("openvas_tcp_scanner(%s): min_cnx=%d - open_ports_nb1=%d - closed_ports_nb1=%d - diff_time1=%d - closed_ports_nb=%d - diff_time=%d\n", inet_ntoa(*pia), min_cnx, open_ports_nb1, closed_ports_nb1, diff_time1, closed_ports_nb, diff_time);
 #endif
 
 	  /*
@@ -1145,7 +1132,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	      }
 #if DEBUG > 1
 	  if (old_filtered != filtered_ports_nb)
-	    fprintf(stderr, "openvas_tcp_scanner(%s): old_filtered=%d filtered_ports_nb=%d\n", inet_ntoa(*pia), old_filtered, filtered_ports_nb);
+	    log_legacy_write ("openvas_tcp_scanner(%s): old_filtered=%d filtered_ports_nb=%d\n", inet_ntoa(*pia), old_filtered, filtered_ports_nb);
 #endif
 	  untested_ports_nb = old_filtered;
 	  filtered_ports_nb = 0;
@@ -1177,13 +1164,13 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	  if ((tbd_nb = double_check_std_ports(ports_states)) == 0)
 	    {
 #if DEBUG > 0
-	      fprintf(stderr, "openvas_tcp_scanner(%s): pass #%d - No filtered standard ports - stopping\n", inet_ntoa(*pia), pass);
+	      log_legacy_write ("openvas_tcp_scanner(%s): pass #%d - No filtered standard ports - stopping\n", inet_ntoa(*pia), pass);
 #endif
 	      break;
 	    }
 #if DEBUG > 0
 	  else
-	    fprintf(stderr, "openvas_tcp_scanner(%s): pass #%d - Double checking %d standard ports\n", inet_ntoa(*pia), pass, tbd_nb);
+	    log_legacy_write ("openvas_tcp_scanner(%s): pass #%d - Double checking %d standard ports\n", inet_ntoa(*pia), pass, tbd_nb);
 #endif
 	  old_filtered = untested_ports_nb = tbd_nb;
 	  filtered_ports_nb = 0;
@@ -1203,7 +1190,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
   if (pass > MAX_PASS_NB)
     {
       pass --;
-      /*fprintf(stderr, "openvas_tcp_scanner(%s): gave up after %d pass\n",
+      /*log_legacy_write ("openvas_tcp_scanner(%s): gave up after %d pass\n",
 	      inet_ntoa(*pia), pass);*/
       filtered_ports_nb = old_filtered;
     }
@@ -1211,7 +1198,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
   plug_set_key(desc, "TCPScanner/NbPasses", ARG_INT, GSIZE_TO_POINTER(pass));
 
 #if DEBUG > 0
-  fprintf(stderr, "openvas_tcp_scanner(%s): ran in %d pass(es) in %d s - min_cnx=%d max_cnx=%d read_timeout=%d - open_ports_nb=%d closed_ports_nb=%d filtered_ports_nb=%d - rtt_min=%f rtt_max=%f cnx_max=%f\n", inet_ntoa(*pia), pass, diff_time, min_cnx, max_cnx, read_timeout, open_ports_nb, closed_ports_nb, filtered_ports_nb, rtt_min[0] / 1e6, rtt_max[0] / 1e6, cnx_max[0] / 1e6);
+  log_legacy_write ("openvas_tcp_scanner(%s): ran in %d pass(es) in %d s - min_cnx=%d max_cnx=%d read_timeout=%d - open_ports_nb=%d closed_ports_nb=%d filtered_ports_nb=%d - rtt_min=%f rtt_max=%f cnx_max=%f\n", inet_ntoa(*pia), pass, diff_time, min_cnx, max_cnx, read_timeout, open_ports_nb, closed_ports_nb, filtered_ports_nb, rtt_min[0] / 1e6, rtt_max[0] / 1e6, cnx_max[0] / 1e6);
 #endif
 
 #if defined COMPUTE_RTT
@@ -1260,7 +1247,7 @@ banner_grab(const struct in6_addr *pia, const char* portrange,
 	  }
 #if DEBUG > 0
 	if (rtt_nb[i] > 0)
-	  fprintf(stderr, "openvas_tcp_scanner: Mean RTT to %s = %g - [%g, %g] - SD = %g - +3SD = %g [%d %s ports]\n",
+	  log_legacy_write ("openvas_tcp_scanner: Mean RTT to %s = %g - [%g, %g] - SD = %g - +3SD = %g [%d %s ports]\n",
 		  inet_ntoa(*pia), mean,
 		  rtt_min[i] / 1e6, cnx_max[i] / 1e6,
 		  sd, emax, rtt_nb[i], rtt_type[i]);
@@ -1282,23 +1269,19 @@ tree_cell *
 plugin_run_openvas_tcp_scanner (lex_ctxt * lexic)
 {
   struct arglist *desc = lexic->script_infos;
-  struct arglist * globals = arg_get_value(desc, "globals");
-  struct arglist * preferences = arg_get_value(desc, "preferences");
   struct arglist * hostinfos = arg_get_value(desc, "HOSTNAME");
-  char * port_range = arg_get_value(preferences, "port_range");
-  char * p;
+  const char * port_range = prefs_get ("port_range");
+  const char * p;
   struct in6_addr *p_addr;
-  int	timeout = 0, max_cnx, min_cnx, safe_checks = 0, x;
+  int	timeout = 0, max_cnx, min_cnx, x;
+  int	safe_checks = prefs_get_bool ("safe_checks");
 
-  p = arg_get_value(preferences, "safe_checks");
-  if (p != NULL && strcmp(p, "yes") == 0) safe_checks = 1;
-
-  p =  arg_get_value(preferences, "checks_read_timeout");
+  p = prefs_get ("checks_read_timeout");
   if (p != NULL) timeout = atoi(p);
   if (timeout <= 0)
     timeout = 5;
 #if DEBUG > 0
-  fprintf(stderr, "openvas_tcp_scanner: safe_checks=%d checks_read_timeout=%d\n", safe_checks, timeout);
+  log_legacy_write ("openvas_tcp_scanner: safe_checks=%d checks_read_timeout=%d\n", safe_checks, timeout);
 #endif
 
   {
@@ -1314,17 +1297,17 @@ plugin_run_openvas_tcp_scanner (lex_ctxt * lexic)
     dup2(devnull_fd, 2);
 #endif
 
-    p = arg_get_value(preferences, "max_hosts");
+    p = prefs_get ("max_hosts");
     if (p != NULL) max_host = atoi(p);
     if (max_host <= 0) max_host = 15;
 
-    p = arg_get_value(preferences, "max_checks");
+    p = prefs_get ("max_checks");
     if (p != NULL) max_checks = atoi(p);
     if (max_checks <= 0 || max_checks > 5)
       {
 	max_checks = 5; /* bigger values do not make sense */
 #if DEBUG > 0
-	fprintf(stderr, "openvas_tcp_scanner: max_checks forced to %d\n", max_checks);
+	log_legacy_write ("openvas_tcp_scanner: max_checks forced to %d\n", max_checks);
 #endif
       }
 
@@ -1399,14 +1382,14 @@ plugin_run_openvas_tcp_scanner (lex_ctxt * lexic)
 	max_cnx /= (1.0 + maxloadavg);
 #if DEBUG > 0
 	/* Useless, as stderr is temporarily closed */
-	fprintf(stderr, "openvas_tcp_scanner: max_cnx reduced from %d to %d because of maxloadavg=%f\n", x, max_cnx, maxloadavg);
+	log_legacy_write ("openvas_tcp_scanner: max_cnx reduced from %d to %d because of maxloadavg=%f\n", x, max_cnx, maxloadavg);
 #endif
       }
 
 
 
 #if DEBUG > 0
-    fprintf(stderr, "openvas_tcp_scanner: max_sys_fd=%d\n", max_sys_fd);
+    log_legacy_write ("openvas_tcp_scanner: max_sys_fd=%d\n", max_sys_fd);
 #endif
     if (max_sys_fd <= 0) max_sys_fd = 16384; /* reasonable default */
     /* Let's leave at least 1024 FD for other processes */
@@ -1419,7 +1402,7 @@ plugin_run_openvas_tcp_scanner (lex_ctxt * lexic)
       }
     if (max_cnx > x) max_cnx = x;
 #if 0
-    fprintf(stderr, "min_cnx = %d ; max_cnx = %d\n", min_cnx, max_cnx);
+    log_legacy_write ("min_cnx = %d ; max_cnx = %d\n", min_cnx, max_cnx);
 #endif
     if (max_cnx > GRAB_MAX_SOCK) max_cnx = GRAB_MAX_SOCK;
     if (max_cnx < GRAB_MIN_SOCK) max_cnx = GRAB_MIN_SOCK;
@@ -1438,16 +1421,15 @@ plugin_run_openvas_tcp_scanner (lex_ctxt * lexic)
     x = max_cnx / 2;
     if (min_cnx > x) min_cnx = x > 0 ? x : 1;
 #if DEBUG > 0
-    fprintf(stderr, "openvas_tcp_scanner: min_cnx = %d ; max_cnx = %d\n", min_cnx, max_cnx);
+    log_legacy_write ("openvas_tcp_scanner: min_cnx = %d ; max_cnx = %d\n", min_cnx, max_cnx);
 #endif
   }
 
   p_addr = arg_get_value(hostinfos, "IP");
   if( p_addr == NULL )
     return NULL; // TODO: before it returned "1";
-  if (banner_grab(p_addr, port_range, timeout, min_cnx, max_cnx, globals, desc, hostinfos) < 0)
+  if (banner_grab(p_addr, port_range, timeout, min_cnx, max_cnx, desc) < 0)
     return NULL; // TODO: before it returned "1";
-  comm_send_status(globals, arg_get_value(hostinfos, "NAME"),"portscan", 65535, 65535);
   plug_set_key(desc, "Host/scanned", ARG_INT, (void*)1);
   plug_set_key(desc, "Host/scanners/openvas_tcp_scanner", ARG_INT, (void*)1);
   return NULL;
