@@ -24,7 +24,10 @@
 *
 */
 
+/* legacy feature macro */
 #define _BSD_SOURCE 1
+/* new feature macros that provides the same plus more */
+#define _DEFAULT_SOURCE 1
 
 #include <unistd.h>      /* for close() */
 #include <stdlib.h>      /* for rand() */
@@ -38,8 +41,9 @@
 #include "nvt_categories.h" /* for ACT_SCANNER */
 #include "../misc/pcap_openvas.h" /* for get_datalink_size */
 #include "../misc/plugutils.h" /* for scanner_add_port */
-#include "../misc/scanners_utils.h" /* for getpts */
-#include "../misc/system.h" /* for efree */
+#include "../misc/openvas_logging.h"
+#include "../misc/prefs.h"          /* for prefs_get */
+#include "network.h"    /* for getpts */
 
 #include "nasl_lex_ctxt.h"
 
@@ -158,13 +162,12 @@ packetdead (unsigned long then, unsigned long rtt)
 
 	then = ntohl(then);
 	now = ntohl(now);
-	rtt = ntohl(rtt);
 
 	if ((now - then) >= 2 << 28 ) {
 		return 1;
-	} else {
-		return 0;
 	}
+
+	return 0;
 }
 
 
@@ -307,7 +310,7 @@ add_packet (struct list * l, unsigned short dport, unsigned long ack)
 		ret->when = ack;
 		return l;
 	}
-	ret = emalloc (sizeof (struct list));
+	ret = g_malloc0 (sizeof (struct list));
 
 	ret->next = l;
 	ret->prev = NULL;
@@ -330,9 +333,9 @@ rm_packet (struct list * l, unsigned short dport)
 	if (p == NULL)
 	{
 #if DEBUG > 1
-		fprintf (stderr, "Odd - no entry for %d - RTT too low ?!\n", dport);
+          log_legacy_write ("Odd - no entry for %d - RTT too low ?!", dport);
 #endif
-		return l;
+          return l;
 	}
 	if (p->next != NULL)
 		p->next->prev = p->prev;
@@ -342,7 +345,7 @@ rm_packet (struct list * l, unsigned short dport)
 	else
 		ret = p->next;
 
-	efree(&p);
+	g_free(p);
 	return ret;
 }
 
@@ -373,7 +376,7 @@ rm_dead_packets (struct list * l, unsigned long rtt, int *retry)
 					p->prev->next = p->next;
 				else
 					ret = p->next;
-				efree(&p);
+				g_free(p);
 			}
 		}
 		p = next;
@@ -528,135 +531,6 @@ mktcpv6 (struct in6_addr *src, int sport, struct in6_addr *dst, int dport,
 }
 /*--------------------------------------------------------------------*/
 
-int
-find_rtt (struct in_addr dst, unsigned long *rtt)
-{
-	int             soc;
-	unsigned short  ports[] = {21, 22, 34, 25, 53, 79, 80, 110, 113, 135, 139, 143, 264, 389, 443, 993, 1454, 1723, 3389, 8080, 0};
-	unsigned short  use[3];
-	int             num = 0;
-	int             n;
-	int             i;
-	int             bpf;
-	int             magic = 4441 + (rand() % 1200);
-	struct sockaddr_in soca;
-	int             len;
-	int             skip;
-	struct in_addr  src;
-	int             j;
-	unsigned long   max, max_max;
-	int             err = 0;
-	int             noresend = 0;
-
-
-	soc = rawsocket(AF_INET);
-	if (soc < 0)
-		return -1;
-
-	bpf = openbpf(dst, &src, magic);
-
-	if (bpf < 0) {
-		close(soc);
-		return -1;
-	}
-	skip = get_datalink_size(bpf_datalink(bpf));
-	bzero(&soca, sizeof (soca));
-	soca.sin_family = AF_INET;
-	soca.sin_addr = dst;
-
-	for (i = 0; ports[i] != 0; i++) {
-		char           *res;
-		unsigned long   ack = maketime();
-		char           *pkt = mktcp(src, magic, dst, ports[i], ack, TH_SYN);
-		int             e;
-		struct timeval  tv = {1, 0};
-		unsigned short  p = ports[i];
-
-#if DEBUG > 1
-		printf("send to port %d\n", ports[i]);
-#endif
-
-		e = sendto(soc, pkt, sizeof (struct ip) + sizeof (struct tcphdr), 0, (struct sockaddr *) & soca, sizeof (soca));
-		if (e < 0) {
-			perror("sendto ");
-			close(soc);
-			bpf_close(bpf);
-			return -1;
-		}
-		res = (char *) bpf_next_tv(bpf, &len, &tv);
-		if (res != NULL) {
-#if DEBUG > 1
-			printf("Found port %d\n", p);
-#endif
-			use[num++] = p;
-			if (num >= 3)
-				break;
-		}
-	}
-
-	if (num == 0) {
-#if DEBUG > 1
-		printf("Found nothing\n");
-#endif
-		bpf_close(bpf);
-		close(soc);
-		*rtt = htonl(1 << 28);	/* One second */
-		return 0;
-	}
-	max = max_max = 0;
-
-	for (j = 0, n = 0; j < 10; j++, n++) {
-		char           *res;
-		unsigned long   ack = maketime();
-		char           *pkt;
-		int             e;
-		struct timeval  tv = {1, 0};
-
-
-		pkt = mktcp(src, magic, dst, use[n % num], ack, TH_SYN);
-		e = sendto(soc, pkt, sizeof (struct ip) + sizeof (struct tcphdr), 0, (struct sockaddr *) & soca, sizeof (soca));
-		if (e < 0)
-			perror("sendto ");
-		res = (char *) bpf_next_tv(bpf, &len, &tv);
-
-
-		if (res != NULL) {
-			unsigned long   val = compute_rtt(extractack(res + skip, len, AF_INET));
-			noresend = 0;
-			if (val && val > max_max) {
-				if (max != 0) {
-					if (val < max * 2) {
-						max = max_max;
-						max_max = val;
-					}
-				} else {
-					max = max_max;
-					max_max = val;
-				}
-			}
-		} else {
-#if DEBUG > 1
-			printf("No reply ?!\n");
-#endif
-			j--;
-			err++;
-			noresend++;
-			if (noresend > 4)
-				noresend = 0;
-			if (err > 10) {
-				*rtt = htonl(1 << 28);
-				return 0;
-			}
-		}
-	}
-
-	close(soc);
-	bpf_close(bpf);
-        if(max == 0) max = htonl(1 << 28);
-	*rtt = max;
-	return 1;
-}
-
 /**
  * @param sniff If != 0, "sniff" (listen to incoming packages), else just
  *              add packet.
@@ -763,7 +637,7 @@ v6_sendpacket (int soc, int bpf, int skip, struct in6_addr *dst,
     packets = add_packet(packets, dport, ack);
     e = sendto(soc, pkt,sizeof(struct tcphdr), 0, (struct sockaddr *) & soca, sizeof(soca));
     if (e < 0) {
-      fprintf(stderr,"sendto error in v6_sendpacket\n");
+      log_legacy_write ("sendto error in v6_sendpacket");
       perror("sendto ");
       close(soc);
       bpf_close(bpf);
@@ -809,7 +683,6 @@ scan (struct arglist * env, char* hostname, char* portrange,
   int             skip;
   int             i;
   struct list    *packets = NULL;
-  struct arglist *globals = (env) ? arg_get_value (env, "globals") : NULL;
   int             retry;
   unsigned short *ports;
   int family;
@@ -834,7 +707,7 @@ scan (struct arglist * env, char* hostname, char* portrange,
   printf ("===> Target = %s\n", inet_ntoa (dst));
 #endif
 
-  ports = (unsigned short *) getpts (portrange ? portrange : "default", &num);
+  ports = (unsigned short *) getpts (portrange, &num);
 
   if (soc < 0)
     {
@@ -854,9 +727,6 @@ scan (struct arglist * env, char* hostname, char* portrange,
 #ifdef DEBUG
       printf ("====> Sending packet to (at least) %d\n", ports[i]);
 #endif
-      if (i % 100 == 0)
-        comm_send_status (globals, hostname, "portscan", i, num);
-
       if (family == AF_INET)
         packets = sendpacket (soc, bpf, skip, dst, src, ports[i], magic,
                               packets, &rtt, 0, env);
@@ -904,14 +774,10 @@ scan (struct arglist * env, char* hostname, char* portrange,
       }
   }
 
-  comm_send_status (globals, hostname, "portscan", num, num);
-#if 0
-  plug_set_key (env, "Host/num_ports_scanned", ARG_INT, (void*)num);
-#endif
   close (soc);
   bpf_close (bpf);
   if (ports != NULL)
-    efree (&ports);
+    g_free (ports);
   if (num >= 65535)
     plug_set_key (env, "Host/full_scan", ARG_INT, (void*) 1);
 
@@ -927,7 +793,9 @@ plugin_run_synscan (lex_ctxt * lexic)
   struct in6_addr *dst6 = plug_get_host_ip (env);
   struct in_addr  *dst;
   struct in_addr  inaddr;
+#ifdef DEBUG
   struct timeval  tv;
+#endif
 
   inaddr.s_addr = dst6->s6_addr32[3];
   dst = &inaddr;
@@ -935,29 +803,18 @@ plugin_run_synscan (lex_ctxt * lexic)
   if (islocalhost (dst))
     return NULL;
 
-#if 0
-	if (find_rtt(*dst, &rtt) < 0) {
-		fprintf(stderr, "Something went wrong, bailing out\n");
-		return NULL;  // TODO: before it returned "1";
-	}
-#endif
     rtt = htonl (1 << 28);
 
 #ifdef DEBUG
-    printf ("RTT = 0x%.8x\n", ntohl (rtt));
-#endif
-
-  /** @TODO belongs to debug section? */
+  printf ("RTT = 0x%.8x\n", ntohl (rtt));
   tv = timeval (rtt);
-
-#ifdef DEBUG
-	printf ("That's %ld seconds and %ld usecs\n", tv.tv_sec, tv.tv_usec);
+  printf ("That's %ld seconds and %ld usecs\n", tv.tv_sec, tv.tv_usec);
 #endif
 
   struct arglist *hostinfos = arg_get_value (env, "HOSTNAME");
   char           *hostname  = arg_get_value (hostinfos, "NAME");
-  char           *range = get_preference (env, "port_range");
-  scan (env, hostname, range, dst6, rtt);
+  const char     *range = prefs_get ("port_range");
+  scan (env, hostname, (char *)range, dst6, rtt);
   plug_set_key (env, "Host/scanned", ARG_INT, (void *) 1);
   plug_set_key (env, "Host/scanners/synscan", ARG_INT, (void*)1);
   return NULL;
