@@ -1,4 +1,4 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2004 Tenable Network Security
  *
@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
@@ -34,16 +34,13 @@
 
 #include "plugutils.h"          /* for find_in_path */
 #include "popen.h"              /* for openvas_popen4 */
-#include "system.h"             /* for erealloc */
 
 #include "nasl_tree.h"
 #include "nasl_global_ctxt.h"
 #include "nasl_func.h"
 #include "nasl_var.h"
 #include "nasl_lex_ctxt.h"
-#include "exec.h"
 
-#include "strutils.h"
 #include "nasl_cmd_exec.h"
 #include "nasl_debug.h"
 
@@ -76,9 +73,6 @@ nasl_pread (lex_ctxt * lexic)
   FILE *fp;
   char cwd[MAXPATHLEN], newdir[MAXPATHLEN];
 
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
   if (pid != 0)
     {
       nasl_perror (lexic, "nasl_pread is not reentrant!\n");
@@ -89,9 +83,11 @@ nasl_pread (lex_ctxt * lexic)
   cmd = get_str_local_var_by_name (lexic, "cmd");
   if (cmd == NULL || a == NULL || (v = a->x.ref_val) == NULL)
     {
+      deref_cell (a);
       nasl_perror (lexic, "pread() usage: cmd:..., argv:...\n");
       return NULL;
     }
+  deref_cell (a);
 
   nice = get_int_local_var_by_name (lexic, "nice", 0);
 
@@ -160,12 +156,12 @@ nasl_pread (lex_ctxt * lexic)
   if (av->hash_elt != NULL)
     nasl_perror (lexic, "pread: named elements in 'cmd' are ignored!\n");
   n = av->max_idx;
-  args = emalloc (sizeof (char **) * (n + 2));  /* Last arg is NULL */
+  args = g_malloc0 (sizeof (char **) * (n + 2));  /* Last arg is NULL */
   for (j = 0, i = 0; i < n; i++)
     {
       str = (char *) var2str (av->num_elt[i]);
       if (str != NULL)
-        args[j++] = estrdup (str);
+        args[j++] = g_strdup (str);
     }
   args[j++] = NULL;
 
@@ -176,13 +172,13 @@ nasl_pread (lex_ctxt * lexic)
   fp = openvas_popen4 ((const char *) cmd, args, &pid, nice);
 
   for (i = 0; i < n; i++)
-    efree (&args[i]);
-  efree (&args);
+    g_free (args[i]);
+  g_free (args);
 
   if (fp != NULL)
     {
       sz = 0;
-      str = emalloc (1);
+      str = g_malloc0 (1);
 
       errno = 0;
       while ((n = fread (buf, 1, sizeof (buf), fp)) > 0 || errno == EINTR)      /* && kill(pid, 0) >= 0) */
@@ -193,12 +189,7 @@ nasl_pread (lex_ctxt * lexic)
               continue;
             }
           sz2 = sz + n;
-          str2 = erealloc (str, sz2);
-          if (str2 == NULL)
-            {
-              nasl_perror (lexic, "nasl_pread: erealloc failed\n");
-              break;
-            }
+          str2 = g_realloc (str, sz2);
           str = str2;
           memcpy (str + sz, buf, n);
           sz = sz2;
@@ -232,11 +223,6 @@ nasl_find_in_path (lex_ctxt * lexic)
   tree_cell *retc;
   char *cmd;
 
-
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
-
   cmd = get_str_var_by_num (lexic, 0);
   if (cmd == NULL)
     {
@@ -253,8 +239,8 @@ nasl_find_in_path (lex_ctxt * lexic)
  * Not a command, but dangerous anyway
  */
 /**
- * @brief Read file if check_authenticated.
- * @ingroup nasl_implement 
+ * @brief Read file
+ * @ingroup nasl_implement
  */
 tree_cell *
 nasl_fread (lex_ctxt * lexic)
@@ -267,9 +253,6 @@ nasl_fread (lex_ctxt * lexic)
   int alen, len, n;
   FILE *fp;
 
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
   fname = get_str_var_by_num (lexic, 0);
   if (fname == NULL)
     {
@@ -329,13 +312,7 @@ nasl_fread (lex_ctxt * lexic)
     }
 
   alen = lstat_info.st_size + 1;
-  buf = emalloc (alen);
-  if (buf == NULL)
-    {
-      nasl_perror (lexic, "fread: cannot malloc %d bytes\n", alen);
-      goto error;
-    }
-
+  buf = g_malloc0 (alen);
   len = 0;
   while ((n = fread (buf + len, 1, alen - len, fp)) > 0)
     {
@@ -343,12 +320,7 @@ nasl_fread (lex_ctxt * lexic)
       if (alen <= len)
         {
           alen += 4096;
-          p = erealloc (buf, alen);
-          if (p == NULL)
-            {
-              nasl_perror (lexic, "fread: cannot realloc %d bytes\n", alen);
-              goto error;
-            }
+          p = g_realloc (buf, alen);
           buf = p;
         }
     }
@@ -356,9 +328,8 @@ nasl_fread (lex_ctxt * lexic)
   buf[len] = '\0';
   if (alen > len + 1)
     {
-      p = erealloc (buf, len + 1);
-      if (p != NULL)
-        buf = p;
+      p = g_realloc (buf, len + 1);
+      buf = p;
     }
 
   retc = alloc_typed_cell (CONST_DATA);
@@ -366,18 +337,13 @@ nasl_fread (lex_ctxt * lexic)
   retc->x.str_val = buf;
   fclose (fp);
   return retc;
-
-error:
-  efree (&buf);
-  fclose (fp);
-  return NULL;
 }
 
 /*
  * Not a command, but dangerous anyway
  */
 /**
- * @brief Unlink file if check_authenticated.
+ * @brief Unlink file
  * @ingroup nasl_implement
  */
 tree_cell *
@@ -385,9 +351,6 @@ nasl_unlink (lex_ctxt * lexic)
 {
   char *fname;
 
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
   fname = get_str_var_by_num (lexic, 0);
   if (fname == NULL)
     {
@@ -406,7 +369,7 @@ nasl_unlink (lex_ctxt * lexic)
 
 /* Definitely dangerous too */
 /**
- * @brief Write file if check_authenticated.
+ * @brief Write file
  */
 tree_cell *
 nasl_fwrite (lex_ctxt * lexic)
@@ -418,13 +381,6 @@ nasl_fwrite (lex_ctxt * lexic)
   int len, i, x;
   FILE *fp;
 
-  if (check_authenticated (lexic) < 0)
-    {
-      nasl_perror (lexic,
-                   "fwrite may only be called by an authenticated script\n");
-      return NULL;
-    }
-
   content = get_str_local_var_by_name (lexic, "data");
   fname = get_str_local_var_by_name (lexic, "file");
   if (content == NULL || fname == NULL)
@@ -524,9 +480,6 @@ nasl_get_tmp_dir (lex_ctxt * lexic)
   tree_cell *retc;
   char path[MAXPATHLEN];
 
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
   snprintf (path, sizeof (path), "%s/", g_get_tmp_dir ());
   if (access (path, R_OK | W_OK | X_OK) < 0)
     {
@@ -549,7 +502,7 @@ nasl_get_tmp_dir (lex_ctxt * lexic)
  */
 
 /**
- * @brief Stat file if check_authenticated.
+ * @brief Stat file
  * @ingroup nasl_implement
  */
 tree_cell *
@@ -559,9 +512,6 @@ nasl_file_stat (lex_ctxt * lexic)
   char *fname;
   struct stat st;
 
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
   fname = get_str_var_by_num (lexic, 0);
   if (fname == NULL)
     {
@@ -578,7 +528,7 @@ nasl_file_stat (lex_ctxt * lexic)
 }
 
 /**
- * @brief Open file if check_authenticated.
+ * @brief Open file
  */
 tree_cell *
 nasl_file_open (lex_ctxt * lexic)
@@ -589,9 +539,6 @@ nasl_file_open (lex_ctxt * lexic)
   int fd;
   int imode = O_RDONLY;
 
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
   fname = get_str_local_var_by_name (lexic, "name");
   if (fname == NULL)
     {
@@ -667,7 +614,7 @@ nasl_file_open (lex_ctxt * lexic)
 }
 
 /**
- * @brief Close file if check_authenticated.
+ * @brief Close file
  */
 tree_cell *
 nasl_file_close (lex_ctxt * lexic)
@@ -675,9 +622,6 @@ nasl_file_close (lex_ctxt * lexic)
   tree_cell *retc;
   int fd;
 
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
   fd = get_int_var_by_num (lexic, 0, -1);
   if (fd < 0)
     {
@@ -698,7 +642,7 @@ nasl_file_close (lex_ctxt * lexic)
 
 
 /**
- * @brief Read file if check_authenticated.
+ * @brief Read file
  */
 tree_cell *
 nasl_file_read (lex_ctxt * lexic)
@@ -709,10 +653,6 @@ nasl_file_read (lex_ctxt * lexic)
   int flength;
   int n;
 
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
-
   fd = get_int_local_var_by_name (lexic, "fp", -1);
   if (fd < 0)
     {
@@ -722,12 +662,7 @@ nasl_file_read (lex_ctxt * lexic)
 
   flength = get_int_local_var_by_name (lexic, "length", 0);
 
-  buf = emalloc (flength + 1);
-  if (buf == NULL)
-    {
-      nasl_perror (lexic, "file_read: cannot malloc %d bytes\n", flength);
-      goto error;
-    }
+  buf = g_malloc0 (flength + 1);
 
   for (n = 0; n < flength;)
     {
@@ -746,14 +681,11 @@ nasl_file_read (lex_ctxt * lexic)
   retc->size = n;
   retc->x.str_val = buf;
   return retc;
-error:
-  efree (&buf);
-  return NULL;
 }
 
 
 /**
- * @brief Write file if check_authenticated.
+ * @brief Write file
  */
 tree_cell *
 nasl_file_write (lex_ctxt * lexic)
@@ -764,9 +696,6 @@ nasl_file_write (lex_ctxt * lexic)
   int fd;
   int n;
 
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
   content = get_str_local_var_by_name (lexic, "data");
   fd = get_int_local_var_by_name (lexic, "fp", -1);
   if (content == NULL || fd < 0)
@@ -802,7 +731,7 @@ nasl_file_write (lex_ctxt * lexic)
 }
 
 /**
- * @brief Seek in file if check_authenticated.
+ * @brief Seek in file.
  */
 tree_cell *
 nasl_file_seek (lex_ctxt * lexic)
@@ -811,10 +740,6 @@ nasl_file_seek (lex_ctxt * lexic)
   int fd;
   int foffset;
 
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
-
   foffset = get_int_local_var_by_name (lexic, "offset", 0);
   fd = get_int_local_var_by_name (lexic, "fp", -1);
   if (fd < 0)
