@@ -134,6 +134,11 @@ nasl_sha1 (lex_ctxt * lexic)
   return nasl_hash (lexic, GCRY_MD_SHA1);
 }
 
+tree_cell *
+nasl_sha256 (lex_ctxt * lexic)
+{
+  return nasl_hash (lexic, GCRY_MD_SHA256);
+}
 
 tree_cell *
 nasl_ripemd160 (lex_ctxt * lexic)
@@ -210,6 +215,85 @@ nasl_get_sign (lex_ctxt * lexic)
   return retc;
 }
 
+static void *
+hmac_sha256 (void *key, int keylen, void *buf, int buflen)
+{
+  void *signature = g_malloc0 (32);
+  gsize signlen = 32;
+  GHmac *hmac;
+
+  hmac = g_hmac_new (G_CHECKSUM_SHA256, key, keylen);
+  g_hmac_update (hmac, buf, buflen);
+  g_hmac_get_digest (hmac, signature, &signlen);
+  g_hmac_unref (hmac);
+  return signature;
+}
+
+tree_cell *
+nasl_hmac_sha256 (lex_ctxt * lexic)
+{
+  void *key, *buf, *signature;
+  int keylen, buflen;
+  tree_cell *retc;
+
+  key = get_str_var_by_name (lexic, "key");
+  buf = get_str_var_by_name (lexic, "buf");
+  keylen = get_int_var_by_name (lexic, "keylen", -1);
+  buflen = get_int_var_by_name (lexic, "buflen", -1);
+  if (!key || !buf || keylen <= 0 || buflen <= 0)
+    {
+      nasl_perror (lexic,
+                   "Syntax : hmac_sha256(buf:<b>, buflen:<bl>, key:<k>, keylen:<kl>)\n");
+      return NULL;
+    }
+  signature = hmac_sha256 (key, keylen, buf, buflen);
+
+  retc = alloc_tree_cell (0, NULL);
+  retc->type = CONST_DATA;
+  retc->size = 32;
+  retc->x.str_val = (char *) signature;
+  return retc;
+}
+
+tree_cell *
+nasl_get_smb2_sign (lex_ctxt * lexic)
+{
+  void *key, *buf, *signature, *ret;
+  int keylen, buflen;
+  tree_cell *retc;
+
+  key = get_str_var_by_name (lexic, "key");
+  buf = get_str_var_by_name (lexic, "buf");
+  keylen = get_var_size_by_name (lexic, "key");
+  buflen = get_var_size_by_name (lexic, "buf");
+  if (!key || !buf || keylen <= 0)
+    {
+      nasl_perror (lexic,
+                   "Syntax : get_smb2_sign(buf:<b>, key:<k>)");
+      return NULL;
+    }
+  if (buflen < 64)
+    {
+      nasl_perror (lexic, "get_smb2_sign: Buffer length < 64");
+      return NULL;
+    }
+
+  /* Zero the SMB2 signature field, then calculate signature */
+  memset(buf + 48, 0, 16);
+  signature = hmac_sha256 (key, keylen, buf, buflen);
+
+  /* Return the header with signature included. */
+  ret = g_malloc0 (buflen);
+  memcpy (ret, buf, buflen);
+  memcpy (ret + 48, signature, 16);
+  g_free (signature);
+  retc = alloc_tree_cell (0, NULL);
+  retc->type = CONST_DATA;
+  retc->size = buflen;
+  retc->x.str_val = (char *) ret;
+  return retc;
+}
+
 tree_cell *
 nasl_ntlmv2_response (lex_ctxt * lexic)
 {
