@@ -27,6 +27,7 @@ tree_cell *nasl_md4 (lex_ctxt *);
 tree_cell *nasl_md5 (lex_ctxt *);
 tree_cell *nasl_sha (lex_ctxt *);
 tree_cell *nasl_sha1 (lex_ctxt *);
+tree_cell *nasl_sha256 (lex_ctxt *);
 tree_cell *nasl_ripemd160 (lex_ctxt *);
 tree_cell *nasl_hmac_md2 (lex_ctxt *);
 tree_cell *nasl_hmac_md5 (lex_ctxt *);
@@ -45,4 +46,6 @@ tree_cell *nasl_keyexchg (lex_ctxt * lexic);
 tree_cell *nasl_insert_hexzeros (lex_ctxt * lexic);
 tree_cell *nasl_get_password (lex_ctxt * lexic);
 tree_cell *nasl_get_sign (lex_ctxt * lexic);
+tree_cell *nasl_get_smb2_sign (lex_ctxt * lexic);
+tree_cell *nasl_hmac_sha256 (lex_ctxt * lexic);
 #endif
