@@ -1,6 +1,6 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
- * Copyright (C) 2002 - 2004 Tenable Network Security 
+ * Copyright (C) 2002 - 2004 Tenable Network Security
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2,
@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *
  */
@@ -27,19 +27,13 @@
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
 
-#include <fcntl.h>              /* for open */
-#include <sys/stat.h>           /* for stat */
-#include <sys/mman.h>           /* for mmap */
-#include <unistd.h>             /* for close */
-#include "system.h"             /* for emalloc */
-
+#include "openvas_logging.h"
 
 #include "nasl_tree.h"
 #include "nasl_global_ctxt.h"
 #include "nasl_func.h"
 #include "nasl_var.h"
 #include "nasl_lex_ctxt.h"
-#include "exec.h"
 
 #include "strutils.h"
 #include "nasl_packet_forgery.h"
@@ -47,17 +41,12 @@
 #include "nasl_misc_funcs.h"
 #include "nasl_crypto2.h"
 
-#ifndef MAP_FAILED
-#define MAP_FAILED (void*)(-1)
-#endif
-
-
 #define INTBLOB_LEN	20
 #define SIGBLOB_LEN	(2*INTBLOB_LEN)
 
 /**
  * @brief Prints a GnuTLS error.
- * 
+ *
  * The parameter err should be the GnuTLS error code
  */
 void
@@ -68,7 +57,7 @@ print_tls_error (lex_ctxt * lexic, char *txt, int err)
 
 /**
  * @brief Prints a libgcrypt error.
- * 
+ *
  * The parameter err should be the libgcrypt error code
  */
 void
@@ -80,7 +69,7 @@ print_gcrypt_error (lex_ctxt * lexic, char *function, int err)
 
 /**
  * @brief Converts a string to a gcry_mpi_t.
- * 
+ *
  * The string of len bytes at data
  * should contain the MPI as an unsigned int in bigendian form
  * (libgcrypt's GCRYMPI_FMT_USG).  The new MPI object is stored in dest.
@@ -97,11 +86,7 @@ mpi_from_string (lex_ctxt * lexic, gcry_mpi_t * dest, void *data, size_t len,
   gcry_error_t err;
   unsigned char *buffer = data;
 
-  if (len < 0)
-    return -1;
-
   err = gcry_mpi_scan (dest, GCRYMPI_FMT_USG, buffer, len, NULL);
-
   if (err)
     {
       nasl_perror (lexic, "%s(): gcry_mpi_scan failed for %s: %s/%s\n",
@@ -115,7 +100,7 @@ mpi_from_string (lex_ctxt * lexic, gcry_mpi_t * dest, void *data, size_t len,
 
 /**
  * @brief Converts a named nasl parameter to a gcry_mpi_t.
- * 
+ *
  * The new MPI object
  * is stored in dest.  The parameter parameter is the name of the
  * parameter to be taken from lexic.  The parameter function is used in
@@ -167,7 +152,7 @@ set_mpi_retc (tree_cell * retc, gcry_mpi_t mpi)
   if (!buffer)
     return -1;
 
-  retc->x.str_val = emalloc (size);
+  retc->x.str_val = g_malloc0 (size);
   memcpy (retc->x.str_val, buffer, size);
   retc->size = size;
 
@@ -190,7 +175,7 @@ nasl_bn_cmp (lex_ctxt * lexic)
   tree_cell *retc = NULL;
   gcry_mpi_t key1 = NULL, key2 = NULL;
 
-  retc = emalloc (sizeof (tree_cell));
+  retc = g_malloc0 (sizeof (tree_cell));
   retc->ref_count = 1;
   retc->type = CONST_INT;
   retc->x.i_val = 1;
@@ -245,7 +230,7 @@ nasl_bn_random (lex_ctxt * lexic)
 
 fail:
   retc->size = 0;
-  retc->x.str_val = emalloc (0);
+  retc->x.str_val = g_malloc0 (1);
 ret:
   gcry_mpi_release (key);
   return retc;
@@ -253,7 +238,7 @@ ret:
 
 /**
  * @brief Loads a private key from a string.
- * 
+ *
  * The string is taken from the nasl
  * parameter whose name is given by priv_name.  The passphrase_name is
  * the name of the parameter holding the passphrase if any.  The string
@@ -329,9 +314,6 @@ nasl_pem_to (lex_ctxt * lexic, int type)
   gcry_mpi_t key = NULL;
   int err;
 
-  if (check_authenticated (lexic) < 0)
-    return FAKE_CELL;
-
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
 
@@ -390,7 +372,7 @@ nasl_pem_to (lex_ctxt * lexic, int type)
 
 fail:
   retc->size = 0;
-  retc->x.str_val = emalloc (0);
+  retc->x.str_val = g_malloc0 (1);
 ret:
   gcry_mpi_release (key);
   gnutls_x509_privkey_deinit (privkey);
@@ -434,7 +416,7 @@ nasl_pem_to_dsa (lex_ctxt * lexic)
 
 /**
  * @brief compute the diffie hellman public key.
- * 
+ *
  * Neither GnuTLS nor Libgcrypt
  * contain a direct counterpart to OpenSSL's DH_generate_key, so we
  * implement it ourselves.  This function was copied from from gnutls
@@ -461,7 +443,7 @@ calc_dh_public (gcry_mpi_t g, gcry_mpi_t prime, gcry_mpi_t priv)
 
 /**
  * @brief Compute the diffie hellman shared secret key.
- * 
+ *
  * Neither GnuTLS nor
  * libgcrypt contain a direct counterpart to OpenSSL's DH_compute_key,
  * so we implement it ourselves.  This function was copied from from
@@ -520,7 +502,7 @@ nasl_dh_generate_key (lex_ctxt * lexic)
     goto ret;
 
 fail:
-  retc->x.str_val = emalloc (0);
+  retc->x.str_val = g_malloc0 (1);
   retc->size = 0;
 ret:
   gcry_mpi_release (p);
@@ -573,7 +555,7 @@ nasl_dh_compute_key (lex_ctxt * lexic)
 
 fail:
   retc->size = 0;
-  retc->x.str_val = emalloc (0);
+  retc->x.str_val = g_malloc0 (1);
 ret:
   gcry_mpi_release (p);
   gcry_mpi_release (g);
@@ -586,7 +568,7 @@ ret:
 
 /**
  * @brief Extracts an MPI value from a libgcryt s-expression.
- * 
+ *
  * The return value
  * is the cadr of the subexpression whose car is given by token.  The
  * function returns NULL if the token doesn't occur in the expression or
@@ -601,8 +583,8 @@ extract_mpi_from_sexp (gcry_sexp_t sexp, const char *token)
   child = gcry_sexp_find_token (sexp, token, strlen (token));
   if (!child)
     {
-      fprintf (stderr, "set_retc_from_sexp: no subexpression with token <%s>\n",
-               token);
+      log_legacy_write ("set_retc_from_sexp: no subexpression with token <%s>",
+                        token);
     }
   else
     {
@@ -616,7 +598,7 @@ extract_mpi_from_sexp (gcry_sexp_t sexp, const char *token)
 
 /**
  * @brief Sets the return value in retc from an sexpression.
- * 
+ *
  * The function uses
  * extract_mpi_from_sexp to extract an MPI from the sexpression sexp and
  * the subexpression given by token.
@@ -670,9 +652,9 @@ strip_pkcs1_padding (tree_cell * retc)
       if (i <= retc->size)
         {
           int rest = retc->size - i;
-          temp = emalloc (rest);
+          temp = g_malloc0 (rest);
           memcpy (temp, p + i, rest);
-          efree (&(retc->x.str_val));
+          g_free (retc->x.str_val);
           retc->x.str_val = temp;
           retc->size = rest;
         }
@@ -739,7 +721,7 @@ nasl_rsa_public_decrypt (lex_ctxt * lexic)
 
 fail:
   retc->size = 0;
-  retc->x.str_val = emalloc (0);
+  retc->x.str_val = g_malloc0 (1);
 ret:
   gcry_sexp_release (decrypted);
   gcry_sexp_release (key);
@@ -793,6 +775,7 @@ nasl_sexp_from_privkey (lex_ctxt * lexic, gnutls_x509_privkey_t privkey)
   if (gcry_mpi_cmp (mpis[3], mpis[4]) > 0)
     {
       gcry_mpi_swap (mpis[3], mpis[4]);
+      gcry_mpi_invm (mpis[5], mpis[3], mpis[4]);
     }
 
   gerr =
@@ -843,9 +826,6 @@ nasl_rsa_sign (lex_ctxt * lexic)
   gnutls_x509_privkey_t priv_key = NULL;
   gcry_error_t err;
 
-  if (check_authenticated (lexic) < 0)
-    return FAKE_CELL;
-
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
 
@@ -884,7 +864,7 @@ nasl_rsa_sign (lex_ctxt * lexic)
 
 fail:
   retc->size = 0;
-  retc->x.str_val = emalloc (0);
+  retc->x.str_val = g_malloc0 (1);
 ret:
   gcry_sexp_release (ssig);
   gcry_sexp_release (sdata);
@@ -913,7 +893,7 @@ nasl_dsa_do_verify (lex_ctxt * lexic)
   gcry_sexp_t ssig = NULL, skey = NULL, sdata = NULL;
   gcry_error_t err;
 
-  retc = emalloc (sizeof (tree_cell));
+  retc = g_malloc0 (sizeof (tree_cell));
   retc->ref_count = 1;
   retc->type = CONST_INT;
   retc->x.i_val = 0;
@@ -1005,10 +985,7 @@ nasl_dsa_do_sign (lex_ctxt * lexic)
   unsigned char *sigblob = NULL;
   gcry_error_t err;
 
-  if (check_authenticated (lexic) < 0)
-    return FAKE_CELL;
-
-  retc = emalloc (sizeof (tree_cell));
+  retc = g_malloc0 (sizeof (tree_cell));
   retc->ref_count = 1;
   retc->type = CONST_DATA;
   retc->x.i_val = 0;
@@ -1064,7 +1041,7 @@ nasl_dsa_do_sign (lex_ctxt * lexic)
       goto fail;
     }
 
-  sigblob = emalloc (SIGBLOB_LEN);
+  sigblob = g_malloc0 (SIGBLOB_LEN);
   memset (sigblob, 0, SIGBLOB_LEN);
 
   err =
@@ -1102,7 +1079,7 @@ fail:
   gcry_sexp_release (ssig);
   gcry_sexp_release (skey);
   gcry_sexp_release (sdata);
-  efree (&sigblob);
+  g_free (sigblob);
 
   return retc;
 }
@@ -1198,7 +1175,7 @@ nasl_bf_cbc (lex_ctxt * lexic, int enc)
       goto fail;
     }
 
-  out = emalloc (datalen);
+  out = g_malloc0 (datalen);
   if (!out)
     goto fail;
 
@@ -1213,7 +1190,7 @@ nasl_bf_cbc (lex_ctxt * lexic, int enc)
     }
 
   retc->type = DYN_ARRAY;
-  retc->x.ref_val = a = emalloc (sizeof (nasl_array));
+  retc->x.ref_val = a = g_malloc0 (sizeof (nasl_array));
 
   /* first encrypted */
   v.var_type = VAR2_DATA;
@@ -1235,11 +1212,11 @@ nasl_bf_cbc (lex_ctxt * lexic, int enc)
 
 fail:
   retc->type = CONST_DATA;
-  retc->x.str_val = emalloc (0);
+  retc->x.str_val = g_malloc0 (1);
   retc->size = 0;
 
 ret:
-  efree (&out);
+  g_free (out);
   gcry_cipher_close (hd);
 
   return retc;
@@ -1286,39 +1263,3 @@ nasl_bf_cbc_decrypt (lex_ctxt * lexic)
 {
   return nasl_bf_cbc (lexic, 0);
 }
-
-
-/*--------------------------------------------------------------*/
-
-/**
- * Reads the contents of the file given by filename into memory and
- * returns it as a gnutls_datum_t.  If an error occurs the
- * gnutls_datum_t's data field is NULL.
- */
-gnutls_datum_t
-map_file (const char *filename)
-{
-  struct stat st;
-  int fd = -1;
-  char *map;
-  gnutls_datum_t contents = { NULL, 0 };
-
-  fd = open (filename, O_RDONLY);
-  if (fd < 0)
-    goto fail;
-  if (fstat (fd, &st) < 0)
-    goto fail;
-
-  map = mmap (NULL, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
-  if (map == NULL || map == MAP_FAILED)
-    goto fail;
-
-  contents.data = (unsigned char *) nasl_strndup (map, st.st_size);
-  contents.size = st.st_size;
-  munmap (map, st.st_size);
-
-fail:
-  if (fd >= 0)
-    close (fd);
-  return contents;
-}
