@@ -1,4 +1,4 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2004 Tenable Network Security
  *
@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
@@ -21,8 +21,7 @@
 #include <stdlib.h>             /* for free */
 #include <string.h>             /* for strcmp */
 
-
-#include "system.h"             /* for emalloc */
+#include <glib.h>               /* for g_free */
 
 #include "nasl_tree.h"
 #include "nasl_global_ctxt.h"
@@ -47,28 +46,14 @@ hash_str (const char *s)
 static nasl_func *
 get_func (lex_ctxt * ctxt, const char *name, int h)
 {
-  nasl_func *v, *prev;
+  nasl_func *v;
   lex_ctxt *c;
 
   for (c = ctxt; c != NULL; c = c->up_ctxt)
     {
-      for (prev = NULL, v = c->functions[h]; v != NULL; v = v->next_func)
+      for (v = c->functions[h]; v != NULL; v = v->next_func)
         if (v->func_name != NULL && strcmp (name, v->func_name) == 0)
-          {
-#ifdef SILLY_OPT
-            if (prev != NULL)   /* Move it to start of list */
-              {
-                prev->next_func = v->next_func;
-                v->next_func = c->functions[h];
-                c->functions[h] = v;
-              }
-#endif
-            return v;
-          }
-#ifdef SILLY_OPT
-        else
-          prev = v;
-#endif
+          return v;
     }
 
   return NULL;
@@ -91,8 +76,8 @@ insert_nasl_func (lex_ctxt * lexic, const char *fname, tree_cell * decl_node)
                    fname);
       return NULL;
     }
-  pf = emalloc (sizeof (nasl_func));
-  pf->func_name = estrdup (fname);
+  pf = g_malloc0 (sizeof (nasl_func));
+  pf->func_name = g_strdup (fname);
 
   if (decl_node != NULL && decl_node != FAKE_CELL)
     {
@@ -102,10 +87,10 @@ insert_nasl_func (lex_ctxt * lexic, const char *fname, tree_cell * decl_node)
         else
           pf->nb_named_args++;
 
-      pf->args_names = emalloc (sizeof (char *) * pf->nb_named_args);
+      pf->args_names = g_malloc0 (sizeof (char *) * pf->nb_named_args);
       for (i = 0, pc = decl_node->link[0]; pc != NULL; pc = pc->link[0])
         if (pc->x.str_val != NULL)
-          pf->args_names[i++] = estrdup (pc->x.str_val);
+          pf->args_names[i++] = g_strdup (pc->x.str_val);
       /* Sort argument names */
       qsort (pf->args_names, pf->nb_named_args, sizeof (pf->args_names[0]),
              (qsortcmp)strcmp);
@@ -178,13 +163,13 @@ nasl_func_call (lex_ctxt * lexic, const nasl_func * f, tree_cell * arg_list)
   /* 1. Create a new context */
   lexic2 = init_empty_lex_ctxt ();
   lexic2->script_infos = lexic->script_infos;
-  lexic2->authenticated = lexic->authenticated;
+  lexic2->oid = lexic->oid;
   lexic2->recv_timeout = lexic->recv_timeout;
   lexic2->fct_ctxt = 1;
 
   if (nasl_trace_fp != NULL)
     {
-      trace_buf = emalloc (TRACE_BUF_SZ);
+      trace_buf = g_malloc0 (TRACE_BUF_SZ);
       tn = snprintf (trace_buf, TRACE_BUF_SZ, "Call %s(", f->func_name);        /* RATS: ignore */
       if (tn > 0)
         trace_buf_len += tn;
@@ -210,7 +195,7 @@ nasl_func_call (lex_ctxt * lexic, const nasl_func * f, tree_cell * arg_list)
                      f->func_name, nb_n, nb_u, f->nb_unnamed_args,
                      f->nb_named_args);
       /*
-       * I should look exactly how unnamed arguments works... 
+       * I should look exactly how unnamed arguments works...
        * Or maybe I should remove this feature?
        */
 
@@ -271,8 +256,7 @@ nasl_func_call (lex_ctxt * lexic, const nasl_func * f, tree_cell * arg_list)
             nasl_trace (lexic, "NASL> %s)\n", trace_buf);
           else
             nasl_trace (lexic, "NASL> %s ...)\n", trace_buf);
-          trace_buf_len = 0;
-          efree (&trace_buf);
+          g_free (trace_buf);
         }
 
       /* 4. Chain new context to old (lexic) */
@@ -363,16 +347,18 @@ free_func (nasl_func * f)
 {
   int i;
 
-  efree (&f->func_name);
+  if (! f) return;
+
+  g_free (f->func_name);
+
   if (!(f->flags & FUNC_FLAG_INTERNAL))
     {
       for (i = 0; i < f->nb_named_args; i++)
-        efree (f->args_names + i);
-      efree (&f->func_name);
-      efree (&f->args_names);
+        g_free (f->args_names[i]);
+      g_free (f->args_names);
       deref_cell (f->block);
     }
-  free (f);
+  g_free (f);
 }
 
 
