@@ -1,4 +1,4 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2004 Tenable Network Security
  *
@@ -13,14 +13,19 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
- /*
-  * This file contains all the functions which deal with the remote host :
-  * which ports are open, what is its IP, what is our IP, what transport
-  * is on the remote port, and so on...
-  */
+
+/**
+ * @file nasl_host.c
+ *
+ * @brief Remote host helper functions.
+ *
+ * This file contains all the functions which deal with the remote
+ * host: which ports are open, what is its IP, what is our IP, what
+ * transport is on the remote port, and so on...
+ */
 
 #include <arpa/inet.h>          /* for inet_aton */
 #include <netdb.h>              /* for gethostbyaddr */
@@ -28,10 +33,9 @@
 #include <string.h>             /* for strlen */
 #include <unistd.h>             /* for gethostname */
 
-#include "network.h"            /* for socket_get_next_source_addr */
+#include "network.h"
+#include "../base/openvas_networking.h"
 #include "plugutils.h"          /* for plug_get_host_fqdn */
-#include "resolve.h"            /* for nn_resolve */
-#include "system.h"             /* for estrdup */
 #include "pcap_openvas.h"       /* for v6_is_local_ip */
 
 #include "nasl_tree.h"
@@ -40,7 +44,6 @@
 #include "nasl_var.h"
 #include "nasl_lex_ctxt.h"
 #include "nasl_debug.h"
-#include "exec.h"
 
 #include "nasl_host.h"
 
@@ -48,7 +51,7 @@ tree_cell *
 get_hostname (lex_ctxt * lexic)
 {
   struct arglist *script_infos = lexic->script_infos;
-  char *hostname = (char *) plug_get_host_fqdn (script_infos);
+  char *hostname = plug_get_host_fqdn (script_infos);
   tree_cell *retc;
 
   if (hostname == NULL)
@@ -57,7 +60,7 @@ get_hostname (lex_ctxt * lexic)
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_STR;
   retc->size = strlen (hostname);
-  retc->x.str_val = estrdup (hostname);
+  retc->x.str_val = hostname;
   return retc;
 }
 
@@ -67,28 +70,16 @@ get_host_ip (lex_ctxt * lexic)
 {
   struct arglist *script_infos = lexic->script_infos;
   struct in6_addr *ip = plug_get_host_ip (script_infos);
-  char *txt_ip;
   tree_cell *retc;
-  struct in_addr inaddr;
-  char name[512];
 
   if (ip == NULL)               /* WTF ? */
     {
       return FAKE_CELL;
     }
 
-  inaddr.s_addr = ip->s6_addr32[3];
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_STR;
-  if (IN6_IS_ADDR_V4MAPPED (ip))
-    {
-      txt_ip = estrdup (inet_ntoa (inaddr));
-    }
-  else
-    {
-      txt_ip = estrdup (inet_ntop (AF_INET6, ip, name, sizeof (name)));
-    }
-  retc->x.str_val = estrdup (txt_ip);
+  retc->x.str_val = addr6_as_str (ip);
   retc->size = strlen (retc->x.str_val);
 
   return retc;
@@ -155,11 +146,9 @@ nasl_islocalhost (lex_ctxt * lexic)
   struct arglist *script_infos = lexic->script_infos;
   struct in6_addr *dst = plug_get_host_ip (script_infos);
   tree_cell *retc;
-  struct in_addr inaddr;
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_INT;
-  inaddr.s_addr = dst->s6_addr32[3];
   retc->x.i_val = v6_islocalhost (dst);
   return retc;
 }
@@ -171,11 +160,9 @@ nasl_islocalnet (lex_ctxt * lexic)
   struct arglist *script_infos = lexic->script_infos;
   struct in6_addr *ip = plug_get_host_ip (script_infos);
   tree_cell *retc;
-  struct in_addr inaddr;
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_INT;
-  inaddr.s_addr = ip->s6_addr32[3];
   retc->x.i_val = v6_is_local_ip (ip);
   return retc;
 }
@@ -186,89 +173,51 @@ nasl_this_host (lex_ctxt * lexic)
 {
   struct arglist *script_infos = lexic->script_infos;
   tree_cell *retc;
-  struct in6_addr addr;
   char hostname[255];
-  char *ret;
   struct in6_addr *ia = plug_get_host_ip (script_infos);
   struct in6_addr in6addr;
-  struct in_addr inaddr;
   struct in6_addr src6;
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
 
-  if (IN6_IS_ADDR_V4MAPPED (ia))
-    addr =
-      socket_get_next_source_v4_addr ();
-  else
-    addr =
-      socket_get_next_source_v6_addr ();
-  if (!IN6_ARE_ADDR_EQUAL (&addr, &in6addr_any))
+  if (openvas_source_iface_is_set ())
     {
-      if (IN6_IS_ADDR_V4MAPPED (&addr))
-        {
-          inaddr.s_addr = addr.s6_addr32[3];
-          retc->x.str_val =
-            estrdup (inet_ntop (AF_INET, &inaddr, hostname, sizeof (hostname)));
-        }
-      else
-        retc->x.str_val =
-          estrdup (inet_ntop (AF_INET6, &addr, hostname, sizeof (hostname)));
+      struct in6_addr addr;
 
+      /* Use source_iface's IP address when available. */
+      if (IN6_IS_ADDR_V4MAPPED (ia))
+        openvas_source_addr_as_addr6 (&addr);
+      else
+        openvas_source_addr6 (&addr);
+      retc->x.str_val = addr6_as_str (&addr);
       retc->size = strlen (retc->x.str_val);
       return retc;
     }
-
-
-  if (ia)
+  else
     {
+      /* Manually find the source IP that will be used. */
+      int err = 1;
       if (v6_islocalhost (ia))
-        {
-          memcpy (&src6, ia, sizeof (struct in6_addr));
-        }
+        memcpy (&src6, ia, sizeof (struct in6_addr));
       else
+        err = v6_getsourceip (&src6, ia);
+
+      if (err && !IN6_ARE_ADDR_EQUAL (&src6, &in6addr_any))
         {
-          (void) v6_routethrough (ia, &src6);
-        }
-
-      if (!IN6_ARE_ADDR_EQUAL (&src6, &in6addr_any))
-        {
-          char *ret;
-
-          if (IN6_IS_ADDR_V4MAPPED (&src6))
-            {
-              inaddr.s_addr = src6.s6_addr32[3];
-              ret =
-                estrdup (inet_ntop
-                         (AF_INET, &inaddr, hostname, sizeof (hostname)));
-            }
-          else
-            ret =
-              estrdup (inet_ntop
-                       (AF_INET6, &src6, hostname, sizeof (hostname)));
-
-          retc->x.str_val = ret;
-          retc->size = strlen (ret);
+          retc->x.str_val = addr6_as_str (&src6);
+          retc->size = strlen (retc->x.str_val);
 
           return retc;
         }
 
       hostname[sizeof (hostname) - 1] = '\0';
       gethostname (hostname, sizeof (hostname) - 1);
-      nn_resolve (hostname, &in6addr);
-
-      if (IN6_IS_ADDR_V4MAPPED (&in6addr))
+      if (openvas_resolve_as_addr6 (hostname, &in6addr))
         {
-          inaddr.s_addr = in6addr.s6_addr32[3];
-          ret =
-            estrdup (inet_ntop (AF_INET, &inaddr, hostname, sizeof (hostname)));
+          retc->x.str_val = addr6_as_str (&in6addr);
+          retc->size = strlen (retc->x.str_val);
         }
-      else
-        ret =
-          estrdup (inet_ntop (AF_INET6, &in6addr, hostname, sizeof (hostname)));
-
-      retc->x.str_val = ret;
-      retc->size = strlen (ret);
     }
   return retc;
 }
@@ -283,7 +232,7 @@ nasl_this_host_name (lex_ctxt * lexic)
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
 
-  hostname = emalloc (256);
+  hostname = g_malloc0 (256);
   gethostname (hostname, 255);
 
   retc->x.str_val = hostname;
@@ -292,6 +241,38 @@ nasl_this_host_name (lex_ctxt * lexic)
 }
 
 
+/**
+ * @brief Return the encapsulation mode of a port.
+ * @naslfn{get_port_transport}
+ *
+ * Takes a port number and returns its encapsulation mode (ENCAPS_*)
+ * The defined encapsulation modes are:
+ *          - @a ENCAPS_AUTO   Automatic encapsulation detection.
+ *          - @a ENCAPS_IP     No encapsulation
+ *          - @a ENCAPS_SSLv23 Request compatibility options
+ *          - @a ENCAPS_SSLv2  SSL version 2
+ *          - @a ENCAPS_SSLv3  SSL version 3
+ *          - @a ENCAPS_TLSv1  TLS version 1.0
+ *          - @a ENCAPS_TLSv11 TLS version 1.1
+ *          - @a ENCAPS_TLSv12 TLS version 1.2
+ *          - @a ENCAPS_TLScustom SSL or TLS with custom priorities
+ *
+ * @nasluparam
+ *
+ * - An integer with the port number.
+ *
+ * @naslnparam
+ *
+ * -@a asstring If not 0 return a human readabale string instead of
+ *   an integer.
+ *
+ * @naslret An integer or a string with the encapsulation mode or NULL
+ * on error.
+ *
+ * @param[in] lexic  Lexical context of the NASL interpreter.
+ *
+ * @return A tree cell.
+ */
 tree_cell *
 get_port_transport (lex_ctxt * lexic)
 {
@@ -302,9 +283,20 @@ get_port_transport (lex_ctxt * lexic)
   if (port >= 0)
     {
       int trp = plug_get_port_transport (script_infos, port);
+
       retc = alloc_tree_cell (0, NULL);
-      retc->type = CONST_INT;
-      retc->x.i_val = trp;
+      if (get_int_local_var_by_name (lexic, "asstring", 0))
+        {
+          const char *s = get_encaps_name (trp);
+          retc->type = CONST_STR;
+          retc->x.str_val = g_strdup (s);
+          retc->size = strlen (s);
+        }
+      else
+        {
+          retc->type = CONST_INT;
+          retc->x.i_val = trp;
+        }
       return retc;
     }
   return NULL;
@@ -321,10 +313,6 @@ nasl_same_host (lex_ctxt * lexic)
   int i, j, n[2], names_nb[2], flag;
   int cmp_hostname = get_int_local_var_by_name (lexic, "cmp_hostname", 0);
 
-  if (check_authenticated (lexic) < 0)
-    return NULL;
-
-
   for (i = 0; i < 2; i++)
     {
       hn[i] = get_str_var_by_num (lexic, i);
@@ -351,8 +339,8 @@ nasl_same_host (lex_ctxt * lexic)
               if (cmp_hostname)
                 {
                   names_nb[i] = 1;
-                  names[i] = emalloc (sizeof (char *));
-                  names[i][0] = estrdup (hn[i]);
+                  names[i] = g_malloc0 (sizeof (char *));
+                  names[i][0] = g_strdup (hn[i]);
                 }
             }
           else
@@ -361,16 +349,16 @@ nasl_same_host (lex_ctxt * lexic)
                    names_nb[i]++)
                 ;
               names_nb[i]++;
-              names[i] = emalloc (sizeof (char *) * names_nb[i]);
-              names[i][0] = estrdup (h->h_name);
+              names[i] = g_malloc0 (sizeof (char *) * names_nb[i]);
+              names[i][0] = g_strdup (h->h_name);
               for (j = 1; j < names_nb[i]; j++)
-                names[i][j] = estrdup (h->h_aliases[j - 1]);
+                names[i][j] = g_strdup (h->h_aliases[j - 1]);
 
               /* Here, we should check that h_addrtype == AF_INET */
               for (n[i] = 0; ((struct in_addr **) h->h_addr_list)[n[i]] != NULL;
                    n[i]++)
                 ;
-              a[i] = emalloc (h->h_length * n[i]);
+              a[i] = g_malloc0 (h->h_length * n[i]);
               for (j = 0; j < n[i]; j++)
                 a[i][j] = *((struct in_addr **) h->h_addr_list)[j];
             }
@@ -383,7 +371,7 @@ nasl_same_host (lex_ctxt * lexic)
             h = NULL;
           if (h == NULL)
             {
-              a[i] = emalloc (sizeof (struct in_addr));
+              a[i] = g_malloc0 (sizeof (struct in_addr));
               memcpy (a[i], &ia, sizeof (struct in_addr));
               n[i] = 1;
             }
@@ -393,69 +381,50 @@ nasl_same_host (lex_ctxt * lexic)
                    names_nb[i]++)
                 ;
               names_nb[i]++;
-              names[i] = emalloc (sizeof (char *) * names_nb[i]);
-              names[i][0] = estrdup (h->h_name);
+              names[i] = g_malloc0 (sizeof (char *) * names_nb[i]);
+              names[i][0] = g_strdup (h->h_name);
               for (j = 1; j < names_nb[i]; j++)
-                names[i][j] = estrdup (h->h_aliases[j - 1]);
+                names[i][j] = g_strdup (h->h_aliases[j - 1]);
 
               /* Here, we should check that h_addrtype == AF_INET */
               for (n[i] = 0; ((struct in_addr **) h->h_addr_list)[n[i]] != NULL;
                    n[i]++)
                 ;
-              a[i] = emalloc (h->h_length * n[i]);
+              a[i] = g_malloc0 (h->h_length * n[i]);
               for (j = 0; j < n[i]; j++)
                 a[i][j] = *((struct in_addr **) h->h_addr_list)[j];
             }
         }
     }
-#if 0
-  fprintf (stderr, "N1=%d\tN2=%d\n", n[0], n[1]);
-#endif
   flag = 0;
   for (i = 0; i < n[0] && !flag; i++)
     for (j = 0; j < n[1] && !flag; j++)
       if (a[0][i].s_addr == a[1][j].s_addr)
         {
           flag = 1;
-#if 0
-          fprintf (stderr, "%s == ", inet_ntoa (a[0][i]));
-          fprintf (stderr, "%s\n", inet_ntoa (a[1][j]));
-#endif
         }
-#if 0
-      else
-        {
-          fprintf (stderr, "%s != ", inet_ntoa (a[0][i]));
-          fprintf (stderr, "%s\n", inet_ntoa (a[1][j]));
-        }
-#endif
 
   if (cmp_hostname)
     for (i = 0; i < names_nb[0] && !flag; i++)
       for (j = 0; j < names_nb[1] && !flag; j++)
         if (strcmp (names[0][i], names[1][j]) == 0)
           {
-#if 0
-            fprintf (stderr, "%s == %s\n", names[0][i], names[1][j]);
-#endif
             flag = 1;
           }
-#if 0
-        else
-          fprintf (stderr, "%s != %s\n", names[0][i], names[1][j]);
-#endif
 
   retc = alloc_typed_cell (CONST_INT);
   retc->x.i_val = flag;
 
   for (i = 0; i < 2; i++)
-    efree (&a[i]);
+    g_free (a[i]);
   if (cmp_hostname)
     {
       for (i = 0; i < 2; i++)
-        for (j = 0; j < names_nb[i]; j++)
-          efree (&names[i][j]);
-      efree (&names[i]);
+        {
+          for (j = 0; j < names_nb[i]; j++)
+            g_free (names[i][j]);
+          g_free (names[i]);
+        }
     }
   return retc;
 }
