@@ -346,6 +346,7 @@ static init_func libfuncs[] = {
   {"MD4", nasl_md4, 1, {NULL}},
   {"MD5", nasl_md5, 1, {NULL}},
   {"SHA1", nasl_sha1, 1, {NULL}},
+  {"SHA256", nasl_sha256, 1, {NULL}},
   {"RIPEMD160", nasl_ripemd160, 1, {NULL}},
   {"HMAC_MD2", nasl_hmac_md2, 0, {"data", "key", NULL}},
   {"HMAC_MD5", nasl_hmac_md5, 0, {"data", "key", NULL}},
@@ -370,6 +371,10 @@ static init_func libfuncs[] = {
   {"dec2str", nasl_dec2str, 0, {"num", NULL}},
   {"get_signature", nasl_get_sign, 0,
    {"buf", "buflen", "key", "seq_number", NULL}},
+  {"get_smb2_signature", nasl_get_smb2_sign, 0,
+   {"buf", "key", NULL}},
+  {"hmac_sha256", nasl_hmac_sha256, 0,
+   {"buf", "buflen", "key", "keylen", NULL}},
   {"dh_generate_key", nasl_dh_generate_key, 0, {"g", "p", "priv", NULL}},
   {"bn_random", nasl_bn_random, 0, {"need", NULL}},
   {"bn_cmp", nasl_bn_cmp, 0, {"key1", "key2", NULL}},
@@ -391,6 +396,8 @@ static init_func libfuncs[] = {
 #ifdef HAVE_NETSNMP
   {"snmpv1_get", nasl_snmpv1_get, 0, {"community", "oid", "port", "protocol",
                                       NULL }},
+  {"snmpv2c_get", nasl_snmpv2c_get, 0, {"community", "oid", "port", "protocol",
+                                        NULL }},
   {"snmpv3_get", nasl_snmpv3_get, 0, {"authpass", "authproto", "oid", "port",
                                       "privpass", "privproto", "username",
                                       NULL }},
@@ -412,6 +419,7 @@ static init_func libfuncs[] = {
   {"ssh_get_server_banner", nasl_ssh_get_server_banner, 1, {NULL }},
 #endif
   {"ssh_get_auth_methods", nasl_ssh_get_auth_methods, 1, {NULL }},
+  {"ssh_get_host_key", nasl_ssh_get_host_key, 1, {NULL }},
 
 #ifdef HAVE_LIBKSBA
   {"cert_open", nasl_cert_open, 1, {"errorvar", NULL }},
