@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
@@ -29,7 +29,6 @@
 #include "bpf_share.h"          /* for bpf_open_live */
 #include "pcap_openvas.h"       /* for routethrough */
 #include "plugutils.h"          /* plug_get_host_ip */
-#include "system.h"             /* for emalloc */
 
 #include "nasl_raw.h"
 
@@ -43,7 +42,6 @@
 
 #include "nasl_debug.h"
 #include "capture_packet.h"
-#include "strutils.h"
 #include "nasl_packet_forgery.h"
 #include "nasl_packet_forgery_v6.h"
 
@@ -120,7 +118,7 @@ forge_ip_packet (lex_ctxt * lexic)
   retc->type = CONST_DATA;
   retc->size = sizeof (struct ip) + data_len;
 
-  pkt = (struct ip *) emalloc (sizeof (struct ip) + data_len);
+  pkt = (struct ip *) g_malloc0 (sizeof (struct ip) + data_len);
   retc->x.str_val = (char *) pkt;
 
   pkt->ip_hl = get_int_local_var_by_name (lexic, "ip_hl", 5);
@@ -265,7 +263,7 @@ get_ip_element (lex_ctxt * lexic)
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
   retc->size = strlen (ret_ascii);
-  retc->x.str_val = estrdup (ret_ascii);
+  retc->x.str_val = g_strdup (ret_ascii);
 
   return retc;
 }
@@ -288,7 +286,7 @@ set_ip_elements (lex_ctxt * lexic)
       return NULL;
     }
 
-  pkt = (struct ip *) emalloc (size);
+  pkt = (struct ip *) g_malloc0 (size);
   bcopy (o_pkt, pkt, size);
 
 
@@ -352,7 +350,7 @@ insert_ip_options (lex_ctxt * lexic)
     pad_len = 0;
 
   hl = ip->ip_hl * 4 < UNFIX (ip->ip_len) ? ip->ip_hl * 4 : UNFIX (ip->ip_len);
-  new_packet = emalloc (size + 4 + value_size + pad_len);
+  new_packet = g_malloc0 (size + 4 + value_size + pad_len);
   bcopy (ip, new_packet, hl);
 
   uc_code = (u_char) code;
@@ -465,7 +463,6 @@ forge_tcp_packet (lex_ctxt * lexic)
   tree_cell *retc;
   char *data;
   int len;
-  u_char *pkt;
   struct ip *ip, *tcp_packet;
   struct tcphdr *tcp;
   int ipsz;
@@ -490,9 +487,8 @@ forge_tcp_packet (lex_ctxt * lexic)
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
-  tcp_packet = (struct ip *) emalloc (ipsz + sizeof (struct tcphdr) + len);
+  tcp_packet = (struct ip *) g_malloc0 (ipsz + sizeof (struct tcphdr) + len);
   retc->x.str_val = (char *) tcp_packet;
-  pkt = (u_char *) tcp_packet;
 
   bcopy (ip, tcp_packet, ipsz);
   /* recompute the ip checksum, because the ip length changed */
@@ -528,7 +524,7 @@ forge_tcp_packet (lex_ctxt * lexic)
     {
       struct pseudohdr pseudoheader;
       char *tcpsumdata =
-        emalloc (sizeof (struct pseudohdr) + (len % 2 ? len + 1 : len));
+        g_malloc0 (sizeof (struct pseudohdr) + (len % 2 ? len + 1 : len));
       struct in_addr source, dest;
 
       source.s_addr = ip->ip_src.s_addr;
@@ -549,7 +545,7 @@ forge_tcp_packet (lex_ctxt * lexic)
       tcp->th_sum =
         np_in_cksum ((unsigned short *) tcpsumdata,
                      12 + sizeof (struct tcphdr) + len);
-      efree (&tcpsumdata);
+      g_free (tcpsumdata);
     }
 
 
@@ -624,7 +620,7 @@ get_tcp_element (lex_ctxt * lexic)
       retc = alloc_tree_cell (0, NULL);
       retc->type = CONST_DATA;
       retc->size = UNFIX (ip->ip_len) - ntohl (tcp->th_off) * 4;
-      retc->x.str_val = emalloc (retc->size);
+      retc->x.str_val = g_malloc0 (retc->size);
       bcopy (tcp + ntohl (tcp->th_off) * 4, retc->x.str_val, retc->size);
       return retc;
     }
@@ -652,7 +648,7 @@ set_tcp_elements (lex_ctxt * lexic)
   int data_len = get_local_var_size_by_name (lexic, "data");
   char *npkt;
 
-  if (pkt == NULL)
+  if (!ip)
     {
       nasl_perror (lexic,
                    "set_tcp_elements : Invalid value for the argument 'tcp'\n");
@@ -675,7 +671,7 @@ set_tcp_elements (lex_ctxt * lexic)
       data = (char *) ((char *) tcp + tcp->th_off * 4);
     }
 
-  npkt = emalloc (ip->ip_hl * 4 + tcp->th_off * 4 + data_len);
+  npkt = g_malloc0 (ip->ip_hl * 4 + tcp->th_off * 4 + data_len);
   bcopy (pkt, npkt, UNFIX (ip->ip_len));
 
   ip = (struct ip *) (npkt);
@@ -711,7 +707,7 @@ set_tcp_elements (lex_ctxt * lexic)
     {
       struct pseudohdr pseudoheader;
       char *tcpsumdata =
-        emalloc (sizeof (struct pseudohdr) + data_len + (data_len % 2));
+        g_malloc0 (sizeof (struct pseudohdr) + data_len + (data_len % 2));
       struct in_addr source, dest;
 
       source.s_addr = ip->ip_src.s_addr;
@@ -731,7 +727,7 @@ set_tcp_elements (lex_ctxt * lexic)
       tcp->th_sum =
         np_in_cksum ((unsigned short *) tcpsumdata,
                      sizeof (pseudoheader) + data_len);
-      efree (&tcpsumdata);
+      g_free (tcpsumdata);
     }
 
   retc = alloc_tree_cell (0, NULL);
@@ -834,7 +830,7 @@ struct pseudo_udp_hdr
 {
   struct in_addr saddr;
   struct in_addr daddr;
-  char nothing;
+  char zero;
   char proto;
   unsigned short len;
   struct udphdr udpheader;
@@ -855,15 +851,8 @@ forge_udp_packet (lex_ctxt * lexic)
       struct ip *udp_packet;
       struct udphdr *udp;
 
-
-
-
-
-
-      pkt =
-        emalloc (sizeof (struct udphdr) + ip->ip_hl * 4 +
-                 sizeof (struct udphdr) + data_len);
-
+      pkt = g_malloc0 (sizeof (struct udphdr) + ip->ip_hl * 4 +
+                       sizeof (struct udphdr) + data_len);
 
       udp_packet = (struct ip *) pkt;
       udp = (struct udphdr *) (pkt + ip->ip_hl * 4);
@@ -887,8 +876,8 @@ forge_udp_packet (lex_ctxt * lexic)
           struct pseudo_udp_hdr pseudohdr;
           struct in_addr source, dest;
           char *udpsumdata =
-            (char *) emalloc (sizeof (struct pseudo_udp_hdr) +
-                              (data_len % 2 ? data_len + 1 : data_len));
+            (char *) g_malloc0 (sizeof (struct pseudo_udp_hdr) +
+                                (data_len % 2 ? data_len + 1 : data_len));
 
           source.s_addr = ip->ip_src.s_addr;
           dest.s_addr = ip->ip_dst.s_addr;
@@ -909,7 +898,7 @@ forge_udp_packet (lex_ctxt * lexic)
           udp->uh_sum =
             np_in_cksum ((unsigned short *) udpsumdata,
                          12 + sizeof (struct udphdr) + data_len);
-          efree (&udpsumdata);
+          g_free (udpsumdata);
         }
 
 
@@ -989,7 +978,7 @@ get_udp_element (lex_ctxt * lexic)
           ipsz)
         sz = ipsz - ip->ip_hl * 4 - sizeof (struct udphdr);
 
-      retc->x.str_val = emalloc (sz);
+      retc->x.str_val = g_malloc0 (sz);
       retc->size = sz;
       bcopy (udp + ip->ip_hl * 4 + sizeof (struct udphdr), retc->x.str_val, sz);
       return retc;
@@ -1018,7 +1007,7 @@ set_udp_elements (lex_ctxt * lexic)
 
   if (ip != NULL)
     {
-      char *pkt = emalloc (sz + data_len);
+      char *pkt;
       struct udphdr *udp;
       tree_cell *retc;
       int old_len;
@@ -1030,12 +1019,12 @@ set_udp_elements (lex_ctxt * lexic)
       if (data != NULL)
         {
           sz = ip->ip_hl * 4 + sizeof (struct udphdr) + data_len;
-          pkt = emalloc (sz);
+          pkt = g_malloc0 (sz);
           bcopy (ip, pkt, ip->ip_hl * 4 + sizeof (struct udphdr));
         }
       else
         {
-          pkt = emalloc (sz);
+          pkt = g_malloc0 (sz);
           bcopy (ip, pkt, sz);
         }
 
@@ -1046,7 +1035,7 @@ set_udp_elements (lex_ctxt * lexic)
         {
           ip->ip_len = FIX (sz);
           ip->ip_sum = 0;
-          ip->ip_sum = np_in_cksum (ip, ip->ip_hl * 4);
+          ip->ip_sum = np_in_cksum ((u_short *) ip, ip->ip_hl * 4);
         }
       udp = (struct udphdr *) (pkt + ip->ip_hl * 4);
 
@@ -1089,8 +1078,8 @@ set_udp_elements (lex_ctxt * lexic)
 
 
           udpsumdata =
-            (char *) emalloc (sizeof (struct pseudo_udp_hdr) +
-                              (len % 2 ? len + 1 : len));
+            (char *) g_malloc0 (sizeof (struct pseudo_udp_hdr) +
+                                (len % 2 ? len + 1 : len));
 
           source.s_addr = ip->ip_src.s_addr;
           dest.s_addr = ip->ip_dst.s_addr;
@@ -1111,7 +1100,7 @@ set_udp_elements (lex_ctxt * lexic)
           udp->uh_sum =
             np_in_cksum ((unsigned short *) udpsumdata,
                          12 + sizeof (struct udphdr) + len);
-          efree (&udpsumdata);
+          g_free (udpsumdata);
         }
       retc = alloc_tree_cell (0, NULL);
       retc->type = CONST_DATA;
@@ -1185,11 +1174,10 @@ forge_icmp_packet (lex_ctxt * lexic)
       if (ip->ip_hl * 4 > ip_sz)
         return NULL;
 
-      pkt = emalloc (sizeof (struct icmp) + ip_sz + len);
+      pkt = g_malloc0 (sizeof (struct icmp) + ip_sz + len);
       ip_icmp = (struct ip *) pkt;
 
 
-
       bcopy (ip, ip_icmp, ip_sz);
       if (UNFIX (ip_icmp->ip_len) <= (ip_icmp->ip_hl * 4))
         {
@@ -1267,7 +1255,7 @@ get_icmp_element (lex_ctxt * lexic)
             get_var_size_by_name (lexic, "icmp") - (ip->ip_hl * 4) - 8;
           if (retc->size > 0)
             retc->x.str_val =
-              nasl_strndup (&(p[ip->ip_hl * 4 + 8]), retc->size);
+              g_memdup (&(p[ip->ip_hl * 4 + 8]), retc->size + 1);
           else
             {
               retc->x.str_val = NULL;
@@ -1321,7 +1309,7 @@ forge_igmp_packet (lex_ctxt * lexic)
     {
       char *data = get_str_local_var_by_name (lexic, "data");
       int len = data ? get_local_var_size_by_name (lexic, "data") : 0;
-      u_char *pkt = emalloc (sizeof (struct igmp) + ip->ip_hl * 4 + len);
+      u_char *pkt = g_malloc0 (sizeof (struct igmp) + ip->ip_hl * 4 + len);
       struct ip *ip_igmp = (struct ip *) pkt;
       struct igmp *igmp;
       char *p;
@@ -1396,7 +1384,6 @@ nasl_tcp_ping (lex_ctxt * lexic)
   int i = 0;
   int bpf;
   char filter[255];
-  u_char *pk = NULL;
   tree_cell *retc;
   int opt = 1;
   struct timeval tv;
@@ -1409,13 +1396,11 @@ nasl_tcp_ping (lex_ctxt * lexic)
     { 139, 135, 445, 80, 22, 515, 23, 21, 6000, 1025, 25, 111, 1028, 9100, 1029,
 79, 497, 548, 5000, 1917, 53, 161, 9001, 65535, 443, 113, 993, 8080, 0 };
   int num_ports = 0;
-  struct in_addr *pinaddr;
   struct in_addr inaddr;
 
   if (dst == NULL || (IN6_IS_ADDR_V4MAPPED (dst) != 1))
     return NULL;
   inaddr.s_addr = dst->s6_addr32[3];
-  pinaddr = &inaddr;
   for (i = 0; i < sizeof (sports) / sizeof (int); i++)
     {
       if (sports[i] == 0)
@@ -1425,15 +1410,12 @@ nasl_tcp_ping (lex_ctxt * lexic)
 
   for (i = 0; ports[i]; i++)
     num_ports++;
-  i = 0;
 
   soc = socket (AF_INET, SOCK_RAW, IPPROTO_RAW);
   if (soc < 0)
     return NULL;
-#ifdef IP_HDRINCL
   if (setsockopt (soc, IPPROTO_IP, IP_HDRINCL, (char *) &opt, sizeof (opt)) < 0)
     perror ("setsockopt ");
-#endif
 
   port = get_int_local_var_by_name (lexic, "port", -1);
   if (port == -1)
@@ -1513,7 +1495,7 @@ nasl_tcp_ping (lex_ctxt * lexic)
                   sizeof (soca));
           tv.tv_sec = 0;
           tv.tv_usec = 100000;
-          if (bpf >= 0 && (pk = bpf_next_tv (bpf, &len, &tv)))
+          if (bpf >= 0 && bpf_next_tv (bpf, &len, &tv))
             flag++;
         }
     }
@@ -1558,10 +1540,8 @@ nasl_send_packet (lex_ctxt * lexic)
   soc = socket (AF_INET, SOCK_RAW, IPPROTO_RAW);
   if (soc < 0)
     return NULL;
-#ifdef IP_HDRINCL
   if (setsockopt (soc, IPPROTO_IP, IP_HDRINCL, (char *) &i, sizeof (i)) < 0)
     perror ("setsockopt ");
-#endif
 
   while ((ip = get_str_var_by_num (lexic, vi)) != NULL)
     {
@@ -1582,7 +1562,7 @@ nasl_send_packet (lex_ctxt * lexic)
       bzero (&sockaddr, sizeof (struct sockaddr_in));
       sockaddr.sin_family = AF_INET;
       sockaddr.sin_addr = sip->ip_dst;
-      if (dstip != NULL && sockaddr.sin_addr.s_addr != inaddr.s_addr)
+      if (sockaddr.sin_addr.s_addr != inaddr.s_addr)
         {
           char txt1[64], txt2[64];
           strncpy (txt1, inet_ntoa (sockaddr.sin_addr), sizeof (txt1));
@@ -1622,7 +1602,7 @@ nasl_send_packet (lex_ctxt * lexic)
               while (answer != NULL
                      && (!memcmp (answer, (char *) ip, sizeof (struct ip))))
                 {
-                  efree (&answer);
+                  g_free (answer);
                   answer = (u_char *) capture_next_packet (bpf, to, &answer_sz);
                 }
 
@@ -1731,7 +1711,7 @@ nasl_pcap_next (lex_ctxt * lexic)
               struct ip *ip;
               ip = (struct ip *) (packet + dl_len);
               sz = UNFIX (ip->ip_len);
-              ret = emalloc (sz);
+              ret = g_malloc0 (sz);
 
               is_ip = (ip->ip_v == 4);
 
@@ -1750,7 +1730,7 @@ nasl_pcap_next (lex_ctxt * lexic)
               struct ip6_hdr *ip;
               ip = (struct ip6_hdr *) (packet + dl_len);
               sz = UNFIX (ip->ip6_plen);
-              ret6 = emalloc (sz);
+              ret6 = g_malloc0 (sz);
 
               is_ip = ((ip->ip6_flow & 0x3ffff) == 96);
               if (is_ip)
@@ -1840,7 +1820,7 @@ nasl_send_capture (lex_ctxt * lexic)
       struct timeval then, now;
 
       retc = nasl_send (lexic);
-      efree (&retc);
+      g_free (retc);
 
       gettimeofday (&then, NULL);
       for (;;)
@@ -1865,7 +1845,7 @@ nasl_send_capture (lex_ctxt * lexic)
               struct ip *ip;
               ip = (struct ip *) (packet + dl_len);
               sz = UNFIX (ip->ip_len);
-              ret = emalloc (sz);
+              ret = g_malloc0 (sz);
 
               is_ip = (ip->ip_v == 4);
               if (is_ip)
@@ -1883,7 +1863,7 @@ nasl_send_capture (lex_ctxt * lexic)
               struct ip6_hdr *ip;
               ip = (struct ip6_hdr *) (packet + dl_len);
               sz = UNFIX (ip->ip6_plen);
-              ret6 = emalloc (sz);
+              ret6 = g_malloc0 (sz);
               is_ip = ((ip->ip6_flow & 0x3ffff) == 96);
               if (is_ip)
                 {
