@@ -1,4 +1,3 @@
-
 /* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2004 Tenable Network Security
@@ -14,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *
  * Modified for IPv6 packet forgery - 04/02/2010
@@ -49,7 +48,6 @@
 #include "bpf_share.h"          /* for bpf_open_live */
 #include "pcap_openvas.h"       /* for routethrough */
 #include "plugutils.h"          /* plug_get_host_ip */
-#include "system.h"             /* for emalloc */
 
 #include "nasl_raw.h"
 
@@ -63,7 +61,6 @@
 
 #include "nasl_debug.h"
 #include "capture_packet.h"
-#include "strutils.h"
 #include "nasl_packet_forgery_v6.h"
 
 /** @todo: It still needs to be taken care
@@ -145,7 +142,7 @@ forge_ipv6_packet (lex_ctxt * lexic)
   retc->type = CONST_DATA;
   retc->size = sizeof (struct ip6_hdr) + data_len;
 
-  pkt = (struct ip6_hdr *) emalloc (sizeof (struct ip6_hdr) + data_len);
+  pkt = (struct ip6_hdr *) g_malloc0 (sizeof (struct ip6_hdr) + data_len);
   retc->x.str_val = (char *) pkt;
 
   version = get_int_local_var_by_name (lexic, "ip6_v", 6);
@@ -271,7 +268,7 @@ get_ipv6_element (lex_ctxt * lexic)
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
   retc->size = strlen (ret_ascii);
-  retc->x.str_val = estrdup (ret_ascii);
+  retc->x.str_val = g_strdup (ret_ascii);
 
   return retc;
 }
@@ -292,9 +289,6 @@ set_ipv6_elements (lex_ctxt * lexic)
   tree_cell *retc = alloc_tree_cell (0, NULL);
   struct ip6_hdr *pkt;
   char *s;
-  int ver;
-  int tc;
-  int fl;
 
   if (o_pkt == NULL)
     {
@@ -302,14 +296,9 @@ set_ipv6_elements (lex_ctxt * lexic)
       return NULL;
     }
 
-  pkt = (struct ip6_hdr *) emalloc (size);
+  pkt = (struct ip6_hdr *) g_malloc0 (size);
   bcopy (o_pkt, pkt, size);
 
-  ver = get_int_local_var_by_name (lexic, "ip6_v", (pkt->ip6_flow & 0x3ffff));
-  tc =
-    get_int_local_var_by_name (lexic, "ip6_tc", (pkt->ip6_flow >> 20) & 0xff);
-  fl = get_int_local_var_by_name (lexic, "ip6_fl", pkt->ip6_flow >> 28);
-
   pkt->ip6_plen = get_int_local_var_by_name (lexic, "ip6_plen", pkt->ip6_plen);
   pkt->ip6_nxt = get_int_local_var_by_name (lexic, "ip6_nxt", pkt->ip6_nxt);
   pkt->ip6_hlim = get_int_local_var_by_name (lexic, "ip6_hlim", pkt->ip6_hlim);
@@ -410,7 +399,7 @@ insert_ipv6_options (lex_ctxt * lexic)
     pad_len = 0;
 
   pl = 40 < UNFIX (ip6->ip6_plen) ? 40 : UNFIX (ip6->ip6_plen);
-  new_packet = emalloc (size + 4 + value_size + pad_len);
+  new_packet = g_malloc0 (size + 4 + value_size + pad_len);
   bcopy (ip6, new_packet, pl);
 
   uc_code = (u_char) code;
@@ -476,7 +465,6 @@ forge_tcp_v6_packet (lex_ctxt * lexic)
   tree_cell *retc;
   char *data;
   int len;
-  u_char *pkt;
   struct ip6_hdr *ip6, *tcp_packet;
   struct tcphdr *tcp;
   int ipsz;
@@ -500,9 +488,8 @@ forge_tcp_v6_packet (lex_ctxt * lexic)
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
-  tcp_packet = (struct ip6_hdr *) emalloc (ipsz + sizeof (struct tcphdr) + len);
+  tcp_packet = (struct ip6_hdr *) g_malloc0 (ipsz + sizeof (struct tcphdr) + len);
   retc->x.str_val = (char *) tcp_packet;
-  pkt = (u_char *) tcp_packet;
 
   bcopy (ip6, tcp_packet, ipsz);
   /* Adjust length in ipv6 header */
@@ -528,7 +515,7 @@ forge_tcp_v6_packet (lex_ctxt * lexic)
     {
       struct v6pseudohdr pseudoheader;
       char *tcpsumdata =
-        emalloc (sizeof (struct v6pseudohdr) + (len % 2 ? len + 1 : len));
+        g_malloc0 (sizeof (struct v6pseudohdr) + (len % 2 ? len + 1 : len));
 
       bzero (&pseudoheader, 38 + sizeof (struct tcphdr));
       memcpy (&pseudoheader.s6addr, &ip6->ip6_src, sizeof (struct in6_addr));
@@ -545,7 +532,7 @@ forge_tcp_v6_packet (lex_ctxt * lexic)
       tcp->th_sum =
         np_in_cksum ((unsigned short *) tcpsumdata,
                      38 + sizeof (struct tcphdr) + len);
-      efree (&tcpsumdata);
+      g_free (tcpsumdata);
     }
 
   retc->size = ipsz + sizeof (struct tcphdr) + len;
@@ -620,7 +607,7 @@ get_tcp_v6_element (lex_ctxt * lexic)
       retc = alloc_tree_cell (0, NULL);
       retc->type = CONST_DATA;
       retc->size = UNFIX (ip6->ip6_plen) - ntohl (tcp->th_off) * 4;
-      retc->x.str_val = emalloc (retc->size);
+      retc->x.str_val = g_malloc0 (retc->size);
       bcopy (tcp + ntohl (tcp->th_off) * 4, retc->x.str_val, retc->size);
       return retc;
     }
@@ -673,7 +660,7 @@ set_tcp_v6_elements (lex_ctxt * lexic)
       data = (char *) ((char *) tcp + tcp->th_off * 4);
     }
 
-  npkt = emalloc (40 + tcp->th_off * 4 + data_len);
+  npkt = g_malloc0 (40 + tcp->th_off * 4 + data_len);
   bcopy (pkt, npkt, UNFIX (ip6->ip6_plen) + 40);
 
   ip6 = (struct ip6_hdr *) (npkt);
@@ -708,8 +695,8 @@ set_tcp_v6_elements (lex_ctxt * lexic)
     {
       struct v6pseudohdr pseudoheader;
       char *tcpsumdata =
-        emalloc (sizeof (struct v6pseudohdr) +
-                 (data_len % 2 ? data_len + 1 : data_len));
+        g_malloc0 (sizeof (struct v6pseudohdr) +
+                   (data_len % 2 ? data_len + 1 : data_len));
 
       bzero (&pseudoheader, 38 + sizeof (struct tcphdr));
       memcpy (&pseudoheader.s6addr, &ip6->ip6_src, sizeof (struct in6_addr));
@@ -727,7 +714,7 @@ set_tcp_v6_elements (lex_ctxt * lexic)
       tcp->th_sum =
         np_in_cksum ((unsigned short *) tcpsumdata,
                      38 + sizeof (struct tcphdr) + data_len);
-      efree (&tcpsumdata);
+      g_free (tcpsumdata);
     }
 
   retc = alloc_tree_cell (0, NULL);
@@ -840,7 +827,6 @@ struct v6pseudo_udp_hdr
 {
   struct in6_addr s6addr;
   struct in6_addr d6addr;
-  char nothing;
   char proto;
   unsigned short len;
   struct udphdr udpheader;
@@ -869,7 +855,7 @@ forge_udp_v6_packet (lex_ctxt * lexic)
       struct ip6_hdr *udp_packet;
       struct udphdr *udp;
 
-      pkt = emalloc (sizeof (struct udphdr) + 40 + data_len);
+      pkt = g_malloc0 (sizeof (struct udphdr) + 40 + data_len);
       udp_packet = (struct ip6_hdr *) pkt;
       udp = (struct udphdr *) (pkt + 40);
 
@@ -889,7 +875,7 @@ forge_udp_v6_packet (lex_ctxt * lexic)
         {
           struct v6pseudo_udp_hdr pseudohdr;
           char *udpsumdata =
-            (char *) emalloc (sizeof (struct v6pseudo_udp_hdr) +
+            (char *) g_malloc0 (sizeof (struct v6pseudo_udp_hdr) +
                               (data_len % 2 ? data_len + 1 : data_len));
 
           bzero (&pseudohdr, sizeof (struct v6pseudo_udp_hdr));
@@ -908,7 +894,7 @@ forge_udp_v6_packet (lex_ctxt * lexic)
           udp->uh_sum =
             np_in_cksum ((unsigned short *) udpsumdata,
                          38 + sizeof (struct udphdr) + data_len);
-          efree (&udpsumdata);
+          g_free (udpsumdata);
         }
 
 
@@ -918,7 +904,7 @@ forge_udp_v6_packet (lex_ctxt * lexic)
           if (v != 0)
             {
               udp_packet->ip6_ctlun.ip6_un1.ip6_un1_plen =
-                FIX (ntohs (udp->uh_ulen) + 40);
+                FIX (ntohs (udp->uh_ulen));
             }
         }
 
@@ -949,7 +935,6 @@ get_udp_v6_element (lex_ctxt * lexic)
   tree_cell *retc;
   char *udp;
   char *element;
-  struct ip6_hdr *ip6;
   int ipsz;
   struct udphdr *udphdr;
   int ret;
@@ -964,7 +949,6 @@ get_udp_v6_element (lex_ctxt * lexic)
       printf ("element = get_udp_v6_element(udp:<udp>,element:<element>\n");
       return NULL;
     }
-  ip6 = (struct ip6_hdr *) udp;
 
   if (40 + sizeof (struct udphdr) > ipsz)
     return NULL;
@@ -988,7 +972,7 @@ get_udp_v6_element (lex_ctxt * lexic)
       if (ntohs (udphdr->uh_ulen) - 40 - sizeof (struct udphdr) > ipsz)
         sz = ipsz - 40 - sizeof (struct udphdr);
 
-      retc->x.str_val = emalloc (sz);
+      retc->x.str_val = g_malloc0 (sz);
       retc->size = sz;
       bcopy (udp + 40 + sizeof (struct udphdr), retc->x.str_val, sz);
       return retc;
@@ -1024,7 +1008,7 @@ set_udp_v6_elements (lex_ctxt * lexic)
 
   if (ip6 != NULL)
     {
-      char *pkt = emalloc (sz + data_len);
+      char *pkt;
       struct udphdr *udp;
       tree_cell *retc;
       int old_len;
@@ -1036,19 +1020,19 @@ set_udp_v6_elements (lex_ctxt * lexic)
       if (data != NULL)
         {
           sz = 40 + sizeof (struct udphdr) + data_len;
-          pkt = emalloc (sz);
+          pkt = g_malloc0 (sz);
           bcopy (ip6, pkt, 40 + sizeof (struct udphdr));
         }
       else
         {
-          pkt = emalloc (sz);
+          pkt = g_malloc0 (sz);
           bcopy (ip6, pkt, sz);
         }
 
       ip6 = (struct ip6_hdr *) pkt;
       if (data != NULL)
         {
-          ip6->ip6_ctlun.ip6_un1.ip6_un1_plen = FIX (sz);
+          ip6->ip6_ctlun.ip6_un1.ip6_un1_plen = FIX (sz - 40);
         }
       udp = (struct udphdr *) (pkt + 40);
 
@@ -1089,8 +1073,8 @@ set_udp_v6_elements (lex_ctxt * lexic)
             }
 
           udpsumdata =
-            (char *) emalloc (sizeof (struct v6pseudo_udp_hdr) +
-                              (len % 2 ? len + 1 : len));
+            (char *) g_malloc0 (sizeof (struct v6pseudo_udp_hdr) +
+                                (len % 2 ? len + 1 : len));
           bzero (&pseudohdr, sizeof (struct v6pseudo_udp_hdr));
 
           pseudohdr.proto = IPPROTO_UDP;
@@ -1108,7 +1092,7 @@ set_udp_v6_elements (lex_ctxt * lexic)
             np_in_cksum ((unsigned short *) udpsumdata,
                          38 + sizeof (struct udphdr) + (len % 2 ? len +
                                                         1 : len));
-          efree (&udpsumdata);
+          g_free (udpsumdata);
         }
       retc = alloc_tree_cell (0, NULL);
       retc->type = CONST_DATA;
@@ -1169,7 +1153,6 @@ struct v6pseudo_icmp_hdr
 {
   struct in6_addr s6addr;
   struct in6_addr d6addr;
-  char nothing;
   char proto;
   unsigned short len;
   struct icmp6_hdr icmpheader;
@@ -1214,7 +1197,7 @@ forge_icmp_v6_packet (lex_ctxt * lexic)
         return NULL;
 
       /* ICMP header size is 8 */
-      pkt = emalloc (ip6_sz + 8 + len);
+      pkt = g_malloc0 (ip6_sz + 8 + len);
       ip6_icmp = (struct ip6_hdr *) pkt;
 
       bcopy (ip6, ip6_icmp, ip6_sz);
@@ -1241,9 +1224,9 @@ forge_icmp_v6_packet (lex_ctxt * lexic)
           {
             if (data != NULL)
               bcopy (data, &(p[8]), len);
-            routersolicit = emalloc (sizeof (struct nd_router_solicit));
+            routersolicit = g_malloc0 (sizeof (struct nd_router_solicit));
             pkt =
-              realloc (pkt, ip6_sz + sizeof (struct nd_router_solicit) + len);
+              g_realloc (pkt, ip6_sz + sizeof (struct nd_router_solicit) + len);
             ip6_icmp = (struct ip6_hdr *) pkt;
             p = (char *) (pkt + ip6_sz);
             struct icmp6_hdr *rs = &routersolicit->nd_rs_hdr;
@@ -1259,9 +1242,9 @@ forge_icmp_v6_packet (lex_ctxt * lexic)
           {
             if (data != NULL)
               bcopy (data, &(p[8]), len);
-            routeradvert = emalloc (sizeof (struct nd_router_advert));
+            routeradvert = g_malloc0 (sizeof (struct nd_router_advert));
             /*do we need lifetime?? Not taking lifetime?? */
-            pkt = realloc (pkt, ip6_sz + sizeof (struct nd_router_advert) - 8 + len);   /*not taking lifetime(8 bytes) into consideration */
+            pkt = g_realloc (pkt, ip6_sz + sizeof (struct nd_router_advert) - 8 + len);   /*not taking lifetime(8 bytes) into consideration */
             ip6_icmp = (struct ip6_hdr *) pkt;
             p = (char *) (pkt + ip6_sz);
             struct icmp6_hdr *ra = &routeradvert->nd_ra_hdr;
@@ -1282,9 +1265,9 @@ forge_icmp_v6_packet (lex_ctxt * lexic)
           break;
         case ND_NEIGHBOR_SOLICIT:
           {
-            neighborsolicit = emalloc (sizeof (struct nd_neighbor_solicit));
+            neighborsolicit = g_malloc0 (sizeof (struct nd_neighbor_solicit));
             pkt =
-              realloc (pkt, ip6_sz + sizeof (struct nd_neighbor_solicit) + len);
+              g_realloc (pkt, ip6_sz + sizeof (struct nd_neighbor_solicit) + len);
             ip6_icmp = (struct ip6_hdr *) pkt;
             p = (char *) (pkt + ip6_sz);
             struct icmp6_hdr *ns = &neighborsolicit->nd_ns_hdr;
@@ -1301,9 +1284,9 @@ forge_icmp_v6_packet (lex_ctxt * lexic)
           break;
         case ND_NEIGHBOR_ADVERT:
           {
-            neighboradvert = emalloc (sizeof (struct nd_neighbor_advert));
+            neighboradvert = g_malloc0 (sizeof (struct nd_neighbor_advert));
             pkt =
-              realloc (pkt, ip6_sz + sizeof (struct nd_neighbor_advert) + len);
+              g_realloc (pkt, ip6_sz + sizeof (struct nd_neighbor_advert) + len);
             ip6_icmp = (struct ip6_hdr *) pkt;
             p = (char *) (pkt + 40);
             struct icmp6_hdr *na = &neighboradvert->nd_na_hdr;
@@ -1325,6 +1308,7 @@ forge_icmp_v6_packet (lex_ctxt * lexic)
                   {
                     nasl_perror (lexic,
                                  "forge_icmp_v6_packet: missing 'target' parameter required for constructing response to a Neighbor Solicitation\n");
+                    g_free (ip6_icmp);
                     return NULL;
                   }
               }
@@ -1349,8 +1333,8 @@ forge_icmp_v6_packet (lex_ctxt * lexic)
         {
           struct v6pseudo_icmp_hdr pseudohdr;
           char *icmpsumdata =
-            (char *) emalloc (sizeof (struct v6pseudo_icmp_hdr) +
-                              (len % 2 ? len + 1 : len));
+            (char *) g_malloc0 (sizeof (struct v6pseudo_icmp_hdr) +
+                                (len % 2 ? len + 1 : len));
 
           bzero (&pseudohdr, sizeof (struct v6pseudo_icmp_hdr));
           memcpy (&pseudohdr.s6addr, &ip6->ip6_src, sizeof (struct in6_addr));
@@ -1364,7 +1348,7 @@ forge_icmp_v6_packet (lex_ctxt * lexic)
             bcopy ((char *) data, icmpsumdata + sizeof (pseudohdr), len);
           icmp->icmp6_cksum =
             np_in_cksum ((unsigned short *) icmpsumdata, size);
-          efree (&icmpsumdata);
+          g_free (icmpsumdata);
         }
       else
         icmp->icmp6_cksum =
@@ -1449,7 +1433,7 @@ get_icmp_v6_element (lex_ctxt * lexic)
           retc->type = CONST_DATA;
           retc->size = get_var_size_by_name (lexic, "icmp") - 40 - 8;
           if (retc->size > 0)
-            retc->x.str_val = nasl_strndup (&(p[40 + 8]), retc->size);
+            retc->x.str_val = g_memdup (&(p[40 + 8]), retc->size + 1);
           else
             {
               retc->x.str_val = NULL;
@@ -1501,7 +1485,7 @@ forge_igmp_v6_packet (lex_ctxt * lexic)
     {
       char *data = get_str_local_var_by_name (lexic, "data");
       int len = data ? get_local_var_size_by_name (lexic, "data") : 0;
-      u_char *pkt = emalloc (sizeof (struct igmp6_hdr) + 40 + len);
+      u_char *pkt = g_malloc0 (sizeof (struct igmp6_hdr) + 40 + len);
       struct ip6_hdr *ip6_igmp = (struct ip6_hdr *) pkt;
       struct igmp6_hdr *igmp;
       char *p;
@@ -1574,7 +1558,6 @@ nasl_tcp_v6_ping (lex_ctxt * lexic)
   int i = 0;
   int bpf;
   char filter[255];
-  u_char *pk = NULL;
   tree_cell *retc;
   int opt = 1;
   struct timeval tv;
@@ -1601,17 +1584,14 @@ nasl_tcp_v6_ping (lex_ctxt * lexic)
 
   for (i = 0; ports[i]; i++)
     num_ports++;
-  i = 0;
 
   soc = socket (AF_INET6, SOCK_RAW, IPPROTO_RAW);
   if (soc < 0)
     return NULL;
 
-#ifdef IP_HDRINCL
   if (setsockopt (soc, IPPROTO_IPV6, IP_HDRINCL, (char *) &opt, sizeof (opt)) <
       0)
     perror ("setsockopt");
-#endif
 
   port = get_int_local_var_by_name (lexic, "port", -1);
   if (port == -1)
@@ -1681,7 +1661,7 @@ nasl_tcp_v6_ping (lex_ctxt * lexic)
                   (struct sockaddr *) &soca, sizeof (struct sockaddr_in6));
           tv.tv_sec = 0;
           tv.tv_usec = 100000;
-          if (bpf >= 0 && (pk = bpf_next_tv (bpf, &len, &tv)))
+          if (bpf >= 0 && bpf_next_tv (bpf, &len, &tv))
             flag++;
         }
     }
@@ -1729,11 +1709,9 @@ nasl_send_v6packet (lex_ctxt * lexic)
   if (soc < 0)
     return NULL;
 
-#ifdef IP_HDRINCL
   if (setsockopt
       (soc, IPPROTO_IPV6, IP_HDRINCL, (char *) &offset, sizeof (offset)) < 0)
     perror ("setsockopt");
-#endif
   while ((ip = get_str_var_by_num (lexic, vi)) != NULL)
     {
       int sz = get_var_size_by_num (lexic, vi);
@@ -1789,7 +1767,7 @@ nasl_send_v6packet (lex_ctxt * lexic)
                      &&
                      (!memcmp (answer, (char *) ip, sizeof (struct ip6_hdr))))
                 {
-                  efree (&answer);
+                  g_free (answer);
                   answer =
                     (u_char *) capture_next_v6_packet (bpf, to, &answer_sz);
                 }
