@@ -1,4 +1,4 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2004 Tenable Network Security
  *
@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
@@ -34,24 +34,23 @@
 
 #include <glib.h>
 
-#include "comm.h"               /* for comm_send_status */
 #include "kb.h"                 /* for KB_TYPE_INT */
-#include "plugutils.h"          /* for plug_set_timeout */
-#include "scanners_utils.h"     /* for getpts */
-#include "system.h"             /* for estrdup */
-
-#include "strutils.h"
+#include "plugutils.h"          /* for plug_set_id */
+#include "network.h"            /* for getpts */
 
 #include "nasl_tree.h"
 #include "nasl_global_ctxt.h"
 #include "nasl_func.h"
 #include "nasl_var.h"
 #include "nasl_lex_ctxt.h"
-#include "exec.h"
 
 #include "nasl_debug.h"
 #include "nasl_scanner_glue.h"
 
+#include "../base/nvticache.h"
+
+#include "../misc/prefs.h" /* for prefs_get */
+
 #ifndef NASL_DEBUG
 #define NASL_DEBUG 0
 #endif
@@ -88,13 +87,13 @@ isalldigit (char *str, int len)
 tree_cell *
 script_timeout (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
+  nvti_t *nvti = arg_get_value (lexic->script_infos, "NVTI");
   int to = get_int_var_by_num (lexic, 0, -65535);
 
   if (to == -65535)
     return FAKE_CELL;
 
-  plug_set_timeout (script_infos, to ? to : -1);
+  nvti_set_timeout (nvti, to ? to : -1);
   return FAKE_CELL;
 }
 
@@ -102,12 +101,16 @@ script_timeout (lex_ctxt * lexic)
 tree_cell *
 script_id (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
   int id;
+  char * oid;
 
   id = get_int_var_by_num (lexic, 0, -1);
   if (id > 0)
-    plug_set_id (script_infos, id);
+    {
+      oid = g_strdup_printf ("%s%i", LEGACY_OID, id);
+      nvti_set_oid (arg_get_value (lexic->script_infos, "NVTI"), oid);
+      g_free (oid);
+    }
 
   return FAKE_CELL;
 }
@@ -115,12 +118,8 @@ script_id (lex_ctxt * lexic)
 tree_cell *
 script_oid (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
-  char *oid = get_str_var_by_num (lexic, 0);
-  if (oid != NULL)
-    {
-      plug_set_oid (script_infos, oid);
-    }
+  nvti_set_oid (arg_get_value (lexic->script_infos, "NVTI"),
+                get_str_var_by_num (lexic, 0));
   return FAKE_CELL;
 }
 
@@ -136,7 +135,7 @@ script_cve_id (lex_ctxt * lexic)
 
   for (i = 0; cve != NULL; i++)
     {
-      plug_set_cve_id (script_infos, cve);
+      nvti_add_cve (arg_get_value (script_infos, "NVTI"), cve);
       cve = get_str_var_by_num (lexic, i + 1);
     }
 
@@ -155,7 +154,7 @@ script_bugtraq_id (lex_ctxt * lexic)
 
   for (i = 0; bid != NULL; i++)
     {
-      plug_set_bugtraq_id (script_infos, bid);
+      nvti_add_bid (arg_get_value (script_infos, "NVTI"), bid);
       bid = get_str_var_by_num (lexic, i + 1);
     }
 
@@ -173,8 +172,25 @@ script_xref (lex_ctxt * lexic)
 
   if (value == NULL || name == NULL)
     {
-      fprintf (stderr,
-               "script_xref() syntax error - should be script_xref(name:<name>, value:<value>)\n");
+      nasl_perror (lexic,
+                   "script_xref() syntax error - should be"
+                   " script_xref(name:<name>, value:<value>)\n");
+      if (name == NULL)
+        {
+          nasl_perror (lexic, "  <name> is empty\n");
+        }
+      else
+        {
+          nasl_perror (lexic, "  <name> is %s\n", name);
+        }
+      if (value == NULL)
+        {
+          nasl_perror (lexic, "  <value> is empty)\n");
+        }
+      else
+        {
+          nasl_perror (lexic, "  <value> is %s\n)", value);
+        }
       return FAKE_CELL;
     }
 
@@ -192,8 +208,34 @@ script_tag (lex_ctxt * lexic)
 
   if (value == NULL || name == NULL)
     {
-      fprintf (stderr,
-               "script_tag() syntax error - should be script_tag(name:<name>, value:<value>)\n");
+      nasl_perror (lexic, "script_tag() syntax error - should be"
+                          " script_tag(name:<name>, value:<value>)\n");
+      if (name == NULL)
+        {
+          nasl_perror (lexic, "  <name> is empty\n");
+        }
+      else
+        {
+          nasl_perror (lexic, "  <name> is %s\n", name);
+        }
+      if (value == NULL)
+        {
+          nasl_perror (lexic, "  <value> is empty)\n");
+        }
+      else
+        {
+          nasl_perror (lexic, "  <value> is %s\n)", value);
+        }
+      return FAKE_CELL;
+    }
+
+  /* Special treatment until OpenVAS-5 is retired and risk_factor
+     settings via script_tag() have been removed in all NVTs.
+     The risk_factor is meanwhile computed from CVSS.
+     Dopping it here reduces overhead.
+   */
+  if (!strcmp (name, "risk_factor"))
+    {
       return FAKE_CELL;
     }
 
@@ -203,80 +245,56 @@ script_tag (lex_ctxt * lexic)
 }
 
 
-
-/* UNUSED */
-tree_cell *
-script_see_also (lex_ctxt * lexic)
-{
-  nasl_perror (lexic, "Error - script_see_also() called\n");
-  return FAKE_CELL;
-}
-
-typedef void (*script_register_func_t) (struct arglist *, const char *);
-
-// TODO: This function can be elminiated (functionality back to script_name, etc)
-// once all NASL scripts are being cleared of old I18N concept
-static tree_cell *
-script_elem (lex_ctxt * lexic, script_register_func_t script_register_func)
-{
-  struct arglist *script_infos = lexic->script_infos;
-  char *str;
-
-  str = get_str_local_var_by_name (lexic, "english");
-  if (str == NULL)
-    {
-      str = get_str_var_by_num (lexic, 0);
-      if (str == NULL)
-        return FAKE_CELL;
-    }
-
-  script_register_func (script_infos, str);
-  return FAKE_CELL;
-}
-
-
 tree_cell *
 script_name (lex_ctxt * lexic)
 {
-  return (script_elem (lexic, plug_set_name));
+  nvti_set_name (arg_get_value (lexic->script_infos, "NVTI"),
+                 get_str_var_by_num (lexic, 0));
+  return FAKE_CELL;
 }
 
 
 tree_cell *
 script_version (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
+  nvti_t *nvti = arg_get_value (lexic->script_infos, "NVTI");
 
   char *version = get_str_var_by_num (lexic, 0);
   if (version == NULL)
     {
       nasl_perror (lexic, "Argument error in function script_version()\n");
-      nasl_perror (lexic, "Function usage is : script_version(<name>)\n");
-      nasl_perror (lexic, "Where <name> is the name of another script\n");
+      nasl_perror (lexic, "Function usage is : script_version(<version>)\n");
+      nasl_perror (lexic, "Where <version> is the version of this script\n");
     }
-
   else
-    plug_set_version (script_infos, version);
+    nvti_set_version (nvti, version);
 
   return FAKE_CELL;
 }
 
+/* Can be removed once OpenVA-6 is retired _and_ the command is
+   removed from any NVT */
 tree_cell *
 script_description (lex_ctxt * lexic)
 {
-  return (script_elem (lexic, plug_set_description));
+  /* Does nothing. Kept for backward compatibility. */
+  return FAKE_CELL;
 }
 
 tree_cell *
 script_copyright (lex_ctxt * lexic)
 {
-  return (script_elem (lexic, plug_set_copyright));
+  nvti_set_copyright (arg_get_value (lexic->script_infos, "NVTI"),
+                      get_str_var_by_num (lexic, 0));
+  return FAKE_CELL;
 }
 
 tree_cell *
 script_summary (lex_ctxt * lexic)
 {
-  return (script_elem (lexic, plug_set_summary));
+  nvti_set_summary (arg_get_value (lexic->script_infos, "NVTI"),
+                    get_str_var_by_num (lexic, 0));
+  return FAKE_CELL;
 }
 
 tree_cell *
@@ -292,18 +310,20 @@ script_category (lex_ctxt * lexic)
       nasl_perror (lexic, "Function usage is : script_category(<category>)\n");
       return FAKE_CELL;
     }
-  plug_set_category (script_infos, category);
+  nvti_set_category (arg_get_value (script_infos, "NVTI"), category);
   return FAKE_CELL;
 }
 
 tree_cell *
 script_family (lex_ctxt * lexic)
 {
-  return (script_elem (lexic, plug_set_family));
+  nvti_set_family (arg_get_value (lexic->script_infos, "NVTI"),
+                 get_str_var_by_num (lexic, 0));
+  return FAKE_CELL;
 }
 
 tree_cell *
-script_dependencie (lex_ctxt * lexic)
+script_dependencies (lex_ctxt * lexic)
 {
   struct arglist *script_infos = lexic->script_infos;
   char *dep = get_str_var_by_num (lexic, 0);
@@ -311,8 +331,8 @@ script_dependencie (lex_ctxt * lexic)
 
   if (dep == NULL)
     {
-      nasl_perror (lexic, "Argument error in function script_dependencie()\n");
-      nasl_perror (lexic, "Function usage is : script_dependencie(<name>)\n");
+      nasl_perror (lexic, "Argument error in function script_dependencies()\n");
+      nasl_perror (lexic, "Function usage is : script_dependencies(<name>)\n");
       nasl_perror (lexic, "Where <name> is the name of another script\n");
 
       return FAKE_CELL;
@@ -332,8 +352,6 @@ script_dependencie (lex_ctxt * lexic)
 tree_cell *
 script_require_keys (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
-
   char *keys = get_str_var_by_num (lexic, 0);
   int i;
 
@@ -348,8 +366,7 @@ script_require_keys (lex_ctxt * lexic)
   for (i = 0; keys != NULL; i++)
     {
       keys = get_str_var_by_num (lexic, i);
-      if (keys != NULL)
-        plug_require_key (script_infos, keys);
+      nvti_add_required_keys (arg_get_value (lexic->script_infos, "NVTI"), keys);
     }
 
   return FAKE_CELL;
@@ -358,8 +375,6 @@ script_require_keys (lex_ctxt * lexic)
 tree_cell *
 script_mandatory_keys (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
-
   char *keys = get_str_var_by_num (lexic, 0);
   int i;
 
@@ -376,8 +391,7 @@ script_mandatory_keys (lex_ctxt * lexic)
   for (i = 0; keys != NULL; i++)
     {
       keys = get_str_var_by_num (lexic, i);
-      if (keys != NULL)
-        plug_mandatory_key (script_infos, keys);
+      nvti_add_mandatory_keys (arg_get_value (lexic->script_infos, "NVTI"), keys);
     }
 
   return FAKE_CELL;
@@ -386,18 +400,21 @@ script_mandatory_keys (lex_ctxt * lexic)
 tree_cell *
 script_exclude_keys (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
-
-  int i;
   char *keys = get_str_var_by_num (lexic, 0);
+  int i;
+
+  if (keys == NULL)
+    {
+      nasl_perror (lexic, "Argument error in function script_exclude_keys()\n");
+      nasl_perror (lexic, "Function usage is : script_exclude_keys(<name>)\n");
+      nasl_perror (lexic, "Where <name> is the name of a key\n");
+      return FAKE_CELL;
+    }
 
   for (i = 0; keys != NULL; i++)
     {
       keys = get_str_var_by_num (lexic, i);
-      if (keys != NULL)
-        {
-          plug_exclude_key (script_infos, keys);
-        }
+      nvti_add_excluded_keys (arg_get_value (lexic->script_infos, "NVTI"), keys);
     }
 
   return FAKE_CELL;
@@ -407,7 +424,6 @@ script_exclude_keys (lex_ctxt * lexic)
 tree_cell *
 script_require_ports (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
   char *port;
   int i;
 
@@ -415,7 +431,7 @@ script_require_ports (lex_ctxt * lexic)
     {
       port = get_str_var_by_num (lexic, i);
       if (port != NULL)
-        plug_require_port (script_infos, port);
+        nvti_add_required_ports (arg_get_value (lexic->script_infos, "NVTI"), port);
       else
         break;
     }
@@ -427,7 +443,6 @@ script_require_ports (lex_ctxt * lexic)
 tree_cell *
 script_require_udp_ports (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
   int i;
   char *port;
 
@@ -435,7 +450,7 @@ script_require_udp_ports (lex_ctxt * lexic)
     {
       port = get_str_var_by_num (lexic, i);
       if (port != NULL)
-        plug_require_udp_port (script_infos, port);
+        nvti_add_required_udp_ports (arg_get_value (lexic->script_infos, "NVTI"), port);
       else
         break;
     }
@@ -452,8 +467,10 @@ script_add_preference (lex_ctxt * lexic)
   struct arglist *script_infos = lexic->script_infos;
 
   if (name == NULL || type == NULL || value == NULL)
-    nasl_perror (lexic,
-                 "Argument error in the call to script_add_preference()\n");
+    {
+      nasl_perror (lexic,
+                   "Argument error in the call to script_add_preference()\n");
+    }
   else
     add_plugin_preference (script_infos, name, type, value);
 
@@ -463,7 +480,6 @@ script_add_preference (lex_ctxt * lexic)
 tree_cell *
 script_get_preference (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
   tree_cell *retc;
   char *pref = get_str_var_by_num (lexic, 0);
   char *value;
@@ -477,7 +493,7 @@ script_get_preference (lex_ctxt * lexic)
       return FAKE_CELL;
     }
 
-  value = get_plugin_preference (script_infos, pref);
+  value = get_plugin_preference (lexic->oid, pref);
   if (value != NULL)
     {
       retc = alloc_tree_cell (0, NULL);
@@ -490,7 +506,7 @@ script_get_preference (lex_ctxt * lexic)
         {
           retc->type = CONST_DATA;
           retc->size = strlen (value);
-          retc->x.str_val = estrdup (value);
+          retc->x.str_val = g_strdup (value);
         }
       return retc;
     }
@@ -517,7 +533,7 @@ script_get_preference_file_content (lex_ctxt * lexic)
       return NULL;
     }
 
-  value = get_plugin_preference (script_infos, pref);
+  value = get_plugin_preference (lexic->oid, pref);
   if (value == NULL)
     return NULL;
 
@@ -525,11 +541,10 @@ script_get_preference_file_content (lex_ctxt * lexic)
   if (content == NULL)
     return FAKE_CELL;
   contentsize = get_plugin_preference_file_size (script_infos, value);
-  if (content <= 0)
+  if (contentsize <= 0)
     {
-      nasl_perror (lexic,
-                   "script_get_preference_file_content: could not get size of file from preference %s\n",
-                   pref);
+      nasl_perror (lexic, "script_get_preference_file_content: could not get "
+                          " size of file from preference %s\n", pref);
       return NULL;
     }
 
@@ -551,15 +566,6 @@ script_get_preference_file_location (lex_ctxt * lexic)
   const char *value, *local;
   int len;
 
-  /* Getting the local file name is not dangerous, but
-   * only signed scripts can access files uploaded by the user */
-  if (check_authenticated (lexic) < 0)
-    {
-      nasl_perror (lexic,
-                   "script_get_preference_file_location: script is not authenticated!\n");
-      return NULL;
-    }
-
   if (pref == NULL)
     {
       nasl_perror (lexic,
@@ -567,7 +573,7 @@ script_get_preference_file_location (lex_ctxt * lexic)
       return NULL;
     }
 
-  value = get_plugin_preference (script_infos, pref);
+  value = get_plugin_preference (lexic->oid, pref);
   if (value == NULL)
     {
       nasl_perror (lexic,
@@ -582,7 +588,7 @@ script_get_preference_file_location (lex_ctxt * lexic)
   len = strlen (local);
   retc = alloc_typed_cell (CONST_DATA);
   retc->size = len;
-  retc->x.str_val = emalloc (len + 1);
+  retc->x.str_val = g_malloc0 (len + 1);
   memcpy (retc->x.str_val, local, len + 1);
 
   return retc;
@@ -592,19 +598,10 @@ script_get_preference_file_location (lex_ctxt * lexic)
 tree_cell *
 safe_checks (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
-  struct arglist *prefs = arg_get_value (script_infos, "preferences");
-  char *value;
   tree_cell *retc = alloc_tree_cell (0, NULL);
 
   retc->type = CONST_INT;
-  value = arg_get_value (prefs, "safe_checks");
-  if ((value && !strcmp (value, "yes")))
-    {
-      retc->x.i_val = 1;
-    }
-  else
-    retc->x.i_val = 0;
+  retc->x.i_val = prefs_get_bool ("safe_checks");
 
   return retc;
 }
@@ -653,15 +650,36 @@ network_targets (lex_ctxt * lexic)
   return retc;
 }
 
-/*--------------------[ KB ]---------------------------------------*/
+/**
+ * @brief Return the OID of the current script.
+ *
+ * @param[in] lexic     NASL lexer.
+ *
+ * @return lex cell containing the OID as a string.
+ */
+tree_cell *
+get_script_oid (lex_ctxt * lexic)
+{
+  const char *oid = lexic->oid;
+  tree_cell *retc = NULL;
 
-#define SECRET_KB_PREFIX	"Secret/"
+  if (oid)
+    {
+      retc = alloc_typed_cell (CONST_DATA);
+      retc->x.str_val = g_strdup (oid);
+      retc->size = strlen (oid);
+    }
+
+  return retc;
+}
+
+/*--------------------[ KB ]---------------------------------------*/
 
 tree_cell *
 get_kb_list (lex_ctxt * lexic)
 {
   struct arglist *script_infos = lexic->script_infos;
-  struct kb_item **kb = plug_get_kb (script_infos);
+  kb_t kb = plug_get_kb (script_infos);
   char *kb_mask = get_str_var_by_num (lexic, 0);
   tree_cell *retc;
   int num_elems = 0;
@@ -675,13 +693,11 @@ get_kb_list (lex_ctxt * lexic)
     }
 
   if (kb == NULL)
-    {
-      return NULL;
-    }
+    return NULL;
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = DYN_ARRAY;
-  retc->x.ref_val = a = emalloc (sizeof (nasl_array));
+  retc->x.ref_val = a = g_malloc0 (sizeof (nasl_array));
 
   top = res = kb_item_get_pattern (kb, kb_mask);
 
@@ -690,35 +706,25 @@ get_kb_list (lex_ctxt * lexic)
       anon_nasl_var v;
       bzero (&v, sizeof (v));
 
-      if (lexic->authenticated
-          || strncmp (res->name, SECRET_KB_PREFIX,
-                      sizeof (SECRET_KB_PREFIX) - 1) != 0)
+      if (res->type == KB_TYPE_INT)
         {
-          if (res->type == KB_TYPE_INT)
-            {
-              v.var_type = VAR2_INT;
-              v.v.v_int = res->v.v_int;
-              add_var_to_array (a, res->name, &v);
-              num_elems++;
-            }
-          else if (res->type == KB_TYPE_STR)
-            {
-              v.var_type = VAR2_DATA;
-              v.v.v_str.s_val = (unsigned char *) res->v.v_str;
-              v.v.v_str.s_siz = strlen (res->v.v_str);
-              add_var_to_array (a, res->name, &v);
-              num_elems++;
-            }
+          v.var_type = VAR2_INT;
+          v.v.v_int = res->v_int;
+          add_var_to_array (a, res->name, &v);
+          num_elems++;
+        }
+      else if (res->type == KB_TYPE_STR)
+        {
+          v.var_type = VAR2_DATA;
+          v.v.v_str.s_val = (unsigned char *) res->v_str;
+          v.v.v_str.s_siz = strlen (res->v_str);
+          add_var_to_array (a, res->name, &v);
+          num_elems++;
         }
-#if NASL_DEBUG > 0
-      else
-        nasl_perror (lexic, "get_kb_list: skipping protected KN entry %s\n",
-                     res->name);
-#endif
       res = res->next;
     }
 
-  kb_item_get_all_free (top);
+  kb_item_free (top);
 
   if (num_elems == 0)
     {
@@ -741,16 +747,6 @@ get_kb_item (lex_ctxt * lexic)
   if (kb_entry == NULL)
     return NULL;
 
-  if (!lexic->authenticated
-      && strncmp (kb_entry, SECRET_KB_PREFIX,
-                  sizeof (SECRET_KB_PREFIX) - 1) == 0)
-    {
-      nasl_perror (lexic,
-                   "Untrusted script cannot read protected KB entry %s\n",
-                   kb_entry);
-      return NULL;
-    }
-
   val = plug_get_key (script_infos, kb_entry, &type);
 
 
@@ -763,6 +759,7 @@ get_kb_item (lex_ctxt * lexic)
     {
       retc->type = CONST_INT;
       retc->x.i_val = GPOINTER_TO_SIZE (val);
+      g_free (val);
       return retc;
     }
   else
@@ -771,7 +768,7 @@ get_kb_item (lex_ctxt * lexic)
       if (val != NULL)
         {
           retc->size = strlen (val);
-          retc->x.str_val = estrdup (val);
+          retc->x.str_val = val;
         }
       else
         {
@@ -784,62 +781,12 @@ get_kb_item (lex_ctxt * lexic)
 }
 
 /**
- * Instead of reading the local copy of the KB, we ask the upstream
- * father the "newest" value of a given KB item. This is especially useful
- * when dealing with shared sockets and SSH.
+ * XXX Deprecated call.
  */
 tree_cell *
 get_kb_fresh_item (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
-
-  char *kb_entry = get_str_var_by_num (lexic, 0);
-  char *val;
-  tree_cell *retc;
-  int type;
-
-  if (kb_entry == NULL)
-    return NULL;
-
-  if (!lexic->authenticated
-      && strncmp (kb_entry, SECRET_KB_PREFIX,
-                  sizeof (SECRET_KB_PREFIX) - 1) == 0)
-    {
-      nasl_perror (lexic,
-                   "Untrusted script cannot read protected KB entry %s\n",
-                   kb_entry);
-      return NULL;
-    }
-
-  val = plug_get_fresh_key (script_infos, kb_entry, &type);
-
-
-  if (val == NULL && type == -1)
-    return NULL;
-
-  retc = alloc_tree_cell (0, NULL);
-  if (type == KB_TYPE_INT)
-    {
-      retc->type = CONST_INT;
-      retc->x.i_val = GPOINTER_TO_SIZE (val);
-      return retc;
-    }
-  else
-    {
-      retc->type = CONST_DATA;
-      if (val != NULL)
-        {
-          retc->size = strlen (val);
-          retc->x.str_val = val;        /* Do NOT estrdup() the value, since plug_get_fresh_key() allocated the memory already */
-        }
-      else
-        {
-          retc->size = 0;
-          retc->x.str_val = NULL;
-        }
-    }
-
-  return retc;
+  return get_kb_item (lexic);
 }
 
 tree_cell *
@@ -856,13 +803,6 @@ replace_kb_item (lex_ctxt * lexic)
       return FAKE_CELL;
     }
 
-  if (!lexic->authenticated
-      && strncmp (name, SECRET_KB_PREFIX, sizeof (SECRET_KB_PREFIX) - 1) == 0)
-    {
-      nasl_perror (lexic, "Only signed scripts can set a Secret/ KB entry\n");
-      return FAKE_CELL;
-    }
-
   if (type == VAR2_INT)
     {
       int value = get_int_local_var_by_name (lexic, "value", -1);
@@ -904,14 +844,6 @@ set_kb_item (lex_ctxt * lexic)
       return FAKE_CELL;
     }
 
-  if (!lexic->authenticated
-      && strncmp (name, SECRET_KB_PREFIX, sizeof (SECRET_KB_PREFIX) - 1) == 0)
-    {
-      nasl_perror (lexic, "Only signed scripts can set a Secret/ KB entry\n");
-      return FAKE_CELL;
-    }
-
-
   if (type == VAR2_INT)
     {
       int value = get_int_local_var_by_name (lexic, "value", -1);
@@ -944,12 +876,12 @@ set_kb_item (lex_ctxt * lexic)
 /**
  * Function is used when the script wants to report a problem back to openvassd.
  */
-typedef void (*proto_post_something_t) (struct arglist *, int, const char *,
-                                        const char *);
+typedef void (*proto_post_something_t) (const char *, struct arglist *, int,
+                                        const char *, const char *);
 /**
  * Function is used when the script wants to report a problem back to openvassd.
  */
-typedef void (*post_something_t) (struct arglist *, int, const char *);
+typedef void (*post_something_t) (const char *, struct arglist *, int, const char *);
 
 
 static tree_cell *
@@ -968,7 +900,7 @@ security_something (lex_ctxt * lexic, proto_post_something_t proto_post_func,
       int len = get_local_var_size_by_name (lexic, "data");
       int i;
 
-      dup = nasl_strndup (data, len);
+      dup = g_memdup (data, len + 1);
       for (i = 0; i < len; i++)
         if (dup[i] == 0)
           dup[i] = ' ';
@@ -991,38 +923,33 @@ security_something (lex_ctxt * lexic, proto_post_something_t proto_post_func,
   if (dup != NULL)
     {
       if (proto == NULL)
-        post_func (script_infos, port, dup);
+        post_func (lexic->oid, script_infos, port, dup);
       else
-        proto_post_func (script_infos, port, proto, dup);
+        proto_post_func (lexic->oid, script_infos, port, proto, dup);
 
-      efree (&dup);
+      g_free (dup);
       return FAKE_CELL;
     }
 
   if (proto == NULL)
-    post_func (script_infos, port, NULL);
+    post_func (lexic->oid, script_infos, port, NULL);
   else
-    proto_post_func (script_infos, port, proto, NULL);
+    proto_post_func (lexic->oid, script_infos, port, proto, NULL);
 
   return FAKE_CELL;
 }
 
+/**
+ * @brief Send a security message to the client.
+ *
+ * @param[in]  lexic  NASL lexer.
+ *
+ * @return FAKE_CELL.
+ */
 tree_cell *
-security_hole (lex_ctxt * lexic)
+security_message (lex_ctxt * lexic)
 {
-  return security_something (lexic, proto_post_hole, post_hole);
-}
-
-tree_cell *
-security_warning (lex_ctxt * lexic)
-{
-  return security_something (lexic, proto_post_info, post_info);
-}
-
-tree_cell *
-security_note (lex_ctxt * lexic)
-{
-  return security_something (lexic, proto_post_note, post_note);
+  return security_something (lexic, proto_post_alarm, post_alarm);
 }
 
 tree_cell *
@@ -1032,33 +959,25 @@ log_message (lex_ctxt * lexic)
 }
 
 tree_cell *
-debug_message (lex_ctxt * lexic)
+error_message (lex_ctxt * lexic)
 {
-  return security_something (lexic, proto_post_debug, post_debug);
+  return security_something (lexic, proto_post_error, post_error);
 }
 
 tree_cell *
 nasl_get_preference (lex_ctxt * lexic)
 {
   tree_cell *retc;
-  char *name, *value;
-  struct arglist *script_infos, *prefs;
+  char *name;
+  const char *value;
 
-
-  script_infos = lexic->script_infos;
-  prefs = arg_get_value (script_infos, "preferences");
-  if (prefs == NULL)
-    {
-      nasl_perror (lexic, "get_preference: not preferences\n");
-      return NULL;
-    }
   name = get_str_var_by_num (lexic, 0);
   if (name == NULL)
     {
       nasl_perror (lexic, "get_preference: no name\n");
       return NULL;
     }
-  value = arg_get_value (prefs, name);
+  value = prefs_get (name);
   if (value == NULL)
     return NULL;
 
@@ -1080,9 +999,7 @@ nasl_scanner_get_port (lex_ctxt * lexic)
 {
   tree_cell *retc;
   int idx = get_int_var_by_num (lexic, 0, -1);
-  struct arglist *script_infos = lexic->script_infos;
-  struct arglist *prefs = arg_get_value (script_infos, "preferences");
-  char *prange = arg_get_value (prefs, "port_range");
+  const char *prange = prefs_get ("port_range");
   static int num = 0;
   static u_short *ports = NULL;
 
@@ -1100,7 +1017,7 @@ nasl_scanner_get_port (lex_ctxt * lexic)
 
   if (ports == NULL)
     {
-      ports = (u_short *) getpts (prange, &num);
+      ports = (u_short *) getpts ((char *)prange, &num);
       if (ports == NULL)
         {
           return NULL;
@@ -1138,115 +1055,6 @@ nasl_scanner_add_port (lex_ctxt * lexic)
 tree_cell *
 nasl_scanner_status (lex_ctxt * lexic)
 {
-  int current = get_int_local_var_by_name (lexic, "current", -1);
-  int total = get_int_local_var_by_name (lexic, "total", -1);
-  struct arglist *script_infos = lexic->script_infos;
-  struct arglist *hostdata = arg_get_value (script_infos, "HOSTNAME");
-
-  if (current != -1 && total != -1)
-    {
-      struct arglist *globs = arg_get_value (script_infos, "globals");
-      if (globs == NULL)
-        return NULL;
-      comm_send_status (globs, arg_get_value (hostdata, "NAME"), "portscan",
-                        current, total);
-    }
+  /* Kept for backward compatiblity. */
   return FAKE_CELL;
 }
-
-
-
-/*--------------------[ SHARED SOCKETS ]---------------------------------------*/
-
-#define SECRET_SOCKET_PREFIX "Secret/"
-
-tree_cell *
-nasl_shared_socket_register (lex_ctxt * lexic)
-{
-  char *name = get_str_local_var_by_name (lexic, "name");
-  int soc = get_int_local_var_by_name (lexic, "socket", -1);
-  struct arglist *script_infos = lexic->script_infos;
-
-  if (name == NULL || soc < 0)
-    {
-      fprintf (stderr,
-               "Usage: shared_socket_register(name:<name>, socket:<soc>)\n");
-      return NULL;
-    }
-
-  if (strncmp (name, SECRET_SOCKET_PREFIX, strlen (SECRET_SOCKET_PREFIX)) == 0
-      && check_authenticated (lexic) < 0)
-    return NULL;
-
-
-  shared_socket_register (script_infos, soc, name);
-  return FAKE_CELL;
-}
-
-tree_cell *
-nasl_shared_socket_acquire (lex_ctxt * lexic)
-{
-  char *name = get_str_var_by_num (lexic, 0);
-  int fd;
-  tree_cell *retc;
-  struct arglist *script_infos = lexic->script_infos;
-
-  if (name == NULL)
-    {
-      fprintf (stderr, "Usage: shared_socket_acquire(<name>)\n");
-      return NULL;
-    }
-
-  if (strncmp (name, SECRET_SOCKET_PREFIX, strlen (SECRET_SOCKET_PREFIX)) == 0
-      && check_authenticated (lexic) < 0)
-    return NULL;
-
-  fd = shared_socket_acquire (script_infos, name);
-  if (fd < 0)
-    return NULL;
-  retc = alloc_tree_cell (0, NULL);
-  retc->type = CONST_INT;
-  retc->x.i_val = fd;
-  return retc;
-}
-
-tree_cell *
-nasl_shared_socket_release (lex_ctxt * lexic)
-{
-  char *name = get_str_var_by_num (lexic, 0);
-  struct arglist *script_infos = lexic->script_infos;
-
-  if (name == NULL)
-    {
-      fprintf (stderr, "Usage: shared_socket_release(<name>)\n");
-      return NULL;
-    }
-
-  if (strncmp (name, SECRET_SOCKET_PREFIX, strlen (SECRET_SOCKET_PREFIX)) == 0
-      && check_authenticated (lexic) < 0)
-    return NULL;
-
-  shared_socket_release (script_infos, name);
-  return NULL;
-}
-
-tree_cell *
-nasl_shared_socket_destroy (lex_ctxt * lexic)
-{
-  char *name = get_str_var_by_num (lexic, 0);
-  struct arglist *script_infos = lexic->script_infos;
-
-  if (name == NULL)
-    {
-      fprintf (stderr, "Usage: shared_socket_release(<name>)\n");
-      return NULL;
-    }
-
-  if (strncmp (name, SECRET_SOCKET_PREFIX, strlen (SECRET_SOCKET_PREFIX)) == 0
-      && check_authenticated (lexic) < 0)
-    return NULL;
-
-
-  shared_socket_destroy (script_infos, name);
-  return NULL;
-}
