@@ -50,8 +50,6 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 
-#include <gcrypt.h>
-
 #include "nasl_tree.h"
 #include "nasl_global_ctxt.h"
 #include "nasl_func.h"
@@ -139,6 +137,7 @@ g_string_comma_str (GString *gstr, const char *str)
    easier.  The idea is that you only need to remove these wrappers
    and s/my_ssh_/ssh_/ on this file.  */
 
+#if LIBSSH_VERSION_INT < SSH_VERSION_INT (0, 6, 0)
 struct my_ssh_key_s
 {
   ssh_private_key privkey;
@@ -387,6 +386,7 @@ my_ssh_userauth_publickey(ssh_session session,
   rc = ssh_userauth_pubkey (session, NULL, key->pubkey_string, key->privkey);
   return rc;
 }
+#endif
 
 
 
@@ -491,7 +491,7 @@ nasl_ssh_connect (lex_ctxt *lexic)
 {
   ssh_session session;
   tree_cell *retc;
-  const char *hostname;
+  const char *hostname, *key_type;
   int port, sock;
   int tbl_slot;
   const char *s;
@@ -544,6 +544,24 @@ nasl_ssh_connect (lex_ctxt *lexic)
       return NULL;
     }
 
+  key_type = get_str_local_var_by_name (lexic, "keytype");
+#if LIBSSH_VERSION_INT >= SSH_VERSION_INT (0, 6, 0)
+  if (key_type && ssh_options_set (session, SSH_OPTIONS_HOSTKEYS, key_type))
+    {
+      log_legacy_write ("Failed to set SSH key type '%s': %s",
+                        key_type, ssh_get_error (session));
+      ssh_free (session);
+      return NULL;
+    }
+#else
+  if (key_type)
+    {
+      log_legacy_write ("SSH_OPTIONS_HOSTKEYS not supported");
+      ssh_free (session);
+      return NULL;
+    }
+#endif
+
   if (port)
     {
       unsigned int my_port = port;
@@ -1138,6 +1156,35 @@ nasl_ssh_userauth (lex_ctxt *lexic)
   /* If we have a private key, try public key authentication.  */
   if (privkeystr && *privkeystr && (methods & SSH_AUTH_METHOD_PUBLICKEY))
     {
+#if LIBSSH_VERSION_INT >= SSH_VERSION_INT (0, 6, 0)
+      ssh_key key = NULL;
+
+      if (ssh_pki_import_privkey_base64 (privkeystr, privkeypass, NULL, NULL,
+                                         &key))
+        {
+          if (verbose)
+            log_legacy_write
+             ("SSH public key authentication failed for "
+              "session %d: %s\n", session_id, "Error converting provided key");
+        }
+      else if (ssh_userauth_try_publickey (session, NULL, key)
+               != SSH_AUTH_SUCCESS)
+        {
+          if (verbose)
+            log_legacy_write
+             ("SSH public key authentication failed for "
+              "session %d: %s\n", session_id, "Server does not want our key");
+        }
+      else if (ssh_userauth_publickey (session, NULL, key) == SSH_AUTH_SUCCESS)
+        {
+          retc_val = 0;
+          ssh_key_free (key);
+          goto leave;
+        }
+      ssh_key_free (key);
+
+#else
+
       my_ssh_key key = NULL;
 
       /* SESSION is only used by our emulation - FIXME: remove it for 0.6.  */
@@ -1166,6 +1213,7 @@ nasl_ssh_userauth (lex_ctxt *lexic)
           goto leave;
         }
       my_ssh_key_free (key);
+#endif
       /* Keep on trying.  */
     }
 
@@ -1518,6 +1566,45 @@ nasl_ssh_get_server_banner (lex_ctxt *lexic)
 }
 #endif
 
+/**
+ * @brief Get the host key
+ * @naslfn{ssh_get_host_key}
+ *
+ * The function returns a string with the MD5 host key. *
+ *
+ * @nasluparam
+ *
+ * - An ssh session id.
+ *
+ * @naslret A data block on success or NULL on error.
+ *
+ * @param[in] lexic Lexical context of NASL interpreter.
+ *
+ * @return A string is returned on success.  NULL indicates that the
+ *         connection has not yet been established.
+ */
+tree_cell *
+nasl_ssh_get_host_key (lex_ctxt *lexic)
+{
+  int tbl_slot;
+  ssh_session session;
+  ssh_string sstring;
+  tree_cell *retc;
+
+  if (!find_session_id (lexic, "ssh_get_host_key", &tbl_slot))
+    return NULL;
+  session = session_table[tbl_slot].session;
+
+  sstring = ssh_get_pubkey (session);
+  if (!sstring)
+    return NULL;
+
+  retc = alloc_typed_cell (CONST_DATA);
+  retc->x.str_val = ssh_string_to_char (sstring);
+  retc->size = ssh_string_len (sstring);
+  ssh_string_free (sstring);
+  return retc;
+}
 
 /**
  * @brief Get the list of authmethods
