@@ -1,4 +1,4 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2004 Tenable Network Security
  * Copyright (C) 2009 Greenbone Networks GmbH
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
@@ -24,11 +24,14 @@
  * text-related utilities in the NASL functions.
  */
 
+#define _GNU_SOURCE
+
 #include <ctype.h>              /* for isspace */
 #include <string.h>             /* for strlen */
 #include <unistd.h>             /* for getpid */
-
-#include "system.h"             /* for erealloc */
+#include <string.h>             /* for memmem */
+#include <glib.h>               /* for g_free */
+#include <regex.h>              /* for regex_t */
 
 #include "nasl_tree.h"
 #include "nasl_global_ctxt.h"
@@ -38,7 +41,6 @@
 #include "exec.h"
 
 #include "strutils.h"
-#include "nasl_regex.h"
 #include "nasl_debug.h"
 
 #include "nasl_text_utils.h"
@@ -53,11 +55,10 @@ nasl_string (lex_ctxt * lexic)
   const char *s, *p1;
   char *p2;
 
-
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
   retc->size = 0;
-  retc->x.str_val = emalloc (0);
+  retc->x.str_val = g_malloc0 (1);
 
   vn = array_max_index (&lexic->ctx_vars);
   for (vi = 0; vi < vn; vi++)
@@ -70,7 +71,7 @@ nasl_string (lex_ctxt * lexic)
         sz = strlen (s);
 
       newlen = retc->size + sz;
-      retc->x.str_val = erealloc (retc->x.str_val, newlen + 1);
+      retc->x.str_val = g_realloc (retc->x.str_val, newlen + 1);
       p2 = retc->x.str_val + retc->size;
       p1 = s;
       retc->size = newlen;
@@ -120,8 +121,8 @@ nasl_string (lex_ctxt * lexic)
                       }
                     break;
                   default:
-                    nasl_perror (lexic, "Unknown%d escape sequence '\\%c'\n",
-                                 getpid (), isprint (p1[1]) ? p1[1] : '.');
+                    nasl_perror (lexic, "Unknown escape sequence '\\%c'\n",
+                                 isprint (p1[1]) ? p1[1] : '.');
                     retc->size--;
                     break;
                   }
@@ -147,11 +148,10 @@ nasl_rawstring (lex_ctxt * lexic)
   const char *s;
   int total_len = 0;
 
-
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
   retc->size = 0;
-  retc->x.str_val = emalloc (RAW_STR_LEN);
+  retc->x.str_val = g_malloc0 (RAW_STR_LEN + 1);
 
   vn = array_max_index (&lexic->ctx_vars);
   for (vi = 0; vi < vn && total_len < RAW_STR_LEN - 1; vi++)
@@ -167,7 +167,7 @@ nasl_rawstring (lex_ctxt * lexic)
         }
       else
         {
-          int current_len = sz;
+          int current_len;
           char str[RAW_STR_LEN];
 
           s = get_str_var_by_num (lexic, vi);
@@ -183,7 +183,7 @@ nasl_rawstring (lex_ctxt * lexic)
           /* Should we test if the variable is composed only of digits? */
           if (typ == VAR2_STRING)
             {
-              /* TBD:I should decide at last if we keep those "purified" 
+              /* TBD:I should decide at last if we keep those "purified"
                * string or not, and if we do, if "CONST_STR" & "VAR2_STR" are
                * "not pure" strings */
               for (i = 0, j = 0; i < sz; i++)
@@ -208,7 +208,6 @@ nasl_rawstring (lex_ctxt * lexic)
                       else if (s[i + 1] == 'x' && isxdigit (s[i + 2])
                                && isxdigit (s[i + 3]))
                         {
-                          x = 0;
                           if (isdigit (s[i + 2]))
                             x = (s[i + 2] - '0') * 16;
                           else
@@ -277,11 +276,10 @@ nasl_strcat (lex_ctxt * lexic)
   int vi, vn, newlen;
   int sz;
 
-
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
   retc->size = 0;
-  retc->x.str_val = emalloc (0);
+  retc->x.str_val = g_malloc0 (1);
 
   vn = array_max_index (&lexic->ctx_vars);
   for (vi = 0; vi < vn; vi++)
@@ -294,7 +292,7 @@ nasl_strcat (lex_ctxt * lexic)
         sz = strlen (s);
 
       newlen = retc->size + sz;
-      retc->x.str_val = erealloc (retc->x.str_val, newlen + 1);
+      retc->x.str_val = g_realloc (retc->x.str_val, newlen + 1);
       memcpy (retc->x.str_val + retc->size, s, sz);
       retc->size = newlen;
     }
@@ -314,6 +312,7 @@ nasl_display (lex_ctxt * lexic)
   for (j = 0; j < r->size; j++)
     putchar (isprint (r->x.str_val[j])
              || isspace (r->x.str_val[j]) ? r->x.str_val[j] : '.');
+  fflush (stdout);
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_INT;
   retc->x.i_val = r->size;
@@ -337,7 +336,7 @@ nasl_hex (lex_ctxt * lexic)
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_STR;
   retc->size = strlen (ret);
-  retc->x.str_val = estrdup (ret);
+  retc->x.str_val = g_strdup (ret);
 
   return retc;
 }
@@ -357,7 +356,7 @@ nasl_hexstr (lex_ctxt * lexic)
   if (s == NULL)
     return NULL;
 
-  ret = emalloc (len * 2 + 1);
+  ret = g_malloc0 (len * 2 + 1);
   for (i = 0; i < len; i++)
     {
       /* if i < len there are at least three chars left in ret + 2 * i */
@@ -403,7 +402,7 @@ nasl_tolower (lex_ctxt * lexic)
   if (str == NULL)
     return NULL;
 
-  str = nasl_strndup (str, str_len);
+  str = g_memdup (str, str_len + 1);
   for (i = 0; i < str_len; i++)
     str[i] = tolower (str[i]);
 
@@ -426,7 +425,7 @@ nasl_toupper (lex_ctxt * lexic)
   if (str == NULL)
     return NULL;
 
-  str = nasl_strndup (str, str_len);
+  str = g_memdup (str, str_len + 1);
   for (i = 0; i < str_len; i++)
     str[i] = toupper (str[i]);
 
@@ -464,8 +463,7 @@ nasl_ereg (lex_ctxt * lexic)
   if (pattern == NULL || string == NULL)
     return NULL;
 
-  nasl_re_set_syntax (RE_SYNTAX_POSIX_EGREP);
-  if (nasl_regcomp (&re, pattern, REG_EXTENDED | REG_NOSUB | copt))
+  if (regcomp (&re, pattern, REG_EXTENDED | REG_NOSUB | copt))
     {
       nasl_perror (lexic, "ereg() : regcomp() failed\n");
       return NULL;
@@ -473,7 +471,7 @@ nasl_ereg (lex_ctxt * lexic)
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_INT;
-  string = estrdup (string);
+  string = g_strdup (string);
   if (multiline)
     s = NULL;
   else
@@ -482,7 +480,7 @@ nasl_ereg (lex_ctxt * lexic)
     s[0] = '\0';
   if (s != string)
     {
-      if (nasl_regexec (&re, string, 0, NULL, 0) == 0)
+      if (regexec (&re, string, 0, NULL, 0) == 0)
         retc->x.i_val = 1;
       else
         retc->x.i_val = 0;
@@ -490,8 +488,8 @@ nasl_ereg (lex_ctxt * lexic)
   else
     retc->x.i_val = 0;
 
-  efree (&string);
-  nasl_regfree (&re);
+  g_free (string);
+  regfree (&re);
   return retc;
 }
 
@@ -523,7 +521,7 @@ _regreplace (const char *pattern, const char *replace, const char *string,
     copts = REG_ICASE;
   if (extended)
     copts |= REG_EXTENDED;
-  err = nasl_regcomp (&re, pattern, copts);
+  err = regcomp (&re, pattern, copts);
   if (err)
     {
       return NULL;
@@ -531,8 +529,8 @@ _regreplace (const char *pattern, const char *replace, const char *string,
 
   /* start with a buffer that is twice the size of the stringo
      we're doing replacements in */
-  buf_len = 2 * string_len + 1;
-  buf = emalloc (buf_len * sizeof (char));
+  buf_len = 2 * string_len;
+  buf = g_malloc0 (buf_len + 1);
 
 
   err = pos = 0;
@@ -541,8 +539,8 @@ _regreplace (const char *pattern, const char *replace, const char *string,
   while (!err)
     {
       err =
-        nasl_regexec (&re, &string[pos], (size_t) NS, subs,
-                      (pos ? REG_NOTBOL : 0));
+        regexec (&re, &string[pos], (size_t) NS, subs,
+                 (pos ? REG_NOTBOL : 0));
 
       if (err && err != REG_NOMATCH)
         {
@@ -575,10 +573,10 @@ _regreplace (const char *pattern, const char *replace, const char *string,
 
           if (new_l + 1 > buf_len)
             {
-              buf_len = 1 + buf_len + 2 * new_l;
-              nbuf = emalloc (buf_len);
-              strcpy (nbuf, buf);
-              efree (&buf);
+              buf_len = buf_len + 2 * new_l;
+              nbuf = g_malloc0 (buf_len + 1);
+              strncpy (nbuf, buf, buf_len);
+              g_free (buf);
               buf = nbuf;
             }
           tmp = strlen (buf);
@@ -610,10 +608,10 @@ _regreplace (const char *pattern, const char *replace, const char *string,
               new_l = strlen (buf) + 1;
               if (new_l + 1 > buf_len)
                 {
-                  buf_len = 1 + buf_len + 2 * new_l;
-                  nbuf = emalloc (buf_len * sizeof (char));
-                  strcpy (nbuf, buf);
-                  efree (&buf);
+                  buf_len = buf_len + 2 * new_l;
+                  nbuf = g_malloc0 (buf_len + 1);
+                  strncpy (nbuf, buf, buf_len);
+                  g_free (buf);
                   buf = nbuf;
                 }
               pos += subs[0].rm_eo + 1;
@@ -630,10 +628,10 @@ _regreplace (const char *pattern, const char *replace, const char *string,
           new_l = strlen (buf) + strlen (&string[pos]);
           if (new_l + 1 > buf_len)
             {
-              buf_len = new_l + 1;      /* now we know exactly how long it is */
-              nbuf = emalloc (buf_len * sizeof (char));
-              strcpy (nbuf, buf);
-              efree (&buf);
+              buf_len = new_l;      /* now we know exactly how long it is */
+              nbuf = g_malloc0 (buf_len + 1);
+              strncpy (nbuf, buf, buf_len);
+              g_free (buf);
               buf = nbuf;
             }
           /* stick that last bit of string on our output */
@@ -643,7 +641,7 @@ _regreplace (const char *pattern, const char *replace, const char *string,
     }
 
   buf[new_l] = '\0';
-  nasl_regfree (&re);
+  regfree (&re);
   /* whew. */
   return (buf);
 }
@@ -717,8 +715,8 @@ nasl_egrep (lex_ctxt * lexic)
   else
     copt = 0;
 
-  rets = emalloc (max_size + 1);
-  string = estrdup (string);
+  rets = g_malloc0 (max_size + 2);
+  string = g_strdup (string);
 
 
   s = string;
@@ -733,15 +731,14 @@ nasl_egrep (lex_ctxt * lexic)
     for (;;)
       {
         bzero (&re, sizeof (re));
-        nasl_re_set_syntax (RE_SYNTAX_POSIX_EGREP);
-        if (nasl_regcomp (&re, pattern, REG_EXTENDED | copt))
+        if (regcomp (&re, pattern, REG_EXTENDED | copt))
           {
             nasl_perror (lexic, "egrep() : regcomp() failed\n");
             return NULL;
           }
 
 
-        if (nasl_regexec (&re, s, (size_t) NS, subs, 0) == 0)
+        if (regexec (&re, s, (size_t) NS, subs, 0) == 0)
           {
             char *t = strchr (s, '\n');
 
@@ -754,7 +751,7 @@ nasl_egrep (lex_ctxt * lexic)
               t[0] = '\n';
           }
 
-        nasl_regfree (&re);
+        regfree (&re);
 
         if (t == NULL)
           s = NULL;
@@ -779,12 +776,12 @@ nasl_egrep (lex_ctxt * lexic)
 #ifdef I_WANT_MANY_DIRTY_ERROR_MESSAGES
   if (rets[0] == '\0')
     {
-      efree (&rets);
-      efree (&string);
+      g_free (rets);
+      g_free (string);
       return FAKE_CELL;
     }
 #endif
-  efree (&string);
+  g_free (string);
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
@@ -820,22 +817,21 @@ nasl_eregmatch (lex_ctxt * lexic)
   if (pattern == NULL || string == NULL)
     return NULL;
 
-  nasl_re_set_syntax (RE_SYNTAX_POSIX_EGREP);
-  if (nasl_regcomp (&re, pattern, REG_EXTENDED | copt))
+  if (regcomp (&re, pattern, REG_EXTENDED | copt))
     {
       nasl_perror (lexic, "regmatch() : regcomp() failed\n");
       return NULL;
     }
 
-  if (nasl_regexec (&re, string, (size_t) NS, subs, 0) != 0)
+  if (regexec (&re, string, (size_t) NS, subs, 0) != 0)
     {
-      nasl_regfree (&re);
+      regfree (&re);
       return NULL;
     }
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = DYN_ARRAY;
-  retc->x.ref_val = a = emalloc (sizeof (nasl_array));
+  retc->x.ref_val = a = g_malloc0 (sizeof (nasl_array));
 
   for (i = 0; i < NS; i++)
     if (subs[i].rm_so != -1)
@@ -846,7 +842,7 @@ nasl_eregmatch (lex_ctxt * lexic)
         (void) add_var_to_list (a, i, &v);
       }
 
-  nasl_regfree (&re);
+  regfree (&re);
   return retc;
 }
 
@@ -883,13 +879,13 @@ nasl_substr (lex_ctxt * lexic)
   retc->type = (typ == CONST_STR ? CONST_STR : CONST_DATA);
   if (i1 > i2)
     {
-      retc->x.str_val = emalloc (0);
+      retc->x.str_val = g_malloc0 (1);
       retc->size = 0;
       return retc;
     }
   sz2 = i2 - i1 + 1;
   retc->size = sz2;
-  retc->x.str_val = emalloc (sz2);
+  retc->x.str_val = g_malloc0 (sz2 + 1);
   memcpy (retc->x.str_val, s1 + i1, sz2);
   return retc;
 }
@@ -943,7 +939,7 @@ nasl_insstr (lex_ctxt * lexic)
   else
     sz3 = sz1 + i1 - i2 - 1 + sz2;
 
-  s3 = retc->x.str_val = emalloc (sz3);
+  s3 = retc->x.str_val = g_malloc0 (sz3 + 1);
   retc->size = sz3;
 
   if (i1 <= sz1)
@@ -1025,7 +1021,7 @@ nasl_split (lex_ctxt * lexic)
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = DYN_ARRAY;
-  retc->x.ref_val = a = emalloc (sizeof (nasl_array));
+  retc->x.ref_val = a = g_malloc0 (sizeof (nasl_array));
 
   bzero (&v, sizeof (v));
   v.var_type = VAR2_DATA;
@@ -1036,8 +1032,7 @@ nasl_split (lex_ctxt * lexic)
       j = 0;
       for (;;)
         {
-          if ((p =
-               (char *) nasl_memmem (str + i, len - i, sep, sep_len)) == NULL)
+          if ((p = memmem (str + i, len - i, sep, sep_len)) == NULL)
             {
               v.v.v_str.s_siz = len - i;
               v.v.v_str.s_val = (unsigned char *) str + i;
@@ -1122,10 +1117,9 @@ nasl_chomp (lex_ctxt * lexic)
   if (p != NULL)
     len = (p - str);
 
-  retc->x.str_val = emalloc (len);
+  retc->x.str_val = g_malloc0 (len + 1);
   retc->size = len;
   memcpy (retc->x.str_val, str, len);
-  retc->x.str_val[len] = '\0';
   return retc;
 }
 
@@ -1168,7 +1162,7 @@ nasl_crap (lex_ctxt * lexic)
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA /*CONST_STR */ ;
-  retc->x.str_val = emalloc (len + 1);
+  retc->x.str_val = g_malloc0 (len + 1);
   retc->size = len;
   if (data == NULL)
     memset (retc->x.str_val, 'X', len);
@@ -1211,14 +1205,14 @@ nasl_strstr (lex_ctxt * lexic)
   if (sz_b > sz_a)
     return NULL;
 
-  c = (char *) nasl_memmem (a, sz_a, b, sz_b);
+  c = memmem (a, sz_a, b, sz_b);
   if (c == NULL)
     return FAKE_CELL;
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
   retc->size = sz_a - (c - a);
-  retc->x.str_val = nasl_strndup (c, retc->size);
+  retc->x.str_val = g_memdup (c, retc->size + 1);
   return retc;
 }
 
@@ -1261,7 +1255,7 @@ nasl_stridx (lex_ctxt * lexic)
   if ((sz_a == start) || (sz_b > sz_a + start))
     return retc;
 
-  c = (char *) nasl_memmem (a + start, sz_a - start, b, sz_b);
+  c = memmem (a + start, sz_a - start, b, sz_b);
   if (c != NULL)
     retc->x.i_val = c - a;
   return retc;
@@ -1307,17 +1301,17 @@ nasl_str_replace (lex_ctxt * lexic)
     }
 
   retc = alloc_typed_cell (CONST_DATA);
-  s = emalloc (1);
+  s = g_malloc0 (1);
   sz2 = 0;
   n = 0;
   for (i1 = i2 = 0; i1 <= sz_a - sz_b;)
     {
-      c = (char *) nasl_memmem (a + i1, sz_a - i1, b, sz_b);
+      c = memmem (a + i1, sz_a - i1, b, sz_b);
       if (c == NULL)
         break;
       l = (c - a) - i1;
       sz2 += sz_r + l;
-      s = erealloc (s, sz2 + 1);
+      s = g_realloc (s, sz2 + 1);
       s[sz2] = '\0';
       if (c - a > i1)
         {
@@ -1338,7 +1332,7 @@ nasl_str_replace (lex_ctxt * lexic)
   if (i1 < sz_a)
     {
       sz2 += (sz_a - i1);
-      s = erealloc (s, sz2 + 1);
+      s = g_realloc (s, sz2 + 1);
       s[sz2] = '\0';
       memcpy (s + i2, a + i1, sz_a - i1);
     }
