@@ -1,4 +1,4 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2004 Tenable Network Security
  *
@@ -13,15 +13,16 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
-#include <stdlib.h>             /* for malloc */
+#include <stdlib.h>             /* for abort */
 #include <string.h>             /* for memcpy */
-#include "system.h"             /* for emalloc */
 
-#include "nasl_regex.h"
+#include <glib.h>               /* for g_free */
+
+#include "regex.h"
 
 #include "nasl_tree.h"
 #include "nasl_global_ctxt.h"
@@ -35,14 +36,9 @@
 tree_cell *
 alloc_tree_cell (int lnb, char *s)
 {
-  tree_cell *p = malloc (sizeof (tree_cell));
+  tree_cell *p = g_malloc0 (sizeof (tree_cell));
   int i;
 
-  if (p == NULL)
-    {
-      perror ("malloc");
-      abort ();
-    }
   p->type = 0;
   p->size = 0;
   p->line_nb = lnb;
@@ -64,23 +60,25 @@ alloc_typed_cell (int typ)
 tree_cell *
 alloc_RE_cell (int lnb, int t, tree_cell * l, char *re_str)
 {
-  regex_t *re = emalloc (sizeof (regex_t));
+  regex_t *re = g_malloc0 (sizeof (regex_t));
   int e;
 
   tree_cell *c = alloc_tree_cell (lnb, NULL);
   c->type = t;                  /* We could check the type... */
   c->link[0] = l;
   c->link[1] = FAKE_CELL;
-  e = nasl_regcomp (re, re_str, REG_EXTENDED | REG_NOSUB | REG_ICASE);
+  e = regcomp (re, re_str, REG_EXTENDED | REG_NOSUB | REG_ICASE);
   if (!e)
     c->x.ref_val = re;
   else
     {
-      nasl_perror (NULL, "Line %d: Cannot compile regex: %s (error = %d)\n",
-                   lnb, re_str, e);
-      efree (&re);
+      char errbuf[100];
+      regerror (e, re, errbuf, sizeof (errbuf));
+      nasl_perror (NULL, "Line %d: Cannot compile regex: %s (error %d: %s)\n",
+                   lnb, re_str, e, errbuf);
+      g_free (re);
     }
-  free (re_str);
+  g_free (re_str);
   return c;
 }
 
@@ -112,7 +110,7 @@ dup_cell (const tree_cell * tc)
     {
     case CONST_STR:
     case CONST_DATA:
-      r->x.str_val = emalloc (tc->size);
+      r->x.str_val = g_malloc0 (tc->size + 1);
       memcpy (r->x.str_val, tc->x.str_val, tc->size);
       break;
     default:
@@ -148,7 +146,7 @@ free_tree (tree_cell * c)
         if (c->size > 0)
           memset (c->x.str_val, 0xFF, c->size);
 #endif
-        efree (&c->x.str_val);
+        g_free (c->x.str_val);
         break;
 
       case CONST_REGEX:
@@ -156,8 +154,8 @@ free_tree (tree_cell * c)
       case COMP_RE_NOMATCH:
         if (c->x.ref_val != NULL)
           {
-            nasl_regfree (c->x.ref_val);
-            efree (&c->x.ref_val);
+            regfree (c->x.ref_val);
+            g_free (c->x.ref_val);
           }
         break;
 
@@ -166,7 +164,7 @@ free_tree (tree_cell * c)
         if (a != NULL)
           {
             free_array (a);
-            efree (&c->x.ref_val);
+            g_free (c->x.ref_val);
           }
         break;
 
@@ -177,13 +175,13 @@ free_tree (tree_cell * c)
       case NODE_ARG:
       case NODE_ARRAY_EL:
       case NODE_FOREACH:
-        efree (&c->x.str_val);
+        g_free (c->x.str_val);
         break;
       }
 #ifdef SCRATCH_FREED_MEMORY
   memset (c, 0xFF, sizeof (*c));
 #endif
-  efree (&c);
+  g_free (c);
 }
 
 void
