@@ -9,9 +9,10 @@
  * Copyright:
  * Copyright (c) 2009 Greenbone Networks GmbH, http://www.greenbone.net
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2
- * (or any later version), as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -48,15 +49,57 @@
 #include <netinet/in.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>
-#include "system.h"
-#include "plugutils.h"
+#include <ctype.h>
+#include <inttypes.h>
 
+#include "plugutils.h"
 #include "nasl_wmi.h"
 #include "openvas_wmi_interface.h"
+#include "../misc/openvas_logging.h"
 
 #define IMPORT(var) char *var = get_str_local_var_by_name(lexic, #var)
 #define max 5
 
+/**
+ * Returns 0 if any alphabets are present
+ */
+int check_alpha(char *val)
+{
+  int i, val_len;
+  val_len = strlen(val);
+
+  if((strcmp(val,"-1")) != 0 )
+  {
+    for(i = 0; i < val_len; i++)
+      if(!isdigit(val[i]))
+        return 0;
+  }
+  else
+     return 0;
+
+ return 1;
+}
+
+/** 
+ * Convert string to unsign int 32 bit 
+ */
+uint32_t stoi_uint32_t(char * s)
+{
+  uint32_t v;
+  sscanf(s, "%" PRIu32, &v);
+  return v;
+}
+
+/** 
+ * Convert string to unsign int 64 bit 
+ */
+uint64_t stoi_uint64_t(char * s)
+{
+  uint64_t v;
+  sscanf(s, "%" PRIu64, &v);
+  return v;
+}
+
 /**
  * @brief Get a version string of the WMI implementation.
  *
@@ -71,9 +114,6 @@ nasl_wmi_versioninfo (lex_ctxt * lexic)
   char *version = wmi_versioninfo ();
   tree_cell *retc = alloc_tree_cell (0, NULL);
 
-  if (!retc)
-    return NULL;
-
   if (!version)
     {
       return NULL;
@@ -110,7 +150,6 @@ nasl_wmi_connect (lex_ctxt * lexic)
   struct arglist *script_infos = lexic->script_infos;
   struct in6_addr *host = plug_get_host_ip (script_infos);
   char *ip;
-  char name[512];
   IMPORT (username);
   IMPORT (password);
   IMPORT(ns);
@@ -127,33 +166,24 @@ nasl_wmi_connect (lex_ctxt * lexic)
 
   if ((host == NULL) || (username == NULL) || (password == NULL))
     {
-      fprintf (stderr, "nasl_wmi_connect: Invalid input arguments\n");
+      log_legacy_write ("nasl_wmi_connect: Invalid input arguments\n");
       return NULL;
     }
 
-  if (IN6_IS_ADDR_V4MAPPED (host))
-    {
-      struct in_addr v4_addr;
-      v4_addr.s_addr = host->s6_addr32[3];
-      ip = estrdup (inet_ntoa (v4_addr));
-    }
-  else
-    {
-      ip = estrdup (inet_ntop (AF_INET6, host, name, sizeof (name)));
-    }
-
+  ip = addr6_as_str (host);
   if ((strlen (password) == 0) || (strlen (username) == 0)
         || strlen (ip) == 0)
     {
-      fprintf (stderr, "nasl_wmi_connect: Invalid input arguments\n");
+      log_legacy_write ("nasl_wmi_connect: Invalid input arguments\n");
+      g_free (ip);
       return NULL;
     }
 
-  argv[0] = (char *) emalloc (strlen (argv1));
-  argv[1] = (char *) emalloc (strlen (argv2));
-  argv[2] = (char *) emalloc (strlen (username) + strlen (password) + 1);
-  argv[3] = (char *) emalloc (strlen (ip) + 2);
-  argv[4] = (char *) emalloc (strlen (ns));
+  argv[0] = (char *) g_malloc0 (strlen (argv1));
+  argv[1] = (char *) g_malloc0 (strlen (argv2));
+  argv[2] = (char *) g_malloc0 (strlen (username) + strlen (password) + 1);
+  argv[3] = (char *) g_malloc0 (strlen (ip) + 2);
+  argv[4] = (char *) g_malloc0 (strlen (ns));
 
   // Construct the WMI query
   strcpy (argv[0], argv1);
@@ -164,16 +194,15 @@ nasl_wmi_connect (lex_ctxt * lexic)
   strcpy (argv[3], "//");
   strcat (argv[3], ip);
   strcpy (argv[4], ns);
+  g_free (ip);
 
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_INT;
   value = wmi_connect (argc, argv, &handle);
   if (value == -1)
     {
-      fprintf (stderr, "nasl_wmi_connect: WMI Connect failed\n");
+      log_legacy_write ("nasl_wmi_connect: WMI Connect failed\n");
       return NULL;
     }
 
@@ -201,16 +230,14 @@ nasl_wmi_close (lex_ctxt * lexic)
     return NULL;
 
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_INT;
 
-  if (wmi_close (handle) == 0);
-  {
-    retc->x.i_val = 1;
-    return retc;
-  }
+  if (wmi_close (handle) == 0)
+    {
+      retc->x.i_val = 1;
+      return retc;
+    }
   return NULL;
 }
 
@@ -239,8 +266,6 @@ nasl_wmi_query (lex_ctxt * lexic)
     return NULL;
 
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_DATA;
   retc->x.str_val = NULL;
@@ -250,7 +275,7 @@ nasl_wmi_query (lex_ctxt * lexic)
 
   if ((value == -1) || (res == NULL))
     {
-      fprintf (stderr, "wmi_query: WMI query failed '%s'\n", query);
+      log_legacy_write ("wmi_query: WMI query failed '%s'\n", query);
       return NULL;
     }
 
@@ -284,7 +309,6 @@ nasl_wmi_connect_rsop (lex_ctxt * lexic)
   struct arglist *script_infos = lexic->script_infos;
   struct in6_addr *host = plug_get_host_ip (script_infos);
   char *ip;
-  char name[512];
   IMPORT (username);
   IMPORT (password);
   char *argv[4];
@@ -296,32 +320,23 @@ nasl_wmi_connect_rsop (lex_ctxt * lexic)
 
   if ((host == NULL) || (username == NULL) || (password == NULL))
     {
-      fprintf (stderr, "nasl_wmi_connect_rsop: Invalid input arguments\n");
+      log_legacy_write ("nasl_wmi_connect_rsop: Invalid input arguments\n");
       return NULL;
     }
 
-  if (IN6_IS_ADDR_V4MAPPED (host))
-    {
-      struct in_addr v4_addr;
-      v4_addr.s_addr = host->s6_addr32[3];
-      ip = estrdup (inet_ntoa (v4_addr));
-    }
-  else
-    {
-      ip = estrdup (inet_ntop (AF_INET6, host, name, sizeof (name)));
-    }
-
+  ip = addr6_as_str (host);
   if ((strlen (password) == 0) || (strlen (username) == 0)
       || strlen (ip) == 0)
     {
-      fprintf (stderr, "nasl_wmi_connect_rsop: Invalid input arguments\n");
+      log_legacy_write ("nasl_wmi_connect_rsop: Invalid input arguments\n");
+      g_free (ip);
       return NULL;
     }
 
-  argv[0] = (char *) emalloc (strlen (argv1));
-  argv[1] = (char *) emalloc (strlen (argv2));
-  argv[2] = (char *) emalloc (strlen (username) + strlen (password) + 1);
-  argv[3] = (char *) emalloc (strlen (ip) + 2);
+  argv[0] = (char *) g_malloc0 (strlen (argv1));
+  argv[1] = (char *) g_malloc0 (strlen (argv2));
+  argv[2] = (char *) g_malloc0 (strlen (username) + strlen (password) + 1);
+  argv[3] = (char *) g_malloc0 (strlen (ip) + 2);
 
   // Construct the WMI query
   strcpy (argv[0], argv1);
@@ -331,16 +346,15 @@ nasl_wmi_connect_rsop (lex_ctxt * lexic)
   strcat (argv[2], password);
   strcpy (argv[3], "//");
   strcat (argv[3], ip);
+  g_free (ip);
 
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_INT;
   value = wmi_connect_rsop (argc, argv, &handle);
   if (value == -1)
     {
-      fprintf (stderr, "nasl_wmi_connect_rsop: WMI Connect failed\n");
+      log_legacy_write ("nasl_wmi_connect_rsop: WMI Connect failed\n");
       return NULL;
     }
 
@@ -371,8 +385,6 @@ nasl_wmi_query_rsop (lex_ctxt * lexic)
   char *res = NULL;
   int value;
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_DATA;
   retc->x.str_val = NULL;
@@ -381,7 +393,7 @@ nasl_wmi_query_rsop (lex_ctxt * lexic)
   value = wmi_query_rsop (handle, query, &res);
   if ((value == -1) || (res == NULL))
     {
-      fprintf (stderr, "wmi_query_rsop: WMI query failed\n");
+      log_legacy_write ("wmi_query_rsop: WMI query failed\n");
       return NULL;
     }
   retc->x.str_val = strdup (res);
@@ -414,7 +426,6 @@ nasl_wmi_connect_reg (lex_ctxt * lexic)
   struct arglist *script_infos = lexic->script_infos;
   struct in6_addr *host = plug_get_host_ip (script_infos);
   char *ip;
-  char name[512];
   IMPORT (username);
   IMPORT (password);
   char *argv[4];
@@ -426,32 +437,23 @@ nasl_wmi_connect_reg (lex_ctxt * lexic)
 
   if ((host == NULL) || (username == NULL) || (password == NULL))
     {
-      fprintf (stderr, "nasl_wmi_connect_reg: Invalid input arguments\n");
+      log_legacy_write ("nasl_wmi_connect_reg: Invalid input arguments\n");
       return NULL;
     }
 
-  if (IN6_IS_ADDR_V4MAPPED (host))
-    {
-      struct in_addr v4_addr;
-      v4_addr.s_addr = host->s6_addr32[3];
-      ip = estrdup (inet_ntoa (v4_addr));
-    }
-  else
-    {
-      ip = estrdup (inet_ntop (AF_INET6, host, name, sizeof (name)));
-    }
-
+  ip = addr6_as_str (host);
   if ((strlen (password) == 0) || (strlen (username) == 0)
       || strlen (ip) == 0)
     {
-      fprintf (stderr, "nasl_wmi_connect_reg: Invalid input arguments\n");
+      log_legacy_write ("nasl_wmi_connect_reg: Invalid input arguments\n");
+      g_free (ip);
       return NULL;
     }
 
-  argv[0] = (char *) emalloc (strlen (argv1));
-  argv[1] = (char *) emalloc (strlen (argv2));
-  argv[2] = (char *) emalloc (strlen (username) + strlen (password) + 1);
-  argv[3] = (char *) emalloc (strlen (ip) + 2);
+  argv[0] = (char *) g_malloc0 (strlen (argv1));
+  argv[1] = (char *) g_malloc0 (strlen (argv2));
+  argv[2] = (char *) g_malloc0 (strlen (username) + strlen (password) + 1);
+  argv[3] = (char *) g_malloc0 (strlen (ip) + 2);
 
   // Construct the WMI query
   strcpy (argv[0], argv1);
@@ -461,16 +463,15 @@ nasl_wmi_connect_reg (lex_ctxt * lexic)
   strcat (argv[2], password);
   strcpy (argv[3], "//");
   strcat (argv[3], ip);
+  g_free (ip);
 
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_INT;
   value = wmi_connect_reg (argc, argv, &handle);
   if (value == -1)
     {
-      fprintf (stderr, "nasl_wmi_connect_reg: WMI Connect failed\n");
+      log_legacy_write ("nasl_wmi_connect_reg: WMI Connect failed\n");
       return NULL;
     }
 
@@ -506,8 +507,6 @@ nasl_wmi_reg_get_sz (lex_ctxt * lexic)
   char *res = NULL;
   int value;
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_DATA;
   retc->x.str_val = NULL;
@@ -517,7 +516,7 @@ nasl_wmi_reg_get_sz (lex_ctxt * lexic)
 
   if ((value == -1) || (res == NULL))
     {
-      fprintf (stderr, "nasl_wmi_reg_get_sz: WMI Registry get failed\n");
+      log_legacy_write ("nasl_wmi_reg_get_sz: WMI Registry get failed\n");
       return NULL;
     }
 
@@ -554,8 +553,6 @@ nasl_wmi_reg_enum_value (lex_ctxt * lexic)
   char *res = NULL;
   int value;
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_DATA;
   retc->x.str_val = NULL;
@@ -565,7 +562,7 @@ nasl_wmi_reg_enum_value (lex_ctxt * lexic)
 
   if ((value == -1) || (res == NULL))
     {
-      fprintf (stderr, "nasl_wmi_reg_enum_value: WMI query failed\n");
+      log_legacy_write ("nasl_wmi_reg_enum_value: WMI query failed\n");
       return NULL;
     }
 
@@ -602,8 +599,6 @@ nasl_wmi_reg_enum_key (lex_ctxt * lexic)
   char *res = NULL;
   int value;
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_DATA;
   retc->x.str_val = NULL;
@@ -613,7 +608,7 @@ nasl_wmi_reg_enum_key (lex_ctxt * lexic)
 
   if ((value == -1) || (res == NULL))
     {
-      fprintf (stderr, "nasl_wmi_reg_enum_key: WMI query failed\n");
+      log_legacy_write ("nasl_wmi_reg_enum_key: WMI query failed\n");
       return NULL;
     }
 
@@ -652,8 +647,6 @@ nasl_wmi_reg_get_bin_val (lex_ctxt * lexic)
   int value;
 
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_DATA;
   retc->x.str_val = NULL;
@@ -663,7 +656,7 @@ nasl_wmi_reg_get_bin_val (lex_ctxt * lexic)
 
   if ((value == -1) || (res == NULL))
     {
-      fprintf (stderr, "nasl_wmi_reg_get_bin_val: WMI query failed\n");
+      log_legacy_write ("nasl_wmi_reg_get_bin_val: WMI query failed\n");
       return NULL;
     }
 
@@ -701,8 +694,6 @@ nasl_wmi_reg_get_dword_val (lex_ctxt * lexic)
   int value;
 
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_DATA;
   retc->x.str_val = NULL;
@@ -715,7 +706,7 @@ nasl_wmi_reg_get_dword_val (lex_ctxt * lexic)
 
   if ((value == -1) || (res == NULL))
     {
-      fprintf (stderr, "nasl_wmi_reg_get_dword_val: WMI query failed\n");
+      log_legacy_write ("nasl_wmi_reg_get_dword_val: WMI query failed\n");
       return NULL;
     }
 
@@ -753,8 +744,6 @@ nasl_wmi_reg_get_ex_string_val (lex_ctxt * lexic)
   int value;
 
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_DATA;
   retc->x.str_val = NULL;
@@ -764,7 +753,7 @@ nasl_wmi_reg_get_ex_string_val (lex_ctxt * lexic)
 
   if ((value == -1) || (res == NULL))
     {
-      fprintf (stderr, "nasl_wmi_reg_get_ex_string_val: WMI query failed\n");
+      log_legacy_write ("nasl_wmi_reg_get_ex_string_val: WMI query failed\n");
       return NULL;
     }
 
@@ -802,8 +791,6 @@ nasl_wmi_reg_get_mul_string_val (lex_ctxt * lexic)
   int value;
 
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_DATA;
   retc->x.str_val = NULL;
@@ -813,7 +800,7 @@ nasl_wmi_reg_get_mul_string_val (lex_ctxt * lexic)
 
   if ((value == -1) || (res == NULL))
     {
-      fprintf (stderr, "nasl_wmi_reg_get_mul_string_val: WMI query failed\n");
+      log_legacy_write ("nasl_wmi_reg_get_mul_string_val: WMI query failed\n");
       return NULL;
     }
 
@@ -832,7 +819,7 @@ nasl_wmi_reg_get_mul_string_val (lex_ctxt * lexic)
  *
  * Retrieves local variables "wmi_handle", "hive", "key", "val_name"
  * from the lexical context, performs the registry operation
- * querying Expanded string value.
+ * querying 64-bit unsigned integer.
  */
 tree_cell *
 nasl_wmi_reg_get_qword_val (lex_ctxt * lexic)
@@ -851,8 +838,6 @@ nasl_wmi_reg_get_qword_val (lex_ctxt * lexic)
   int value;
 
   tree_cell *retc = alloc_tree_cell (0, NULL);
-  if (!retc)
-    return NULL;
 
   retc->type = CONST_DATA;
   retc->x.str_val = NULL;
@@ -862,7 +847,7 @@ nasl_wmi_reg_get_qword_val (lex_ctxt * lexic)
 
   if ((value == -1) || (res == NULL))
     {
-      fprintf (stderr, "nasl_wmi_reg_get_qword_val: WMI query failed\n");
+      log_legacy_write ("nasl_wmi_reg_get_qword_val: WMI query failed\n");
       return NULL;
     }
 
@@ -870,3 +855,276 @@ nasl_wmi_reg_get_qword_val (lex_ctxt * lexic)
   retc->size = strlen (res);
   return retc;
 }
+
+/**
+ * @brief Set Registry DWORD value.
+ *
+ * @param[in] lexic Lexical context of NASL interpreter.
+ *
+ * @return NULL on failure
+ *
+ * Retrieves local variables "wmi_handle", "key", "val_name", "val"
+ * from the lexical context, performs the registry set/create operation
+ * for double word data type.
+ *
+ * It will work only if the key exist
+ */
+tree_cell *
+nasl_wmi_reg_set_dword_val (lex_ctxt * lexic)
+{
+  WMI_HANDLE handle =
+    (WMI_HANDLE) get_int_local_var_by_name (lexic, "wmi_handle", 0);
+
+  if (!handle)
+    return NULL;
+
+  char *key = get_str_local_var_by_name (lexic, "key"); // REGISTRY KEY
+  char *val_name = get_str_local_var_by_name (lexic, "val_name");       // REGISTRY VALUE NAME
+  char *val = get_str_local_var_by_name (lexic, "val");  //REGISTERY VALUE TO SET
+
+  uint32_t val1;
+  int value;
+
+  // Return NULL if any alphabet is present
+  if (check_alpha(val) == 0)
+    return NULL;
+
+  // Convert string to proper 64 bit integer
+  val1 = stoi_uint32_t(val);
+
+  tree_cell *retc = alloc_tree_cell (0, NULL);
+
+  retc->type = CONST_INT;
+  retc->x.i_val = 1;
+
+  value = wmi_reg_set_dword_val (handle, key, val_name, val1);
+
+  if (value == -1)
+    {
+      log_legacy_write ("nasl_wmi_reg_set_dword_val: WMI registry set"
+                        " operation failed\n");
+      return NULL;
+    }
+  return retc;
+}
+
+/**
+ * @brief Set Registry QWORD value.
+ *
+ * @param[in] lexic Lexical context of NASL interpreter.
+ *
+ * @return NULL on failure
+ *
+ * Retrieves local variables "wmi_handle", "key", "val_name", "val"
+ * from the lexical context, performs the registry set/create operation
+ * for 64-bit unsigned integer.
+ *
+ * It will work only if the key exist
+ */
+tree_cell *
+nasl_wmi_reg_set_qword_val (lex_ctxt * lexic)
+{
+  WMI_HANDLE handle =
+    (WMI_HANDLE) get_int_local_var_by_name (lexic, "wmi_handle", 0);
+
+  if (!handle)
+    return NULL;
+
+  char *key = get_str_local_var_by_name (lexic, "key"); // REGISTRY KEY
+  char *val_name = get_str_local_var_by_name (lexic, "val_name");       // REGISTRY VALUE NAME
+  char *val = get_str_local_var_by_name (lexic, "val");  //REGISTERY VALUE TO SET
+
+  uint64_t val1;
+  int value;
+
+  // Return if alphabets present
+  if (check_alpha(val) == 0)
+    return NULL;
+
+  // Convert string to proper integer
+  val1 = stoi_uint64_t(val);
+
+  tree_cell *retc = alloc_tree_cell (0, NULL);
+
+  retc->type = CONST_INT;
+  retc->x.i_val = 1;
+
+  value = wmi_reg_set_qword_val (handle, key, val_name, val1);
+
+  if (value == -1)
+    {
+      log_legacy_write ("nasl_wmi_reg_set_qword_val: WMI register"
+                        " set operation failed\n");
+      return NULL;
+    }
+  return retc;
+}
+
+/**
+ * @brief Set Registry Expanded string value.
+ *
+ * @param[in] lexic Lexical context of NASL interpreter.
+ *
+ * @return NULL on failure
+ *
+ * Retrieves local variables "wmi_handle", "key", "val_name", "val"
+ * from the lexical context, performs the registry set/create operation
+ * for string value.
+ *
+ * It will work only if the key exist
+ */
+tree_cell *
+nasl_wmi_reg_set_ex_string_val (lex_ctxt * lexic)
+{
+  WMI_HANDLE handle =
+    (WMI_HANDLE) get_int_local_var_by_name (lexic, "wmi_handle", 0);
+
+  if (!handle)
+    return NULL;
+
+  char *key = get_str_local_var_by_name (lexic, "key"); // REGISTRY KEY
+  char *val_name = get_str_local_var_by_name (lexic, "val_name");       // REGISTRY VALUE NAME
+  char *val = get_str_local_var_by_name (lexic, "val");  //REGISTERY VALUE TO SET
+
+  int value;
+
+  tree_cell *retc = alloc_tree_cell (0, NULL);
+
+  retc->type = CONST_INT;
+  retc->x.i_val = 1;
+
+  value = wmi_reg_set_ex_string_val (handle, key, val_name, val);
+
+  if (value == -1)
+    {
+      log_legacy_write ("nasl_wmi_reg_set_ex_string_val: WMI registery set operation failed\n");
+      return NULL;
+    }
+  return retc;
+}
+
+/**
+ * @brief Set Registry string value.
+ *
+ * @param[in] lexic Lexical context of NASL interpreter.
+ *
+ * @return NULL on failure
+ *
+ * Retrieves local variables "wmi_handle", "key", "val_name", "val"
+ * from the lexical context, performs the registry set/create operation
+ * for string value.
+ *
+ * It will work only if the key exist
+ */
+tree_cell *
+nasl_wmi_reg_set_string_val (lex_ctxt * lexic)
+{
+  WMI_HANDLE handle =
+    (WMI_HANDLE) get_int_local_var_by_name (lexic, "wmi_handle", 0);
+
+  if (!handle)
+    return NULL;
+
+  char *key = get_str_local_var_by_name (lexic, "key"); // REGISTRY KEY
+  char *val_name = get_str_local_var_by_name (lexic, "val_name");       // REGISTRY VALUE NAME
+  char *val = get_str_local_var_by_name (lexic, "val");  //REGISTERY VALUE TO SET
+
+  int value;
+
+  tree_cell *retc = alloc_tree_cell (0, NULL);
+
+  retc->type = CONST_INT;
+  retc->x.i_val = 1;
+
+  value = wmi_reg_set_string_val (handle, key, val_name, val);
+
+  if (value == -1)
+    {
+      log_legacy_write ("nasl_wmi_reg_set_string_val: WMI registery"
+                        " set operation failed\n");
+      return NULL;
+    }
+    return retc;
+}
+
+/**
+ * @brief Create Registry key.
+ *
+ * @param[in] lexic Lexical context of NASL interpreter.
+ *
+ * @return NULL on failure
+ *
+ * Retrieves local variables "wmi_handle", "key"
+ * from the lexical context, performs the registry create operation
+ * for the key.
+ */
+tree_cell *
+nasl_wmi_reg_create_key (lex_ctxt * lexic)
+{
+  WMI_HANDLE handle =
+    (WMI_HANDLE) get_int_local_var_by_name (lexic, "wmi_handle", 0);
+
+  if (!handle)
+    return NULL;
+
+  char *key = get_str_local_var_by_name (lexic, "key"); // REGISTRY KEY
+
+  int value;
+
+  tree_cell *retc = alloc_tree_cell (0, NULL);
+
+  retc->type = CONST_INT;
+  retc->x.i_val = 1;
+
+  value = wmi_reg_create_key (handle, key);
+
+  if (value == -1)
+    {
+      log_legacy_write ("nasl_wmi_reg_create_key: WMI registery key create"
+                        " operation failed\n");
+      return NULL;
+    }
+    return retc;
+}
+
+/**
+ * @brief Delete Registry key.
+ *
+ * @param[in] lexic Lexical context of NASL interpreter.
+ *
+ * @return NULL on failure
+ *
+ * Retrieves local variables "wmi_handle", "key"
+ * from the lexical context, performs the registry delete operation
+ * for the key.
+ *
+ * It will work only if the key exist
+ */
+tree_cell *
+nasl_wmi_reg_delete_key (lex_ctxt * lexic)
+{
+  WMI_HANDLE handle =
+    (WMI_HANDLE) get_int_local_var_by_name (lexic, "wmi_handle", 0);
+
+  if (!handle)
+    return NULL;
+
+  char *key = get_str_local_var_by_name (lexic, "key"); // REGISTRY KEY
+
+  int value;
+
+  tree_cell *retc = alloc_tree_cell (0, NULL);
+
+  retc->type = CONST_INT;
+  retc->x.i_val = 1;
+
+  value = wmi_reg_delete_key (handle, key);
+
+  if (value == -1)
+    {
+      log_legacy_write ("nasl_wmi_reg_delete_key: WMI registery key"
+                        " delete operation failed\n");
+      return NULL;
+    }
+    return retc;
+}
