@@ -8,10 +8,10 @@
  * Copyright:
  * Copyright (C) 2009 Greenbone Networks GmbH
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2,
- * or, at your option, any later version as published by the Free
- * Software Foundation
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,7 +40,6 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <glib.h>
-#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
@@ -140,26 +139,6 @@ add_entity (entities_t * entities, const char *name, const char *text)
   return entity;
 }
 
-/**
- * @brief Add an attribute to an XML entity.
- *
- * @param[in]  entity  The entity.
- * @param[in]  name    Name of the attribute.  Copied, copy is freed by
- *                     free_entity.
- * @param[in]  value   Text of the attribute.  Copied, copy is freed by
- *                     free_entity.
- *
- * @return The new entity.
- */
-void
-add_attribute (entity_t entity, const char *name, const char *value)
-{
-  if (entity->attributes == NULL)
-    entity->attributes =
-      g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-  g_hash_table_insert (entity->attributes, g_strdup (name), g_strdup (value));
-}
-
 /**
  * @brief Free an entity, recursively.
  *
@@ -170,8 +149,8 @@ free_entity (entity_t entity)
 {
   if (entity)
     {
-      free (entity->name);
-      free (entity->text);
+      g_free (entity->name);
+      g_free (entity->text);
       if (entity->attributes)
         g_hash_table_destroy (entity->attributes);
       if (entity->entities)
@@ -184,6 +163,7 @@ free_entity (entity_t entity)
             }
           g_slist_free (entity->entities);
         }
+      g_free (entity);
     }
 }
 
@@ -265,21 +245,6 @@ entity_attribute (entity_t entity, const char *name)
   return NULL;
 }
 
-/**
- * @brief Buffer for reading from the manager.
- */
-char buffer_start[BUFFER_SIZE];
-
-/**
- * @brief Current position in the manager reading buffer.
- */
-char *buffer_point = buffer_start;
-
-/**
- * @brief End of the manager reading buffer.
- */
-char *buffer_end = buffer_start + BUFFER_SIZE;
-
 /**
  * @brief Add attributes from an XML callback to an entity.
  *
@@ -389,8 +354,14 @@ handle_text (GMarkupParseContext * context, const gchar * text, gsize text_len,
 {
   context_data_t *data = (context_data_t *) user_data;
   entity_t current = (entity_t) data->current->data;
-  current->text =
-    current->text ? g_strconcat (current->text, text, NULL) : g_strdup (text);
+  if (current->text)
+    {
+      gchar *old = current->text;
+      current->text = g_strconcat (current->text, text, NULL);
+      g_free (old);
+    }
+  else
+    current->text = g_strdup (text);
 }
 
 /**
@@ -433,11 +404,16 @@ try_read_entity_and_string (gnutls_session_t * session, int timeout,
   int socket;
   time_t last_time;
 
+  // Buffer for reading from the manager.
+  char buffer_start[BUFFER_SIZE];
+  // End of the manager reading buffer.
+  char *buffer_end = buffer_start + BUFFER_SIZE;
+
   /* Record the start time. */
 
   if (time (&last_time) == -1)
     {
-      g_message ("   failed to get current time: %s\n",
+      g_warning ("   failed to get current time: %s\n",
                  strerror (errno));
       return -1;
     }
@@ -488,7 +464,7 @@ try_read_entity_and_string (gnutls_session_t * session, int timeout,
       ssize_t count;
       while (1)
         {
-          g_message ("   asking for %i\n", (int) (buffer_end - buffer_start));
+          g_debug ("   asking for %i\n", (int) (buffer_end - buffer_start));
           count =
             gnutls_record_recv (*session, buffer_start,
                                 buffer_end - buffer_start);
@@ -503,8 +479,9 @@ try_read_entity_and_string (gnutls_session_t * session, int timeout,
                   if ((timeout - (time (NULL) - last_time))
                       <= 0)
                     {
-                      g_message ("   timeout\n");
+                      g_warning ("   timeout\n");
                       fcntl (socket, F_SETFL, 0L);
+                      g_markup_parse_context_free (xml_context);
                       return -4;
                     }
                   continue;
@@ -521,6 +498,7 @@ try_read_entity_and_string (gnutls_session_t * session, int timeout,
                 g_string_free (string, TRUE);
               if (timeout > 0)
                 fcntl (socket, F_SETFL, 0L);
+              g_markup_parse_context_free (xml_context);
               return -1;
             }
           if (count == 0)
@@ -529,7 +507,7 @@ try_read_entity_and_string (gnutls_session_t * session, int timeout,
               g_markup_parse_context_end_parse (xml_context, &error);
               if (error)
                 {
-                  g_message ("   End error: %s\n", error->message);
+                  g_warning ("   End error: %s\n", error->message);
                   g_error_free (error);
                 }
               if (context_data.first && context_data.first->data)
@@ -541,12 +519,13 @@ try_read_entity_and_string (gnutls_session_t * session, int timeout,
                 g_string_free (string, TRUE);
               if (timeout > 0)
                 fcntl (socket, F_SETFL, 0L);
+              g_markup_parse_context_free (xml_context);
               return -3;
             }
           break;
         }
 
-      g_message ("<= %.*s\n", (int) count, buffer_start);
+      g_debug ("<= %.*s\n", (int) count, buffer_start);
 
       if (string)
         g_string_append_len (string, buffer_start, count);
@@ -564,6 +543,7 @@ try_read_entity_and_string (gnutls_session_t * session, int timeout,
             g_string_free (string, TRUE);
           if (timeout > 0)
             fcntl (socket, F_SETFL, 0L);
+          g_markup_parse_context_free (xml_context);
           return -2;
         }
       if (context_data.done)
@@ -571,7 +551,7 @@ try_read_entity_and_string (gnutls_session_t * session, int timeout,
           g_markup_parse_context_end_parse (xml_context, &error);
           if (error)
             {
-              g_message ("   End error: %s\n", error->message);
+              g_warning ("   End error: %s\n", error->message);
               g_error_free (error);
               if (context_data.first && context_data.first->data)
                 {
@@ -580,6 +560,7 @@ try_read_entity_and_string (gnutls_session_t * session, int timeout,
                 }
               if (timeout > 0)
                 fcntl (socket, F_SETFL, 0L);
+              g_markup_parse_context_free (xml_context);
               return -2;
             }
           *entity = (entity_t) context_data.first->data;
@@ -587,14 +568,16 @@ try_read_entity_and_string (gnutls_session_t * session, int timeout,
             *string_return = string;
           if (timeout > 0)
             fcntl (socket, F_SETFL, 0L);
+          g_markup_parse_context_free (xml_context);
           return 0;
         }
 
       if ((timeout > 0) && (time (&last_time) == -1))
         {
-          g_message ("   failed to get current time (1): %s\n",
+          g_warning ("   failed to get current time (1): %s\n",
                      strerror (errno));
           fcntl (socket, F_SETFL, 0L);
+          g_markup_parse_context_free (xml_context);
           return -1;
         }
     }
@@ -644,7 +627,11 @@ read_entity_and_text (gnutls_session_t * session, entity_t * entity,
       GString *string = NULL;
       int ret = read_entity_and_string (session, entity, &string);
       if (ret)
-        return ret;
+        {
+          if (string)
+            g_string_free (string, TRUE);
+          return ret;
+        }
       *text = g_string_free (string, FALSE);
       return 0;
     }
@@ -680,7 +667,7 @@ read_string (gnutls_session_t * session, GString ** string)
  *                        wait forever.
  * @param[out]  entity    Pointer to an entity tree.
  *
- * @return 0 success, -1 read error, -2 parse error, -3 end of file.
+ * @return 0 success, -1 read error, -2 parse error, -3 end of file, -4 timeout.
  */
 int
 try_read_entity (gnutls_session_t * session, int timeout, entity_t * entity)
@@ -753,7 +740,7 @@ parse_entity (const char *string, entity_t * entity)
       g_markup_parse_context_end_parse (xml_context, &error);
       if (error)
         {
-          g_message ("   End error: %s\n", error->message);
+          g_warning ("   End error: %s\n", error->message);
           g_error_free (error);
           if (context_data.first && context_data.first->data)
             {
@@ -824,18 +811,6 @@ print_entity_to_string (entity_t entity, GString * string)
   g_string_append_printf (string, "</%s>", entity->name);
 }
 
-/**
- * @brief Print an XML entity tree to string.
- *
- * @param[in]  string    The string to which to print.
- * @param[in]  entities  The entities.
- */
-void
-print_entities_to_string (GString * string, entities_t entities)
-{
-  g_slist_foreach (entities, foreach_print_entity_to_string, string);
-}
-
 /**
  * @brief Print an XML entity for g_slist_foreach.
  *
@@ -883,18 +858,6 @@ print_entity (FILE * stream, entity_t entity)
   fflush (stream);
 }
 
-/**
- * @brief Print an XML entity tree.
- *
- * @param[in]  stream    The stream to which to print.
- * @param[in]  entities  The entities.
- */
-void
-print_entities (FILE * stream, entities_t entities)
-{
-  g_slist_foreach (entities, foreach_print_entity, stream);
-}
-
 /* "Formatted" (indented) output of entity_t */
 
 /**
@@ -953,21 +916,6 @@ print_entity_format (entity_t entity, gpointer indent)
   printf ("</%s>\n", entity->name);
 }
 
-/**
- * @brief Pretty print XML entities to stdout, recursively printing children.
- *
- * Does very basic indentation for pretty printing.
- *
- * @param[in]  entity  The entity.
- * @param[in]  indent  Indentation level, indentation width is 2 spaces.
- */
-void
-print_entities_format (entities_t entities, int indent)
-{
-  g_slist_foreach (entities, (GFunc) print_entity_format,
-                   GINT_TO_POINTER (indent));
-}
-
 /**
  * @brief Look for a key-value pair in a hash table.
  *
@@ -983,7 +931,7 @@ compare_find_attribute (gpointer key, gpointer value, gpointer attributes2)
   gchar *value2 = g_hash_table_lookup (attributes2, key);
   if (value2 && strcmp (value, value2) == 0)
     return FALSE;
-  g_message ("  compare failed attribute: %s\n", (char *) value);
+  g_debug ("  compare failed attribute: %s\n", (char *) value);
   return TRUE;
 }
 
@@ -1005,13 +953,13 @@ compare_entities (entity_t entity1, entity_t entity2)
 
   if (strcmp (entity1->name, entity2->name))
     {
-      g_message ("  compare failed name: %s vs %s\n", entity1->name,
+      g_debug ("  compare failed name: %s vs %s\n", entity1->name,
                  entity2->name);
       return 1;
     }
   if (strcmp (entity1->text, entity2->text))
     {
-      g_message ("  compare failed text %s vs %s (%s)\n", entity1->text,
+      g_debug ("  compare failed text %s vs %s (%s)\n", entity1->text,
                  entity2->text, entity1->name);
       return 1;
     }
@@ -1029,7 +977,7 @@ compare_entities (entity_t entity1, entity_t entity2)
           (entity1->attributes, compare_find_attribute,
            (gpointer) entity2->attributes))
         {
-          g_message ("  compare failed attributes\n");
+          g_debug ("  compare failed attributes\n");
           return 1;
         }
     }
@@ -1041,7 +989,7 @@ compare_entities (entity_t entity1, entity_t entity2)
     {
       if (compare_entities (list1->data, list2->data))
         {
-          g_message ("  compare failed subentity\n");
+          g_debug ("  compare failed subentity\n");
           return 1;
         }
       list1 = g_slist_next (list1);
@@ -1050,7 +998,7 @@ compare_entities (entity_t entity1, entity_t entity2)
   if (list1 == list2)
     return 0;
   /* More entities in one of the two. */
-  g_message ("  compare failed number of entities (%s)\n", entity1->name);
+  g_debug ("  compare failed number of entities (%s)\n", entity1->name);
   return 1;
 }
 
@@ -1072,3 +1020,25 @@ xml_count_entities (entities_t entities)
     }
   return count;
 }
+
+/**
+ * @brief Append formatted escaped XML to a string.
+ *
+ * @param[in]  xml     XML string.
+ * @param[in]  format  Format string.
+ * @param[in]  ...     Arguments for format string.
+ *
+ * @return Result of XSL transformation.
+ */
+void
+xml_string_append (GString *xml, const char *format, ...)
+{
+  gchar *piece;
+  va_list args;
+
+  va_start (args, format);
+  piece = g_markup_vprintf_escaped (format, args);
+  va_end (args);
+  g_string_append (xml, piece);
+  g_free (piece);
+}
